
// ******************************************************************
//
//	I. Matchikhilian, S. Shuvalov
//
// 	Prototype for ECAL EP/MIP calibration on 4LT FARM
//      =================================================
//
//	10 May 2000 - preliminary version to estimate statistics
//                                         Initial revision;
//
//      15 May 2000 - Optimization of histogram dimensions; 
//                    Disk space is not needed anymore;
//               
//      16 May 2000 - RHP ntuples introducing
//                         
// ******************************************************************


#include <alca/AC_ecal_ifarm.h>


/***
    
    Global variables declaration:

***/


static float                                 ECAL_ihists_hits[N_IHIST];
static short                                 ECAL_ihists_XY[56][42];
static short                                 ECAL_ihists_XYPoolPointers[56][42];
static struct ECAL_ihists_THINGS_TO_KNOW     ECAL_ihistsThingsToKnow;


/*-----------       General functions:   --------------------*/


/***
    
    Should be called once during ARTE initialization:

***/


void AC_ECAL_ihists_initialize()
{
  
  /*-------------------------------------------------------------*/

  int                                        i, k, rc;
  int                                        index;
  int                                        x_mod, y_mod;
  struct utsname                             ECAL_ihistsUtsname;
  
  /*--------------------------------------------------------------*/


  /***
	      
      Initialization:

  ***/

  ECAL_ihistsThingsToKnow.ECAL_ihists_EPlow = ECAL_ihists_EPlow_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_EPup = ECAL_ihists_EPup_default;
	    
  ECAL_ihistsThingsToKnow.ECAL_ihists_MIPlow = ECAL_ihists_MIPlow_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_MIPup = ECAL_ihists_MIPup_default ;
  
  ECAL_ihistsThingsToKnow.ECAL_ihists_et_cut = ECAL_ihists_et_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_pmip_cut = ECAL_ihists_pmip_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_ptrack_cut = ECAL_ihists_ptrack_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_pttrack_cut = ECAL_ihists_pttrack_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_ptrack_low = ECAL_ihists_ptrack_low_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_a_ell = ECAL_ihists_a_ell_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_ncells_betw_2cl = ECAL_ihists_ncells_betw_2cl_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_ncells_betw_tr_cl =ECAL_ihists_ncells_betw_tr_cl_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_isol_cut = ECAL_ihists_isol_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_ztrack_cut = ECAL_ihists_ztrack_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_xmid_cut = ECAL_ihists_xmid_cut_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_zcal = ECAL_ihists_zcal_default;
  ECAL_ihistsThingsToKnow.ECAL_ihists_EP_NTUPLE_ID = ECAL_ihists_NTUPLE_RULE(ECAL_ihists_EP_NTUPNUM);
  ECAL_ihistsThingsToKnow.ECAL_ihists_MIP_NTUPLE_ID = ECAL_ihists_NTUPLE_RULE(ECAL_ihists_MIP_NTUPNUM);


  sprintf ( ECAL_ihistsThingsToKnow.ECAL_ihists_hostname_prefix,
	    "%s",
	    ECAL_ihists_hostname_prefix_default);
 
	 
  uname( & ECAL_ihistsUtsname );

  if( strncmp( ECAL_ihistsUtsname.nodename, ECAL_ihistsThingsToKnow.ECAL_ihists_hostname_prefix, strlen(ECAL_ihistsThingsToKnow.ECAL_ihists_hostname_prefix))){

    ECAL_ihistsThingsToKnow.ECAL_ihists_do = 0;
    return;

  }
  else
    ECAL_ihistsThingsToKnow.ECAL_ihists_do = 1;

  /*------------------------*/

  // RESET ALL:
  
  memset((void *) ECAL_ihists_XY,
	 0,
	 sizeof(ECAL_ihists_XY));
  

  memset((void *) ECAL_ihists_XYPoolPointers,
	 0,
	 sizeof(ECAL_ihists_XYPoolPointers));

  /*------------------------*/
  
  // Initialize geometry:
	  
  
  // 1. Define xmin / ymin of ECAL( center - beam pipe )

  k = 0;

  for( ArteTable<GCAL>::iterator gcal = ArteTable<GCAL>::begin();
       gcal != ArteTable<GCAL>::end();
       gcal++ ) {
    

    if( k == 0 ) {

      k = 1;
      ECAL_ihistsThingsToKnow.ECAL_ihists_xcell = gcal->xmax - gcal->xmin;
      ECAL_ihistsThingsToKnow.ECAL_ihists_ycell = gcal->ymax - gcal->ymin;
      ECAL_ihistsThingsToKnow.ECAL_ihists_xmin = gcal->xmin;
      ECAL_ihistsThingsToKnow.ECAL_ihists_ymin = gcal->ymin;

    }
    else {
      
      if( gcal->xmin < ECAL_ihistsThingsToKnow.ECAL_ihists_xmin )
	ECAL_ihistsThingsToKnow.ECAL_ihists_xmin = gcal->xmin;
      if( gcal->ymin < ECAL_ihistsThingsToKnow.ECAL_ihists_ymin )
	ECAL_ihistsThingsToKnow.ECAL_ihists_ymin = gcal->ymin;

    }
	 

  }

  // 2. Fill ECAL_ihists_XY - different ECAL sections id

  for( ArteTable<GCAL>::iterator gcal = ArteTable<GCAL>::begin();
       gcal != ArteTable<GCAL>::end();
       gcal++ ) {

    x_mod = (int) (( gcal->xmin - ECAL_ihistsThingsToKnow.ECAL_ihists_xmin ) / (gcal->xmax - gcal->xmin ) + 0.5);

    y_mod = (int) (( gcal->ymin - ECAL_ihistsThingsToKnow.ECAL_ihists_ymin ) / (gcal->ymax - gcal->ymin ) + 0.5);

    ECAL_ihists_XY[x_mod][y_mod] = gcal->ncl;
    
  }


  // 3. Fill ECAL_ihists_XYPoolPointers - pointers to histogramms:

  index = 0;
  
  for( k = 0; k < 42; k++ ) 
    for( i = 0; i < 56; i++ ) {
      
      // Skip beam pipes:
      
      if( ECAL_ihists_XY[i][k] == 0 ) 
	continue;

      ECAL_ihists_XYPoolPointers[i][k] = index;
	 
      index += ECAL_ihists_XY[i][k]*ECAL_ihists_XY[i][k];
      
    }

  /*------------------------------------------*/

 
  // RHP ntuples:

  rc = rhp_hbookn(ECAL_ihistsThingsToKnow.ECAL_ihists_EP_NTUPLE_ID,
		  "ECAL_CALIB:EP",
		  ECAL_ihists_NTUPLE_NCOLUMNS,
		  RHP_MAXCOL );

  if( rc != GRR_OK ) {
    cout << "Error in rhp_hbookn EP:" << rc << " " << grr_stript(rc) << endl;
    ECAL_ihistsThingsToKnow.ECAL_ihists_do = 0;
    return;

  }


  rc = rhp_hbookn(ECAL_ihistsThingsToKnow.ECAL_ihists_MIP_NTUPLE_ID,
		  "ECAL_CALIB:MIP",
		  ECAL_ihists_NTUPLE_NCOLUMNS,
		  RHP_MAXCOL );

  if( rc != GRR_OK ) {
    cout << "Error in rhp_hbookn:" << rc << " " << grr_stript(rc) << endl;
    ECAL_ihistsThingsToKnow.ECAL_ihists_do = 0;
    return;

  }

	 
}



/***

    Should be called once per each event:
    
***/


void AC_ECAL_ihists_event()
{

  /*-----------------------------------------------------------------------------------------*/

  int                                        tc_track;
  float                                      my_x = 0;
  float                                      my_y = 0;
  float                                      my_tx = 0;
  float                                      my_ty = 0;
  float                                      my_ze = 0;
  float                                      cur_dist, min_dist;
  float                                      my_e = 0;
  int                                        xmod_ecal, ymod_ecal;
  int                                        xmod_neighb, ymod_neighb;
  int                                        chan, chan_neighb;
  float                                      xcal, ycal;
  short                                      ECAL_ihistnum;
  int                                        i, j, rc;
  int                                        ECAL_ihists_neigb_flag;
  float                                      xneighb, yneighb; 
  static int                                 ECALihist_runnumber = -1;
  int                                        ECALihist_tmp_runnumber;
  static int                                 ECALihist_Nevt = 0;
  static int                                 ECALihist_nwritten = 0;
  float                                      ECALihist_ntup_var[ECAL_ihists_NTUPLE_NCOLUMNS];


  /*----------------------------------------------------------------------------------------*/

  if( ECAL_ihistsThingsToKnow.ECAL_ihists_do == 0 ) 
    return;
    
	   
  ECALihist_tmp_runnumber = faDDAQrun_number();

  if( ECALihist_tmp_runnumber != ECALihist_runnumber ) {
	    

    ECALihist_Nevt = 0;
    ECALihist_nwritten = 0;
    ECALihist_runnumber = ECALihist_tmp_runnumber;
  
    // Reset RHP ntuples:

    rhp_reset(ECAL_ihistsThingsToKnow.ECAL_ihists_EP_NTUPLE_ID);

    rhp_reset(ECAL_ihistsThingsToKnow.ECAL_ihists_MIP_NTUPLE_ID);


  }


  ECALihist_Nevt++;

  
  memset((void *)ECAL_ihists_hits,
	 0,
	 sizeof(ECAL_ihists_hits));

  // 1. Store ECAL hits: 
  
  for( ArteTable<HITC>::iterator hitc = ArteTable<HITC>::begin();
       hitc != ArteTable<HITC>::end();
       hitc++ ) {

    rc = ECAL_ihists_get_neigb( hitc->x,
				hitc->y,
				&xmod_ecal,
				&ymod_ecal,
				&chan);

    if( rc == -1 )
      continue;
	 

    ECAL_ihistnum = ECAL_ihists_XYPoolPointers[xmod_ecal][ymod_ecal] + chan;
    ECAL_ihists_hits[ECAL_ihistnum] = hitc->q;


  }


  // Loop over tracks:


  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end();
       rtra++ ) {
    
    if( fabs(rtra->pf) > ECAL_ihistsThingsToKnow.ECAL_ihists_ptrack_low &&  fabs(rtra->pf) > ECAL_ihistsThingsToKnow.ECAL_ihists_pmip_cut )  
      continue;
  
    tc_track = 0;	    

    for( ArteRelationIterator<RSEG> patt1 = rtra->rseg.begin();
	 patt1 != rtra->rseg.end();
	 patt1++ ) {
      
	        
      if( ((*patt1)->cmp & Rsegc::patt ) && (*patt1)->fit == 0 ) {
	               
	for( ArteRelationIterator<RSEG> patt = (*patt1)->rend.begin();
	     patt != (*patt1)->rend.end();
	     patt++ ) {
	
	  if( ((*patt)->cmp & Rsegc::trit ) && (*patt)->fit == 0 && (*patt)->ze > ECAL_ihistsThingsToKnow.ECAL_ihists_ztrack_cut ) {
	    
	    tc_track = 1;
	    my_x = (*patt)->xe;
	    my_y = (*patt)->ye;
	    my_tx = (*patt)->txe;   
	    my_ty = (*patt)->tye;
	    my_ze = (*patt)->ze;
	    break;

	  }

	}

	if( tc_track != 0 )
	  break;

      }
    }


    if( tc_track == 0)
      continue;

    xcal = my_tx * (ECAL_ihistsThingsToKnow.ECAL_ihists_zcal - my_ze) + my_x;
    ycal = my_ty * (ECAL_ihistsThingsToKnow.ECAL_ihists_zcal - my_ze) + my_y;
    
    if( fabs(rtra->pf) <= ECAL_ihistsThingsToKnow.ECAL_ihists_ptrack_low ) {
      
      if( fabs(rtra->pf) <= ECAL_ihistsThingsToKnow.ECAL_ihists_ptrack_cut || ( fabs(rtra->pf) * ( my_x * my_x + my_y * my_y + my_ze * my_ze ) /( my_x * my_x + my_y * my_y )) <= ECAL_ihistsThingsToKnow.ECAL_ihists_pttrack_cut  ) {
	
	min_dist = -1;
	
	for( ArteTable<RCCL>::iterator hitc = ArteTable<RCCL>::begin();
	     hitc != ArteTable<RCCL>::end();
	     hitc++ ) {
		

	  cur_dist =  hitc->x * hitc->x + hitc->y * hitc->y;
	      
	  if( hitc->esum * sqrt( cur_dist /( cur_dist + hitc->z * hitc->z)) < ECAL_ihistsThingsToKnow.ECAL_ihists_et_cut ) 
	    continue;

	  cur_dist = (hitc->x - xcal) * (hitc->x - xcal ) +  (hitc->y - ycal) * (hitc->y - ycal );
	  
	  if( min_dist == -1 || cur_dist < min_dist ) {
		    
	    min_dist = cur_dist;
	    my_x = hitc->x;
	    my_y = hitc->y;
	    my_e = hitc->esum;
	    min_dist = cur_dist;

	  }


	}

	
	if( min_dist == -1 )
	  continue;

	rc = ECAL_ihists_get_neigb(  my_x,
				     my_y,
				     &xmod_ecal,
				     &ymod_ecal,
				     &chan);

	if( rc == -1 )
	  continue;


	if( (xcal - my_x) * (xcal - my_x) + (ycal - my_y) * (ycal - my_y) / ECAL_ihistsThingsToKnow.ECAL_ihists_a_ell > ECAL_ihistsThingsToKnow.ECAL_ihists_ncells_betw_tr_cl * ECAL_ihistsThingsToKnow.ECAL_ihists_xcell * ECAL_ihistsThingsToKnow.ECAL_ihists_ycell / ( ECAL_ihists_XY[xmod_ecal][ymod_ecal] * ECAL_ihists_XY[xmod_ecal][ymod_ecal] ) )
		
	  continue;


	min_dist = -1;
	 

	for( ArteTable<RCCL>::iterator hitc = ArteTable<RCCL>::begin();
	     hitc != ArteTable<RCCL>::end();
	     hitc++ ) {
	  
	  cur_dist =  hitc->x * hitc->x + hitc->y * hitc->y;
	  
	  if( hitc->esum * sqrt( cur_dist /( cur_dist + hitc->z * hitc->z)) < ECAL_ihistsThingsToKnow.ECAL_ihists_et_cut ) 
	    continue;
		

	  if( hitc->x == my_x && hitc->y == my_y )
	    continue;

	  cur_dist = (hitc->x - my_x) * (hitc->x - my_x ) +  (hitc->y - my_y) * (hitc->y - my_y );
		  
	  if( min_dist == -1 || cur_dist < min_dist ) 
	    min_dist = cur_dist; 

	}


	if( min_dist != -1 && min_dist <  ECAL_ihistsThingsToKnow.ECAL_ihists_ncells_betw_2cl * ECAL_ihistsThingsToKnow.ECAL_ihists_xcell * ECAL_ihistsThingsToKnow.ECAL_ihists_ycell / ( ECAL_ihists_XY[xmod_ecal][ymod_ecal] * ECAL_ihists_XY[xmod_ecal][ymod_ecal] ) )
	  continue;

	
	 ECALihist_ntup_var[1] = my_e * rtra->pf;

	// Fill EP ntuple:

	   
	if( my_x <= ECAL_ihistsThingsToKnow.ECAL_ihists_xmid_cut  )  

	  // Electrons:
	  
	  ECALihist_ntup_var[1]  = -ECALihist_ntup_var[1] ;
	    
	else if( fabs(my_x) < fabs(ECAL_ihistsThingsToKnow.ECAL_ihists_xmid_cut))

	  // Middle row:

	  ECALihist_ntup_var[1]  = fabs(ECALihist_ntup_var[1] );

       
	if(  ECALihist_ntup_var[1]  < ECAL_ihistsThingsToKnow.ECAL_ihists_EPlow || ECALihist_ntup_var[1]  >= ECAL_ihistsThingsToKnow.ECAL_ihists_EPup )
	  continue;
	
	ECALihist_ntup_var[0] = (float) ( 10000 * xmod_ecal + 100 * ymod_ecal + chan );
 
	// Fill rhp ntuples:

	rc = rhp_hfn( ECAL_ihistsThingsToKnow.ECAL_ihists_EP_NTUPLE_ID,
		      ECALihist_ntup_var);

	if( rc != GRR_OK ) 
	  cout << "Error in rhp_hfn EP:" << rc << " " << grr_stript(rc) << endl;

	ECALihist_nwritten++;

      }
    }


    if( fabs(rtra->pf) <= ECAL_ihistsThingsToKnow.ECAL_ihists_pmip_cut) {
	 
      rc = ECAL_ihists_get_neigb(xcal,
				 ycal,
				 &xmod_ecal,
				 &ymod_ecal,
				 &chan);

      if( rc == -1 )
	continue;
	      
      ECALihist_ntup_var[1] = ECAL_ihists_hits[ECAL_ihists_XYPoolPointers[xmod_ecal][ymod_ecal] + chan ];

      if(  ECALihist_ntup_var[1] < ECAL_ihistsThingsToKnow.ECAL_ihists_MIPlow || ECALihist_ntup_var[1] >= ECAL_ihistsThingsToKnow.ECAL_ihists_MIPup )
	continue;


      // Loop over all neighbours cells:



      ECAL_ihists_neigb_flag = 0;
      
      for( i = -1; i < 2; i++ )
	for( j = -1; j < 2; j++ ) {

	  if( j == 0 && i == 0 ) // cell itself
	    continue;
	  
	  xneighb = xcal + 3 * i * ( ECAL_ihistsThingsToKnow.ECAL_ihists_xcell / 10 + ECAL_ihistsThingsToKnow.ECAL_ihists_xcell / (2 * ECAL_ihists_XY[xmod_ecal][ymod_ecal]));

	  yneighb = ycal + 3 * j * ( ECAL_ihistsThingsToKnow.ECAL_ihists_ycell / 10 + ECAL_ihistsThingsToKnow.ECAL_ihists_ycell / (2 * ECAL_ihists_XY[xmod_ecal][ymod_ecal]));


	  rc = ECAL_ihists_get_neigb(xneighb,
				     yneighb,
				     &xmod_neighb,
				     &ymod_neighb,
				     &chan_neighb);

	  if( rc == -1 )
	    continue;

	 
	  if( ECAL_ihists_hits[ECAL_ihists_XYPoolPointers[xmod_neighb][ymod_neighb] + chan_neighb ] > ECAL_ihistsThingsToKnow.ECAL_ihists_isol_cut ) {

	    ECAL_ihists_neigb_flag = 1;
	    break;
		      
	  }
		    

	}
	      
      
      if( ECAL_ihists_neigb_flag == 1 )
	continue;
       
      // Write RHP ntuples:

	ECALihist_ntup_var[0] = (float) ( 10000 * xmod_ecal + 100 * ymod_ecal + chan );
 
	// Fill rhp ntuples:
	
	rc = rhp_hfn( ECAL_ihistsThingsToKnow.ECAL_ihists_MIP_NTUPLE_ID,
		      ECALihist_ntup_var);

	if( rc != GRR_OK ) 
	  cout << "Error in rhp_hfn MIP:" << rc << " " << grr_stript(rc) << endl;




      ECALihist_nwritten++;
      
    }


  }

}


/*--------------------    INTERNAL functions:        -----------------------*/



/***
	    

 ***/


int ECAL_ihists_get_neigb(float xcal,
			  float ycal,
			  int *xmod_ecal,
			  int *ymod_ecal,
			  int *chan_ecal)
{

  /*----------------------------------------------------------------------*/
  
  float                                      low_x, low_y;
  int                                        tmp_x, tmp_y;
  
  /*-----------------------------------------------------------------------*/

  *xmod_ecal = (int)( ( xcal - ECAL_ihistsThingsToKnow.ECAL_ihists_xmin ) / ECAL_ihistsThingsToKnow.ECAL_ihists_xcell );

  *ymod_ecal = (int)( ( ycal - ECAL_ihistsThingsToKnow.ECAL_ihists_ymin ) / ECAL_ihistsThingsToKnow.ECAL_ihists_ycell );

  if(*xmod_ecal < 0 || *xmod_ecal >= 56 || *ymod_ecal < 0 || *ymod_ecal >= 42) 
    return(-1);

  switch(ECAL_ihists_XY[*xmod_ecal][*ymod_ecal]) {

  case(1):
    *chan_ecal = 0;
    break;
  case(2):
  case(5):

    low_x = ECAL_ihistsThingsToKnow.ECAL_ihists_xmin + *xmod_ecal * ECAL_ihistsThingsToKnow.ECAL_ihists_xcell ;
    low_y = *ymod_ecal *  ECAL_ihistsThingsToKnow.ECAL_ihists_ycell + ECAL_ihistsThingsToKnow.ECAL_ihists_ymin;

    tmp_x = (int) (( xcal - low_x ) / (ECAL_ihistsThingsToKnow.ECAL_ihists_xcell / ECAL_ihists_XY[*xmod_ecal][*ymod_ecal]) );

    tmp_y= (int) (( ycal - low_y ) / (ECAL_ihistsThingsToKnow.ECAL_ihists_ycell / ECAL_ihists_XY[*xmod_ecal][*ymod_ecal]) );
		
    if( tmp_x >= ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] || tmp_x < 0 )
      return(-1);
		

    if( tmp_y >= ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] || tmp_y < 0 ) 
      return(-1);

    *chan_ecal = ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] * tmp_y + tmp_x;
	      
    break;
  default:
    return(-1);

  };
	      
  return(0);

}



/*-----------------------------------------------------------------------*/

