/*
*************************************************************************
**
** Description: Fills statistics in MEMA/MEMO for a given table or all tables
**
** Language:	C
**
** Called from: anywhere
**
* Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
** Created: 	28 Oct 98
** Updated:  
**
*************************************************************************
*/

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <arte/ArteFlag.h>
#include <arte/Arte.h>
#include <auto/MEMO.h>
#include <auto/MEMA.h>

#include <arte/mecCount.h>

static void FillTableStatistics(int tab);
static void ArteTableName(int tab, char *tableName);

void FillTableAttrStatistics(int tab);

extern void chhtot_ (int *H, char *T, int *N);

void mmstatistics_(int *tabPtr)

/************************************************************************
**
** PARAMETERS
**	tabPtr	 - table index (0 - all tables)
**
** RETURNS
**	None
**
** FUNCTION DEFINITION
**	Fills statistics in MEMA/MEMO for a given table or all tables
*************************************************************************
*/

{
int tab;

if (*tabPtr)
   {
   FillTableStatistics(*tabPtr);
   }
else
   {
   for (tab=1; tab <= NMEMO(); tab++)
      FillTableStatistics(tab);
   }
}

static void FillTableStatistics(int tab)

/************************************************************************
**
** PARAMETERS
**	tab	 - table index
**
** RETURNS
**	None
**
** FUNCTION DEFINITION
**	Fills statistics in MEMA/MEMO for a given table
*************************************************************************
*/

{
int	*adr;
int	nrow;
double	n;
double	acc;
double	meanOld;
double	meanNew;
double	sig2;



if (artelink_.jlink[tab-1] == 0)
   {
   nrow = 0;
   }
else
   {
   adr = gcbank_.iq + artelink_.jlink[tab-1];
   nrow = *(adr+1);
   }

/*
** Statistics for table (MEMO)
*/

/*  calculate sigma**2  of nrow  */

/*TEMP sig2 = MEMO(tab).sig2; */
/*TEMP*/ sig2 = 0.;
if (MEMO(tab).book > 1)
   {
   acc = MEMO(tab).accrow;
   n   = MEMO(tab).book;
   meanOld = acc/(n-1.);
   meanNew = (acc+nrow)/n;

   sig2 = (  meanNew*meanNew - 
            (nrow*nrow + (n-1.)*(meanOld*meanOld - (n-2.)*sig2))/n
          )/(n-1.);
   }
else
   sig2 = 0.;
   
/*TEMP MEMO(tab).sig2 = sig2; */

MEMO(tab).accrow += nrow;

/*
** Statistics for columns (MEMA) and maxr in (MEMO)
*/

FillTableAttrStatistics(tab);

return;
}

void FillTableAttrStatistics(int tab)

/************************************************************************
**
** PARAMETERS
**	tab	 - table index
**
** RETURNS
**	None
**
** FUNCTION DEFINITION
**	Fills attribute statistics in MEMA for a given table
**	and maxr in (MEMO)
*************************************************************************
*/

{
int	mema;
int	nCol;
int	iCol;
int	col;
int	type;

int	imin;
int	imax;
int	ilow;
int	iupp;
float	fmin;
float	fmax;
float	flow;
float	fupp;

int	*data;
int	*adr;
int	ncolTotal;
int	nrow;
int	irow;
int	itName[namelen];
char	*tName = (char *)itName;

char	errMess[500];

ArteTableName(tab,tName);

if (artelink_.jlink[tab-1] == 0)
   return;
   
adr = gcbank_.iq + artelink_.jlink[tab-1];
nrow = *(adr+1);
data  = adr+2;

MEMO(tab).maxr = MEMO(tab).maxr > nrow ? MEMO(tab).maxr : nrow;

/*
** Statistics for columns (MEMA)
*/

if (MEMO(tab).flag > 0)
   return;
   
mema = MEMO(tab).mema;
nCol = MEMO(tab).col;
ncolTotal = MEMO(tab).col;


for (iCol=0; iCol<nCol; iCol++)
   {
   col = mema+iCol;
   type = MEMA(col).type;

   if (type == 9)
      type = 5;

   if (!(type == 5 || type == 7))
      continue;
   
   switch (type)
      {
      case 5: /* int or signed short */
         {
         int *v;
         
         imin = MEMA(col).imin;
         imax = MEMA(col).imax;
         ilow = MEMA(col).ilow;
         iupp = MEMA(col).iupp;
         
         v = data + iCol;
         for (irow=0; irow<nrow; irow++, v += ncolTotal)
            {
            imin = imin < *v ? imin : *v;
            imax = imax > *v ? imax : *v;
            if (*v < ilow || *v > iupp)
               {
               static int errPtr;
      
               sprintf(errMess, "Out of range (%d , %d): %s col#%d %8.8s row#%d",
                  ilow, iupp, tName,iCol+1,MEMA(col).name,irow+1);
               mecWarningCount(&errPtr, errMess);
               }
            }
         
         MEMA(col).imin = imin;
         MEMA(col).imax = imax;
         
         break;
         }
         
      case 7: /* float */
         {
         float *v;
         
         fmin = MEMA(col).fmin;
         fmax = MEMA(col).fmax;
         flow = MEMA(col).flow;
         fupp = MEMA(col).fupp;
         
         v = (float *)data + iCol;
         for (irow=0; irow<nrow; irow++, v += ncolTotal)
            {
            fmin = fmin < *v ? fmin : *v;
            fmax = fmax > *v ? fmax : *v;
            if (*v < flow || *v > fupp)
               {
               static int errPtr;
      
               sprintf(errMess, "Out of range (%g , %g): %s col#%d %8.8s row#%d",
                  flow,fupp,tName,iCol+1,MEMA(col).name,irow+1);
               mecWarningCount(&errPtr, errMess);
               }
            }

         MEMA(col).fmin = fmin;
         MEMA(col).fmax = fmax;
         
         break;
         }
         
      
      default:;
      }
   }

return;
}

/***********************************************************************/

static void ArteTableName(int tab, char *tableName)

/************************************************************************
**
** PARAMETERS
**	tab	  - table index (data start from iq(jlink(tab)+1) )
**	tableName - (output) table name
**
** RETURNS
**	None
**
** FUNCTION DEFINITION
**	Returns table name as a null terminated string.
*************************************************************************
*/

{
int  len = namelen;

chhtot_ ((int *)MEMO(tab).name, tableName, &len);
}

