//                              -*- Mode: C++ -*-
//: CmdCareECDB.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareECDB.hh"
#include "care/care.hh"
#include "arte/arte.hh"
#include "care/caredata.hh"
#include "arte/ArteMessage.hh"
#include "care/ECHF_status.hh"
#include "care/MMBank.hh"
#include <fstream>
//#include <stdlib.h>
//#include <stdio.h>
#include <ctype.h>
#include <string>
#include <math.h>
#include <time.h>


/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void ecdbase_(){
    CmdCareECDB boardfile;
    boardfile.readAndExe();
    ECalInfo.dbflag = 1;
  }
}

#endif

/////////////// Static data definition

char CmdCareECDB::filename[256] = "\0";

/////////////// CmdCareECDB class member function definition

CmdCareECDB::CmdCareECDB(){
  defValues();
}
CmdCareECDB::~CmdCareECDB(){}

void CmdCareECDB::defValues(){
  strcpy(filename,"ECAL/EcalCalib.db");
  action = none;
  major = 1;
  minor = 10;
  verbose = 0;
}


//--
//   Execute stored command // read or write from the DB
void CmdCareECDB::execute(){
  switch(action){
  case dbread:
    readDB(TALL);
    break;
  case dbwrite:
    writeDB(TALL);
    break;
  default:
    break;
  }
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareECDB::read(){
  // 
  // read ASCII DB file name
  //
  char* name = readString();
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
    filename[i] = name[i];
  filename[i] = '\0';
  //
  // ACTION to do
  //
  int act = readInt();
  action = none;
  if( act==1) action = dbread;
  else if( act==2) action = dbwrite;
  //
  // DB version numbers
  //
  major = readInt();
  minor = readInt();
 
}
#endif

char* CmdCareECDB::calibfile(){
  if( filename[0]==0) 
    defValues();
  return filename;
}

void CmdCareECDB::dump(){
  arteMsg<<" Calibration file name :"<<filename;
  arteMsg.allp();
}

void CmdCareECDB::readDB(Tbset whichtable){

  arteMsg << "Opening "<<calibfile();
  arteMsg.allp();
  // open the DB (the same as ECDBaseIO)
  std::ifstream ifl( calibfile(), std::ios::in );
  if(  !ifl.is_open()){
    std::cerr<<" Error opening the file "<<calibfile()<<std::endl;
  } else {
    switch(whichtable){
    case TALL:
      readECBO(ifl);
      readECHS(ifl);
      readECPD(ifl);
      readECSN(ifl);
      readECHT(ifl);
      makeECHF();
      break;
    case TECBO:
      readECBO(ifl);
      break;
    case TECHS:
      readECHS(ifl);
      break;
    case TPED:
      readECPD(ifl);
      makeECHF();
      break;
    case TSENS:
      readECSN(ifl);
      makeECHF();
      break;
    case THOT:
      readECHT(ifl);
      makeECHF();
      break;
    }
    arteMsg << "Closing "<<calibfile();
    arteMsg.allp();
    ifl.close();
  }
}

void CmdCareECDB::writeDB(Tbset whichtable){

  arteMsg << "Opening "<<calibfile();
  arteMsg.allp();
  // open the DB (the same as ECDBaseIO)
  std::ofstream of( calibfile(), std::ios::out );
  if(  !of.is_open()){
    std::cerr<<" Error opening the file "<<calibfile()<<std::endl;
  } else {
    switch(whichtable){
    case TALL:
      writeECBO(of);
      writeECHS(of);
      writeECPD(of);
      writeECSN(of);
      writeECHT(of);
      break;
    case TECBO:
      writeECBO(of);
      break;
    case TECHS:
      writeECHS(of);
      break;
    case TPED:
      writeECPD(of);
      break;
    case TSENS:
      writeECSN(of);
      break;
    case THOT:
      writeECHT(of);
      break;
    }
    arteMsg << "Closing "<<calibfile();
    arteMsg.allp();
    of.close();
  }
}

void CmdCareECDB::setDB( char *name, int imaj, int imin){
  major = imaj;
  minor = imin;
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
      filename[i] = name[i];
  filename[i] = '\0';
}


void CmdCareECDB::setInfo(int itime1, int itime2, int itime3,
			  const char* iname, int iversion){
  info.firstTime = itime1;
  info.lastTime = itime2;
  info.storageTime = itime3;
  info.version = iversion;
  strncpy(info.name,iname,12);
}


int CmdCareECDB::readECHT(std::ifstream & ifs){
  arteMsg <<"Reading Hot channel list ";
  arteMsg.allp();
  int fedid, chan, gcce,stat;
  char  buffer[256];
  char  c;
  bool cont = true;
  while( !ifs.eof() && ifs.good()  && cont){
    ifs.getline(buffer, 256);
    std::cout<<buffer<<std::endl;
    c = buffer[0];
    if( c=='#' ){
      if( strncmp(buffer,"# END_ECHT",10)==0 ) cont = false;
    } else if ( c=='T' ){
      sscanf(&buffer[1],"%d %d ", &major, &minor);
      std::cout<<"Major,minor:"<<major<<" "<<minor<<std::endl;
    } else if ( c=='D' ){
      sscanf(&buffer[1],"%d %d %d %d %12s", &info.firstTime, &info.lastTime,
	     &info.storageTime,&info.version, info.name);     
    } else if( c=='F' ) { // read couple FED ID, Channel
      sscanf(&buffer[1],"%d %d %d ",&fedid, &chan, &stat);
      gcce = gccenum(fedid, chan);
      if( gcce!=0){
	ECHT hotc;
	hotc.stat = stat;
	hotc.chan = gcce;
	ArtePointer<ECHT>::append(hotc);
	if( verbose>0){
	  std::cout<<" Channel to be switched off (Fed, Ch): "
	      <<fedid<<" "<<chan<<std::endl;
	}
      } else {
	std::cout<<" Channel to be switched off (Fed, Ch): "
	    <<fedid<<" "<<chan<<" WRONG NUMBERS!"<<std::endl; 
      }
    } else if( c=='I' ){ // read index
      sscanf(&buffer[1],"%d %d ", &gcce, &stat);
      if( gcce>0 && gcce<=5956){
	ECHT hotc;
	hotc.stat = stat;
	hotc.chan = gcce;
	ArtePointer<ECHT>::append(hotc);
      	if( verbose>0){
	  std::cout<<" Channel to be switched off (index): "<<gcce<<std::endl;
       	}
      } else {
	std::cout<<" Channel to be switched off (index): "<<gcce
	    <<" WRONG NUMBER!"<<std::endl; 
      }
    }
  }
  return ArtePointer<ECHT>::nrow();
}

int CmdCareECDB::readECSN(std::ifstream & ifs){
  MMBank<ECSN>::Drop();
  ECSN sensb = {0,0.0, 0.0};
  for(int i=0; i<5956; i++)
    ArtePointer<ECSN>::append(sensb);

  arteMsg<<"Reading Sensibility data ";
  arteMsg.allp();
  int fedid, chan, gcce,stat;
  float sens,esens;
  char  buffer[256];
  char  c;
  bool cont = true;
  while( !ifs.eof() && ifs.good() && cont){
    ifs.getline(buffer, 256);
    c = buffer[0];
    if( c=='#' ){
      if( strncmp(buffer,"# END_ECSN",10)==0 ) cont = false;
    } else if ( c=='T' ){
      sscanf(&buffer[1],"%d %d ", &major, &minor);
      std::cout<<"Major,minor:"<<major<<" "<<minor<<std::endl;
    } else if ( c=='D' ){
      sscanf(&buffer[1],"%d %d %d %d %12s", &info.firstTime, &info.lastTime,
	     &info.storageTime,&info.version, info.name);  
    } else if( c=='F' ) { // read couple FED ID, Channel
      sscanf(&buffer[1],"%d %d %d %f %f ",&fedid, &chan, &stat, &sens, &esens);
      gcce = gccenum(fedid, chan);
      if( gcce!=0){
	ArtePointer<ECSN> ecsens(gcce);
	ecsens->stat = stat;
	ecsens->sens = sens;
	ecsens->esen = esens;
	if( verbose>0){
	  std::cout<<" Sensibility for channel (Fed, Ch): "
	      <<fedid<<" "<<chan<<" "<<sens<<"+-"<<esens<<" stat:"
	      <<stat<<std::endl;
	}
      } else {
	std::cout<<" Wrong channel info(Fed, Ch): "
	    <<fedid<<" "<<chan<<" WRONG NUMBERS!"<<std::endl; 
      }
    } else if( c=='I' ){ // read index
      sscanf(&buffer[1],"%d %d %f %f ", &gcce, &stat, &sens, &esens);
      if( gcce>0 && gcce<=5956){
	ArtePointer<ECSN> ecsens(gcce);
	ecsens->stat = stat;
	ecsens->sens = sens;
	ecsens->esen = esens;
	if( verbose>0){
	  std::cout<<" Sensibility for channel : "
	      <<gcce<<" "<<sens<<"+-"<<esens<<" stat:"
	      <<stat<<std::endl;
	}
      } else {
	std::cout<<" Wrong channel (index): "<<gcce<<std::endl; 
      }
    }
  }
  return ArtePointer<ECSN>::nrow();
}


int CmdCareECDB::readECPD(std::ifstream & ifs){
  MMBank<ECPD>::Drop();

  ECPD ped = {0,0.0, 0.0};
  for(int i=0; i<5956; i++)
    ArtePointer<ECPD>::append(ped);

  arteMsg<<"Reading Pedestal data ";
  arteMsg.allp();
  int fedid, chan, gcce,stat;
  float pedf,wid;
  char  buffer[256];
  char  c;
  bool cont = true;
  while( !ifs.eof() && ifs.good() && cont){
    ifs.getline(buffer, 256);
    c = buffer[0];
    if( c=='#' ){
      if( strncmp(buffer,"# END_ECPD",10)==0 ) cont = false;
    } else if ( c=='T' ){
      sscanf(&buffer[1],"%d %d ", &major, &minor);
    } else if ( c=='D' ){
      sscanf(&buffer[1],"%d %d %d %d %12s", &info.firstTime, &info.lastTime,
	     &info.storageTime,&info.version, info.name);
    } else if( c=='F' ) { // read couple FED ID, Channel
      sscanf(&buffer[1],"%d %d %d %f %f",&fedid, &chan, &stat, &pedf, &wid);
      gcce = gccenum(fedid, chan);
      if( gcce!=0){
	ArtePointer<ECPD> ecped(gcce);
	ecped->stat = stat;
	ecped->pedf = pedf;
	ecped->wid = wid;
	if( verbose>0){
	  std::cout<<" Pedestal for channel (Fed, Ch): "
	      <<fedid<<" "<<chan<<" "<<pedf<<"+-"<<wid<<" stat:"
	      <<stat<<std::endl;
	}
      } else {
	std::cout<<" Wrong channel info(Fed, Ch): "
	    <<fedid<<" "<<chan<<" WRONG NUMBERS!"<<std::endl; 
      }
    } else if( c=='I' ){ // read index
      sscanf(&buffer[1],"%d %d %f %f",&gcce, &stat, &pedf, &wid);
      if( gcce>0 && gcce<=5956){
	ArtePointer<ECPD> ecped(gcce);
	ecped->stat = stat;
	ecped->pedf = pedf;
	ecped->wid = wid;
	if( verbose>0){
	  std::cout<<" Pedestal for channel : "
	      <<gcce<<" "<<pedf<<"+-"<<wid<<" stat:"
	      <<stat<<std::endl;
	}
      } else {
	std::cout<<" Wrong channel (index): "<<gcce<<std::endl; 
      }
    }
  }
  return ArtePointer<ECPD>::nrow();
}


int CmdCareECDB::readECHS(std::ifstream & ifs){

  MMBank<ECHS>::Drop();
  ECHS echsb = {0, 0.0, 0.0, 0.0, -1, -1, -1, 
		{0.0, 0.0, 0.0, 0.0, 0.0,},
		{0.0, 0.0, 0.0, 0.0, 0.0,},
		0.0, 0.0};

  for(int i=0; i<5956; i++)
    ArtePointer<ECHS>::append(echsb);

  arteMsg<<"Reading ECHS data ";
  arteMsg.allp();
  int fedid, chan, gcce;
  int mx,my,ix,iy,sz,ec;
  char  buffer[256];
  char  c;
  bool cont = true;
  while( !ifs.eof() && ifs.good() && cont){
    ifs.getline(buffer, 256);
    c = buffer[0];
    if( c=='#' ){
      if( strncmp(buffer,"# END_ECHS",10)==0 ) cont = false;
    } else if ( c=='T' ){
      sscanf(&buffer[1],"%d %d ", &major, &minor);
    } else if ( c=='D' ){
      sscanf(&buffer[1],"%d %d %d %d %12s", &info.firstTime, &info.lastTime,
       &info.storageTime,&info.version, info.name);
    } else if( c=='F' || c=='I' ) { // read couple FED ID, Channel
      if( c=='F'){
	sscanf(&buffer[1],"%d %d %d %d %d %d %d %d",
	       &fedid, &chan, &mx, &my, &ix, &iy, &sz, &ec );
	gcce = gccenum(fedid, chan);
	if( verbose>0){
	  std::cout<<" Slow data for channel (Fed, Ch): "
	      <<fedid<<" "<<chan;
	}
	if( gcce==0 )
	  std::cout<<" Wrong channel info(Fed, Ch): "
	      <<fedid<<" "<<chan<<" WRONG NUMBERS!"<<std::endl; 
      } else {
	sscanf(&buffer[1],"%d %d %d %d %d %d %d",
	       &gcce, &mx, &my, &ix, &iy, &sz, &ec);
	if( verbose>0){
	  std::cout<<" Slow data for channel : "
	      <<gcce;
	}
      }
      if( gcce!=0 && gcce<=5956){
	ArtePointer<ECHS> echs(gcce);
	echs->mod = ((mx&0xFF)<<24)+((my&0xFF)<<16)
	  +((ix&0xF)<<12)+((iy&0xF)<<8)+((sz&0xF)<<4)+(ec&0xF);
	//std::cout<<"CH: "<<gcce<<" "<<mx<<" "<<my<<" "<<echs->mod<<std::endl;
	int necbo;
	ifs>>echs->x>>echs->y>>echs->r;
	ifs>>echs->fdid>>echs->chan>>necbo;
	echs->ecbo = necbo;
	for(int i=0; i<6; i++) ifs>>echs->pcal[i];
	for(int i=0; i<6; i++) ifs>>echs->ecal[i];
	ifs>>echs->peds>>echs->thrs;
      
      } else {
	std::cout<<" Wrong channel info: "<<gcce<<std::endl;
      }
    }
  }
  return ArtePointer<ECHS>::nrow();
}

int CmdCareECDB::readECBO(std::ifstream & ifs){
  MMBank<ECBO>::Drop();
  ECBO ecbob = {0, 0, 0, 0,
		{-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
		 -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1}  };

  for(int i=0; i<226; i++)
    ArtePointer<ECBO>::append(ecbob);

  arteMsg<<"Reading ECBO data ";
  arteMsg.allp();

  int fedid, lin=0;
  char  buffer[256];
  char  c;
  bool cont = true;
  while( !ifs.eof() && ifs.good() && cont){
    ifs.getline(buffer, 256);
    c = buffer[0];
    if( c=='#' ){
      if( strncmp(buffer,"# END_ECBO",10)==0 ) cont = false;
    } else if ( c=='T' ){
      sscanf(&buffer[1],"%d %d ", &major, &minor);
    } else if ( c=='D' ){
      sscanf(&buffer[1],"%d %d %d %d %12s", &info.firstTime, &info.lastTime,
	     &info.storageTime,&info.version, info.name);
    } else if( c=='F' ) { // read couple FED ID, Channel
      int fdid,stat,numb,link;
      sscanf(&buffer[1],"%d %d %d %d %d",&fedid, &fdid,&stat,&numb,&link);
      if(fedid>=16 && fedid<=241 ){
	ArtePointer<ECBO> ecbo(fedid-15);
	ecbo->fdid = fdid;
	ecbo->stat = stat;
	ecbo->numb = numb;
	ecbo->link = link;
	for(int i=0; i<32; i++){
	  ifs>>(ecbo->chan[i]);
	  //	  std::cout<<" "<<(ecbo->chan[i])<<std::endl;
	}
      } else {
	std::cout<<" Wrong board info(Fed, Ch): "
	    <<fedid<<std::endl; 
      }
      if(verbose>0)
	std::cout<<"Board "<<fedid<<" read"<<std::endl;
    }    
    lin++;
  }
  std::cout<<"read "<<lin<<" lines of the file"<<std::endl;
  return ArtePointer<ECBO>::nrow();
}


int CmdCareECDB::writeECHT(std::ofstream & of){
  writeInfo(of,"ECHT");
  arteMsg <<"Writing Hot channel list "<<ArtePointer<ECHT>::nrow();
  arteMsg.allp();
  for(ArteTable<ECHT>::iterator echotc=ArteTable<ECHT>::begin(); 
       echotc!=ArteTable<ECHT>::end(); ++echotc){
    of<<"I "<<echotc->chan<<" "<<echotc->stat<<std::endl;
  }
  of <<"# END_ECHT "<<std::endl;
  return ArtePointer<ECHT>::nrow();
}

int CmdCareECDB::writeECSN(std::ofstream & of){
  writeInfo(of,"ECSN");
  arteMsg <<"Writing Sensibilities list ";
  arteMsg.allp();
  for(ArteTable<ECSN>::iterator echsens=ArteTable<ECSN>::begin(); 
       echsens!=ArteTable<ECSN>::end(); ++echsens){
    of<<"I "<<echsens.index()<<" "
      <<echsens->stat<<" "<<echsens->sens<<" "<<echsens->esen<<std::endl;
  }
  of <<"# END_ECSN "<<std::endl;
  return ArtePointer<ECSN>::nrow();
}

int CmdCareECDB::writeECPD(std::ofstream & of){
  writeInfo(of,"ECPD");
  arteMsg <<"Writing Pedestal list ";
  arteMsg.allp();
  for(ArteTable<ECPD>::iterator echped=ArteTable<ECPD>::begin(); 
      echped!=ArteTable<ECPD>::end(); ++echped){
    of<<"I "<<echped.index()<<" "
      <<echped->stat<<" "<<echped->pedf<<" "<<echped->wid<<std::endl;
  }
  of <<"# END_ECPD "<<std::endl;
  return ArtePointer<ECPD>::nrow();
}


int CmdCareECDB::writeECHS(std::ofstream & of){
  writeInfo(of,"ECHS");
  for(ArteTable<ECHS>::iterator echs=ArteTable<ECHS>::begin(); 
      echs!=ArteTable<ECHS>::end(); ++echs){
    of<<"I "<<echs.index()<<" "
      <<((echs->mod>>24)& 0xFF)<<" "<<((echs->mod>>16)& 0xFF)<<" "
      <<((echs->mod>>12)& 0xF)<<" "<<((echs->mod>>8)& 0xF)<<" "
      <<((echs->mod>>4)& 0xF)<<" "<<((echs->mod)& 0xF)<<std::endl;
    int necbo = (echs->ecbo).index();
    of<<echs->x<<" "<<echs->y<<" "<<echs->r<<std::endl
      <<echs->fdid<<" "<<echs->chan<<" "<<necbo<<std::endl;
    for(int i=0; i<6; i++) of<<echs->pcal[i]<<" ";  
    of<<std::endl;
    for(int i=0; i<6; i++) of<<echs->ecal[i]<<" ";
    of<<std::endl
      <<echs->peds<<" "<<echs->thrs<<std::endl;
  }
  of <<"# END_ECHS "<<std::endl;
  return ArtePointer<ECHS>::nrow();
}

int CmdCareECDB::writeECBO(std::ofstream & of){
  writeInfo(of,"ECBO");
  for(ArteTable<ECBO>::iterator ecbo=ArteTable<ECBO>::begin(); 
      ecbo!=ArteTable<ECBO>::end(); ++ecbo){
    of<<"F "<<ecbo.index()+15<<" "<<ecbo->fdid<<" "<<ecbo->stat<<" "
      <<ecbo->numb<<" "<<ecbo->link<<std::endl;
    for(int i=0; i<32; i++) of<<ecbo->chan[i]<<" ";  
    of<<std::endl;
  }
  of <<"# END_ECBO "<<std::endl;
  return ArtePointer<ECBO>::nrow();
}
 
int CmdCareECDB::gccenum(int fedid, int ch){
 if( fedid<16 || fedid>241 || ch<0 || ch>31){
    std::cout<<" Wrong numbering for: Fed "<<fedid<<"  Channel "<<ch<<std::endl;
    return 0;
  }
  ArtePointer<ECBO> ecbo(fedid-15);
  int gcce = ecbo->chan[ch];
  if( gcce<=0 || gcce>5956) gcce = 0; // error
  return gcce;
}

void CmdCareECDB::writeInfo(std::ofstream & of, const char *tablename){
  arteMsg <<"Writing "<<tablename<<" data ";
  arteMsg.allp();
  of<<"# "<<std::endl
    <<"# "<<tablename<<" bank "<<std::endl
    <<"# "<<std::endl
    <<"T "<<major<<" "<<minor<<std::endl;
  time_t first = ((time_t)info.firstTime);
  time_t last = ((time_t)info.lastTime);
  time_t storage = ((time_t)info.storageTime);
  of<<"D "<<info.firstTime<<" "<<info.lastTime
    <<" "<<info.storageTime<<" "<<info.version<<" "<<info.name<<std::endl;
  of<<"#first time: "<<ctime(&first)
    <<"#last time: "<<ctime(&last)
    <<"#filling time:"<<ctime(&storage)
    <<"# "<<std::endl
    <<"# START_"<<tablename<<std::endl;
}


void  CmdCareECDB::makeECHF(){
  std::cout<<"Now making ECHF with "<<ArtePointer<ECPD>::nrow()<<" entries"<<std::endl;
  std::cout<<"Dropping"<<std::endl;
  MMBank<ECHF>::Drop();
  std::cout<<"Booking "<<std::endl;
  //  MMBank<ECHF>::Book(ArtePointer<ECPD>::nrow());  
  MMBank<ECHF>::Book(5956);
  ArteTable<ECHF>::iterator echf=ArteTable<ECHF>::begin();
  ArteTable<ECPD>::iterator echped=ArteTable<ECPD>::begin();
  ArteTable<ECSN>::iterator echsens=ArteTable<ECSN>::begin();
  for(; echf!=ArteTable<ECHF>::end(); ++echf, ++echped, ++echsens){
    //std::cout<<" Channel "<<echf<<std::endl;
    echf->stat = (echped->stat & 0xFFFF)+ ((echsens->stat & 0xFFFF)<<16);
    if( (echsens->stat & 1)==0 ) echf->stat =echf->stat & 0xFFFFFFFE; 
    echf->pedf = echped->pedf;
    echf->wid = echped->wid;
    echf->sens = echsens->sens;
    echf->esen = echsens->esen;
  }
  int entries = ArtePointer<ECHF>::nrow();
  for(ArteTable<ECHT>::iterator echotc=ArteTable<ECHT>::begin(); 
      echotc!=ArteTable<ECHT>::end(); ++echotc){
    if( echotc->chan>0 && echotc->chan<=entries ){
      ArtePointer<ECHF> echfx(echotc->chan);
      echfx->stat =  echfx->stat & 0xFFFFFFFE;
    }
  }
}
