#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
char *ku_gets(void);

DbTableDesc *DbLedaRelTabDesc(char *tName1, char *tName2, int **pos1, int **pos2)

/************************************************************************
**
** PARAMETERS
**	tName1, tName2	 in    table names
**  pos1, pos2       out   integer arrays containig key positions in tables.
**                         First colunm marked '1', end of array marked '0'
**
** RETURNS
**	Table description for LEDA relation table in MIZZI
**
** FUNCTION DEFINITION
**	Returns  table description to be used in MIZZI for LEDA relations
**           and keys positions in tables to construct relation table.
**
*************************************************************************
*/

{
	int tab;
	static DbTableDesc TD;
	static int arr1[MAX_KEYS_NUMBER], arr2[MAX_KEYS_NUMBER];
	char    errMess[256];
	char    lastName[90],curName[90];
	int	mema;
	int	nCol;
	int	iCol;
	int	col;
	int curpos;
	int	isFirst;
	int	type;
	int     i;
	char	oneDesc[100];
	int	lastType = -1;
	int	nEq;
	int var;


	{ /* Lookup table index */
		int n,nlink,var;
		char mess[80]="ArteTableDesc: ";
		char curName[DB_MAX_NAME_SIZE+1];

		/* Look for bank name in MEMO bank. */

		for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ )
		{
			var = DB_MAX_NAME_SIZE/4;
			chhtot_(MEMO(n).name, curName, &var );
			if( strcmp( tName1, curName ) == 0 )
			{
				nlink = n;
				break;
			}
		}

		if( nlink == 0 )
		{
			static int errPtr;
			sprintf(mess+strlen(mess),
			    "bank %s is not booked in MEMO bank",tName1);
			mecErrorCount( &errPtr, mess );
			return NULL;
		}

		tab=nlink;
	}

	var = DB_MAX_NAME_SIZE/4;
	chhtot_(MEMO(tab).name, TD.Name, &var);
	chhtot_(MEMO(tab).name, TD.Comm, &var);
	TD.Entries=0;

	TD.Fmt[0]=0;

	lastName[0]=0;
	isFirst = 1;

	mema = MEMO(tab).mema;
	nCol = MEMO(tab).ocol;

	nEq = -1;
	curpos=0;

	for (iCol=0; iCol<nCol; iCol++)
	{
		int i;
		col = mema+iCol;
		type = MEMA(col).type;
		strncpy(curName,MEMA(col).name,85);
		for(i=0;i<80;i++)
		{
			if(curName[i]==' ')
			{
				curName[i]=0;
				break;
			}
			curName[i]=toupper(curName[i]);
		}

		if (type == 1 || type == 2 || type == 3 || type == 4)
			type = 1;

		/* check possible types */
		if (type <= 0 || type >= 11)
		{
			static int errPtr;

			sprintf(errMess, "ArteTableDesc: Table %s: unknown type %d in MEMA\n",
			    tName1,type);
			mecWarningCount(&errPtr, errMess);
			return(NULL);
		}

		/* 
		 ** If sequence of equal attributes is empty, save attributes 
		 */
		
		if(nEq==-1)
		{
			if(strncmp(MEMA(col+1).name,MEMA(col).name,80)==0)
				nEq=0;
		}
		else if (strncmp(MEMA(col-1).name,MEMA(col).name,80)!=0)
			nEq=-1;
		else
			nEq++;
		
		
		if(MEMA(col).key==0)
			continue;
		
		arr1[curpos]=iCol+1;
		curpos++;
		
		lastType = type;
		strcpy(lastName,curName);

		
		/* 
		   ** Generate description string
		   */
		strcpy(oneDesc,"");
		if (!isFirst)
			strcat(oneDesc,",");
			else
			isFirst = 0;

		if (nEq != 1)
			sprintf(oneDesc+strlen(oneDesc),"%d",nEq);

		/* generate */
		switch(lastType)

		{
		case 5:	/* int */
			strcpy(oneDesc,"I1");
			break;

		case 6:	/* bit */
			strcpy(oneDesc,"B1");
			break;

		case 7:	/* float */
			strcpy(oneDesc,"F");
			break;

		case 8:	/* holl */
		case 11:
			strcpy(oneDesc,"H");
			break;

		case 9:    /* short */
			strcpy(oneDesc,"S");
			break;
		case 10:    /* ushort */
			strcpy(oneDesc,"U");
			break;

		case 1:
		case 2:
		case 3:
		case 4:	/* relation */
			strcpy(oneDesc,"I1");
			break;
		}

		if (strlen(TD.Fmt)+strlen(oneDesc) > TABLE_DESC_LEN)
		{
			static int errPtr;

			sprintf(errMess, "ArteTableDesc: Table %s: too long description\n",
				tName1);
			mecWarningCount(&errPtr, errMess);
			return(NULL);
		}
		else
		{
			char MizziType[4];
			MizziType[1]=MizziType[2]=0;
			switch(oneDesc[0])
			{
			case 'I': 
				MizziType[0]='I'; 
				break;
			case 'F': 
				MizziType[0]='F'; 
				break;
			case 'B': 
				MizziType[0]='U'; 
				MizziType[1]='I'; 
				break;
			case 'H': 
				MizziType[0]= 'B'; 
				break;
			case 'S': 
				MizziType[0]='S'; 
				break;
			case 'U': 
				MizziType[0]='U'; 
				MizziType[1]='S'; 
				break;
			default:  
				MizziType[0]='?';
			}

			TD.Type[TD.Entries]=oneDesc[0];
			TD.Dim[TD.Entries]=0; /*nEq==1?0:nEq;*/
			TD.Entries++;


			if(nEq==-1 && MizziType[0]=='B')
				sprintf(TD.Fmt+strlen(TD.Fmt),
						" B:%s-0 B:%s-1 B:%s-2 B:%s-3",
						lastName,lastName,lastName,lastName);
			else if(MizziType[0]=='B')
				sprintf(TD.Fmt+strlen(TD.Fmt),
						" B:%s-%d B:%s-%d B:%s-%d B:%s-%d",
						lastName,nEq*4,lastName,nEq*4+1,
						lastName,nEq*4+2,lastName,nEq*4+3);
			else if(nEq==-1)
				sprintf(TD.Fmt+strlen(TD.Fmt)," %s:%s",MizziType,lastName);
			else
				sprintf(TD.Fmt+strlen(TD.Fmt)," %s:%s-%d",
						MizziType,lastName,nEq);
		}

	}

	if(curpos==0)
		*pos1=NULL;
	else
	{
		arr1[curpos]=0;
		*pos1=arr1;
	}
	
/***** All the same for the second table *****/
	
	{ /* Lookup table index */
		int n,nlink,var;
		char mess[80]="ArteTableDesc: ";
		char curName[DB_MAX_NAME_SIZE+1];

		/* Look for bank name in MEMO bank. */

		for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ )
		{
			var = DB_MAX_NAME_SIZE/4;
			chhtot_(MEMO(n).name, curName, &var );
			if( strcmp( tName2, curName ) == 0 )
			{
				nlink = n;
				break;
			}
		}

		if( nlink == 0 )
		{
			static int errPtr;
			sprintf(mess+strlen(mess),
			    "bank %s is not booked in MEMO bank",tName2);
			mecErrorCount( &errPtr, mess );
			return NULL;
		}

		tab=nlink;
	}

	var = DB_MAX_NAME_SIZE/4;

	lastName[0]=0;
	isFirst = 1;

	mema = MEMO(tab).mema;
	nCol = MEMO(tab).ocol;

	nEq = -1;
	curpos=0;

	for (iCol=0; iCol<nCol; iCol++)
	{
		int i;
		col = mema+iCol;
		type = MEMA(col).type;
		strncpy(curName,MEMA(col).name,85);
		for(i=0;i<80;i++)
		{
			if(curName[i]==' ')
			{
				curName[i]=0;
				break;
			}
			curName[i]=toupper(curName[i]);
		}

		if (type == 1 || type == 2 || type == 3 || type == 4)
			type = 1;

		/* check possible types */
		if (type <= 0 || type >= 11)
		{
			static int errPtr;

			sprintf(errMess, "ArteTableDesc: Table %s: unknown type %d in MEMA\n",
			    tName2,type);
			mecWarningCount(&errPtr, errMess);
			return(NULL);
		}

		/* 
		 ** If sequence of equal attributes is empty, save attributes 
		 */
		
		if(nEq==-1)
		{
			if(strncmp(MEMA(col+1).name,MEMA(col).name,80)==0)
				nEq=0;
		}
		else if (strncmp(MEMA(col-1).name,MEMA(col).name,80)!=0)
			nEq=-1;
		else
			nEq++;
		
		
		if(MEMA(col).key==0)
			continue;
		
		arr2[curpos]=iCol+1;
		curpos++;
		
		lastType = type;
		strcpy(lastName,curName);

		
		/* 
		   ** Generate description string
		   */
		strcpy(oneDesc,"");
		if (!isFirst)
			strcat(oneDesc,",");
			else
			isFirst = 0;

		if (nEq != 1)
			sprintf(oneDesc+strlen(oneDesc),"%d",nEq);

		/* generate */
		switch(lastType)

		{
		case 5:	/* int */
			strcpy(oneDesc,"I1");
			break;

		case 6:	/* bit */
			strcpy(oneDesc,"B1");
			break;

		case 7:	/* float */
			strcpy(oneDesc,"F");
			break;

		case 8:	/* holl */
		case 11:
			strcpy(oneDesc,"H");
			break;

		case 9:    /* short */
			strcpy(oneDesc,"S");
			break;
		case 10:    /* ushort */
			strcpy(oneDesc,"U");
			break;

		case 1:
		case 2:
		case 3:
		case 4:	/* relation */
			strcpy(oneDesc,"I1");
			break;
		}

		if (strlen(TD.Fmt)+strlen(oneDesc) > TABLE_DESC_LEN)
		{
			static int errPtr;

			sprintf(errMess, "ArteTableDesc: Table %s: too long description\n",
				tName1);
			mecWarningCount(&errPtr, errMess);
			return(NULL);
		}
		else
		{
			char MizziType[4];
			MizziType[1]=MizziType[2]=0;
			switch(oneDesc[0])
			{
			case 'I': 
				MizziType[0]='I'; 
				break;
			case 'F': 
				MizziType[0]='F'; 
				break;
			case 'B': 
				MizziType[0]='U'; 
				MizziType[1]='I'; 
				break;
			case 'H': 
				MizziType[0]= 'B'; 
				break;
			case 'S': 
				MizziType[0]='S'; 
				break;
			case 'U': 
				MizziType[0]='U'; 
				MizziType[1]='S'; 
				break;
			default:  
				MizziType[0]='?';
			}

			TD.Type[TD.Entries]=oneDesc[0];
			TD.Dim[TD.Entries]=0;
			TD.Entries++;


			if(nEq==-1 && MizziType[0]=='B')
				sprintf(TD.Fmt+strlen(TD.Fmt),
						" B:%s-0 B:%s-1 B:%s-2 B:%s-3",
						lastName,lastName,lastName,lastName);
			else if(MizziType[0]=='B')
				sprintf(TD.Fmt+strlen(TD.Fmt),
						" B:%s-%d B:%s-%d B:%s-%d B:%s-%d",
						lastName,nEq*4,lastName,nEq*4+1,
						lastName,nEq*4+2,lastName,nEq*4+3);
			else if(nEq==-1)
				sprintf(TD.Fmt+strlen(TD.Fmt)," %s:%s",MizziType,lastName);
			else
				sprintf(TD.Fmt+strlen(TD.Fmt)," %s:%s-%d",
						MizziType,lastName,nEq);
		}

	}

	if(curpos==0)
		*pos2=NULL;
	else
	{
		arr2[curpos]=0;
		*pos2=arr2;
	}

	
	for(i=0;TD.Fmt[i];i++)
		if(TD.Fmt[i]=='_')
			TD.Fmt[i]='-';
	
	var=strlen(TD.Fmt);
	for(i=0;i<var;i++)
		TD.Fmt[i]=TD.Fmt[i+1];
	
	return &TD;
}


