/************************************************************************
* This routine translates GEANT output hits into ARTE tables
* HITB and DIGB for OTR modular geometry.
*
* Called from DGEVNT.
*
* Written by S.Nowak, C.Stegmann, A.Lanyov     Oct 1999.
*
* Changes: 
* 11/11/2000: implementation of OtrI into the hit digitization 
*
* Arguments  kind         description
*-----------------------------------------------------------------------
* IPL        in/out       Detector number in GEDE bank
************************************************************************/
#include <vector.h>
#include <math.h>
#include <iostream.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/ZGESL.h"
#include "digi/zgdig.hh"
#include "digi/dglev.h"
#include "digi/dgnoise.h"
#include "digi/dgocell.hh"
#include "Otr/OtrI.h"
#include "Otr/OtrMc.h"

extern "C" {
  void dgomod_(int &);
  void grndm_(float [], const int &);
  void granor_(float &, float &);
  void gpoiss_(float *,int * ,int &);
  void sortzv_(int[], int[], int&, int&, int&, int&);
  int otr_gede_status(int, int, int);
}

inline int OtrEventChannelValid(int ci);


void dgomod_(int & ipl) {

   static int hitprepflag=-1;

   int i0=0, i1=1, mi1=-1; 
   float ydg10 = 2.5, rndm[3];
   vector<digx> *dg_otr[4];
   for(int i=0; i<4; i++)    dg_otr[i] = NULL;


//==================================================================
//  Switch between WAX and OtrI for maskfile treatment    UU 
//==================================================================
   
  OtrMcKillStatistics("Tstart", 0);

  if (hitprepflag==-1) {
    char *f;
    int ci, asd, cha;
    int dead=0, noisy=0;

/*  check which hit preparation is selected for OTR  */
    if ((f=getenv("OtrHitPreparation"))) hitprepflag = atoi(f);
    else hitprepflag=0;
    if (hitprepflag!=1) {
      printf("dgomod> OtrDigitization: OtrI used for dead channel killing.\n");
      if(dglev_.idglev[LGESL_WCH-1] == 3) {
	for (asd=0; asd<OtrIAsdNumberOf(); asd++){
	  for (cha=0; cha<OtrIAsdNumberOfChannel(); cha++) {
	    ci = OtrIFromAsd(asd, cha);
	    if (OtrIIsDead(ci)) dead++; 
	    else if (OtrIIsBad(ci)) noisy++; 
	  }
	} 
	printf("dgomod> OtrDigitization: OtrI Noisy/Dead Channels = %d %d \n\n",
	       noisy, dead);
      } else {
	printf("dgomod> OtrDigitization: --- dead cell killing not selected ---\n");
      }
    } else {
      printf("dgomod> OtrDigitization: WAX used for dead channel killing.\n");
      if(dglev_.idglev[LGESL_WCH-1] != 3) 
	printf("dgomod> OtrDigitization: --- dead cell killing not selected ---\n");
    }
  }
//--------------------------------------------------------------------------
 

   /* In GEAN96 all MIMP's of a module are assigned to the first GEDE of this 
   module. Therefore one has to calculate the number of GEDE's belonging 
   to this module (nsec) */
   ArtePointer<GEDE> gd(ipl);
   unsigned int j=0;
   for(j=ipl+1;;j++) {
       if(j > ArteTable<GEDE>::size()) break; 
       if(ArtePointer<GEDE>(j)->gcop != gd->gcop ) break;  
   }
   int nsec = j - ipl;
   if(nsec > 4) { 
     arteMsg << "dgomod: in GEDE plane "<<ipl<<" there are " <<nsec<<" sectors";
     arteMsg.error();
   }

// define some often used quantitites identical for all sublayers
   zgdig ZGDIG(ipl);
   float rout =  gd->pit/3.;              // cell''s outer radius
   float rin  =  0.8660254*rout;          // cell''s inner radius

// calculate number of mimps 
   int nmim=0, ifmimp=0;
   nmim = ZGDIG.Get_nmimp(ifmimp);

// prepare intermediate storage
   digx otrdg;
   for(int i=0; i<nsec; i++)    dg_otr[i] = new vector<digx>;

/*The x coordinate will be
       transformed into the plane zmin of the given detector and rotated */

// loop over all MIMPs
   for(int ih=0;ih<nmim;ih++) {
      ArtePointer<MIMP> mp(ih+ifmimp);
      float x0[3],  x_rot[3], tx0[3],  tx_rot[3]; 
      x0[0] = mp->x;
      x0[1] = mp->y;
      x0[2] = 0.;  
      ZGDIG.Rotate2plane_Z(x0,x_rot);    //rotate x into wire orientation  
      tx0[0] = mp->tx;   
      tx0[1] = mp->ty;   
      tx0[2] = 0.; 
      ZGDIG.Rotate2plane_Z(tx0,tx_rot); //rotate tx into wire orientation
      float cx_rot = 0.;
      if(fabs(tx_rot[0]) < 1.e+10) cx_rot = 1./sqrt(1.+ tx_rot[0]*tx_rot[0]);

      float ytv = x_rot[1] + tx_rot[1]*(mp->z-gd->zmin); 
      float yth = x_rot[1] + tx_rot[1]*(gd->zmax-mp->z);

//    Loop over all sectors - check into which sector the mimp goes
      for(int j=0; j<nsec; j++) {
         ArtePointer<GEDE> gf(ipl+j);
         zgdig ZGDIG(ipl+j);

         if ( (ytv <= gf->symn || ytv >= gf->symx)    &&
              (yth <= gf->symn || yth >= gf->symx))   continue;
	 for(int itube=0;itube<gf->nsl;itube++){ // loop over monolayers =tubes 
	   if(gf->nwi[itube] <= 0 ) continue;
            float zwir = gf->zfw[itube];
            float y1w = gf->y1w[itube] + 0.5 * ydg10;
            float y2w = gf->y2w[itube] - 0.5 * ydg10;
            float yttv = (zwir - mp->z - rin) * tx_rot[1] + x_rot[1];
            float ytth = (zwir - mp->z + rin) * tx_rot[1] + x_rot[1];
            if (yttv < y1w && ytth < y1w)  break;
            if (yttv > y2w && ytth > y2w)  break;

            float xfw = gf->xfw[itube];
            int n_wire = gf->nwi[itube];
            float xtref = (zwir- mp->z) * tx_rot[0] + x_rot[0];
            float xttv = xtref - rin*tx_rot[0];
            float xtth = xtref + rin*tx_rot[0];

//          first/last cell hidden by the track 
            int nv = int((xttv-(xfw-rout))/gf->pit) + 1;
            int nh = int((xtth-(xfw-rout))/gf->pit) + 1;
	    if(nh<nv) swap(nh,nv) ;
	    if(nv<=0) nv=1;
	    if(nh>n_wire) nh=n_wire ;
	    
//          loop on wires
            for (int n=nv;n<=nh;n++) {          
	      // Check status of wire and ignore the hit if this wire is bad.
	      if(dglev_.idglev[LGESL_WCH-1] == 3) {  // dglev OTR 3
		int status = 0; 
                if (hitprepflag==1) {
		  status = otr_gede_status(ipl+j, itube, n-1);
		} else {
		  int ci = OtrIFromNedeLayerWire(gf.index(), itube, n-1);
		  // int ci = OtrIFromHwcLayerWire(gf->hwc, itube, n-1);
		  // status = OtrIIsDead(ci); 
		  status = !OtrEventChannelValid(ci);
		}

		OtrMcKillStatistics("Add", status);
		if (status) continue;
	      }
	      float xwir = xfw + (float(n)-1.)*gf->pit;
	      float dd = (xtref-xwir) * cx_rot;  // smallest distance to wires
	      // calculate path length in cell
	      float dpath = 
		dgocell::PathLengthHC(gf->pit,xtref-xwir,tx_rot[0],tx_rot[1]) ;
	      
	      if (dpath <= 0.) continue;    //  track does not hit cell
	      if(dglev_.idglev[LGESL_WCH-1] != 0) {    //long version
		float dlen = dgocell::FirstCluster(fabs(dd),gf->eff);
		// if length needed for the first cluster is longer then half
		// the path length in the cell then no hit is produced
		if (dlen > 0.5*dpath) continue;
		// new  distance to wire 
		float ddion = sqrt( dd*dd + dlen*dlen); 
		int knum = itube << 16;
		otrdg.strip = knum + n;                    // strip number
		otrdg.charge= dgocell::MeasuredTime(ddion,gf->acc) ;   // measured time
		otrdg.mimp = mp.index();                   // mimp number
		ArtePointer<MTRA> mt = mp->mtra;
		otrdg.track = mt.index() ;                 // track number
		dg_otr[j]->push_back(otrdg);
		// simulate cross talk
		//otr_crosstalk(itube, n, gf, dg_otr[j], ddion);
	      } else  {  //short version
		grndm_(rndm,i1);
		if(rndm[0] > gf->eff) continue;
		granor_(rndm[0],rndm[1]);
		dd = dd + gd->acc*rndm[0]; 
		if (fabs(dd) > rout) continue;
		if(fabs(dd) > gf->dod) continue; 
		int knum = itube << 16;
		otrdg.strip = knum + n;                    // strip number
		otrdg.charge= dd ;//+ gd->acc*rndm[0];        // distance
		otrdg.mimp = mp.index();                   // mimp number
		ArtePointer<MTRA> mt = mp->mtra;
		otrdg.track = mt.index() ;                 // track number
		dg_otr[j]->push_back(otrdg);
	      }
            }       //======================== end of loop on wires
         }          //======================== end of loop on monolayers
	  
      }             //======================== end of loop on sectors 
   }                //======================== end of loop on mimps

// simulate noise
//    if(dglev_.idglev[LGESL_WCH-1] != 0) {    //long version
//       for (int j = 0; j< nsec; j++) {
//           ArtePointer<GEDE> gf(ipl+j);
//           otr_noise(gf, dg_otr[j]);
//       }
//    }

// preparation for sorting 

   int *dg_srt = NULL; int *ind = NULL; int *ind1 = 0;
   for (int j=0;j< nsec;j++) {  //======================== loop on sectors
      int ndigde = dg_otr[j]->size();
      if (ndigde == 0) {
         zgdig ZGDIG(ipl+j);
         ZGDIG.Set_hit_for_nomimp(); 
         continue;
      }
      ArtePointer<GEDE> gf(ipl+j);
      int max_wire = gf->nwi[0];
      for (int i = 1; i<gf->nsl; i++) {
         if(gf->nwi[i] > max_wire) max_wire = gf->nwi[i];
      }

      dg_srt = new int[ndigde];
      ind    = new int[ndigde];
      ind1   = new int[gf->nsl*max_wire];
      for(int i=0; i<ndigde; i++) {
         vector<digx>::iterator dg = dg_otr[j]->begin() + i;
         dg_srt[i] = dg->strip;
      }
      for (int i=0; i<ndigde; i++) ind[i]  = 0; 
      for (int i=0; i<gf->nsl*max_wire; i++) ind1[i] = 0;

//    sort the  matrix digx  due to increasing tube/wire number
      sortzv_(dg_srt,ind,ndigde,mi1,i0,i0);

//    set digit and hit bank
     int iflag = 0,   iflag1 = 0;
     for(int ih = 0; ih < ndigde; ih++) {

//      first the digits
        vector<digx>::iterator dg = dg_otr[j]->begin() + ind[ih] - 1;
        mmbook_(ArtePointer<DIGB>::getLink().getRef(),1);
        int nn1 =ArteTable<DIGB>::size() ;
        ArtePointer<DIGB> dgb(nn1);
        if(iflag == 0) {        // first digit in this detector
           gf->dig1 = nn1;        iflag = 1;           
        }
        gf->digl = nn1;

        int itube = dg->strip >> 16;
        int iwire = dg->strip&65535;
        dgb->id = 10000*((iwire-1)*4 + itube+1) + ipl + j;
//      id = 111 0068 means strip=28, tube=3, ipl+j=68
        dgb->q  = dg->charge; 
        if (dg->mimp != 0) 
           createRelation(ArtePointer<MIMP>(dg->mimp),dgb,&MIMP::digb);

//      next the hitb bank
        ArtePointer<HITB> htb;
        if(!ind1[itube*max_wire+iwire-1] ) {    //****book a new hitb bank
           mmbook_(ArtePointer<HITB>::getLink().getRef(),1);
           int nn = ArteTable<HITB>::size();
           htb = ArtePointer<HITB>(nn);
           if(!iflag1) {
             gf->hit1 = nn;    iflag1 =1;
           }
           gf->hitl = nn;
           htb->gede = ipl + j;
           htb->x  = gf->xfw[itube] + float(iwire-1) * gf->pit;
           htb->z  = gf->zfw[itube];
	   if(dglev_.idglev[LGESL_WCH-1] != 0) {    //long version
	     htb->y = dg->charge ;
	     // Apply rt-relation:
	     htb->q = dgocell::EstimatedRadius(dg->charge,gf->acc,gf->pit) ;
	     htb->r = dgocell::EstimatedSigma(dg->charge,gf->acc,gf->pit) ;
	   } else {
	     htb->q = dg->charge ;
	     htb->r = gf->acc;
	   }
	   htb->chid = iwire-1 + 32*itube ;
           ind1[itube*max_wire+iwire-1] = nn;
        } else {
	  htb = ArtePointer<HITB>(ind1[itube*max_wire+iwire-1]);
	  if(fabs(dg->charge) < fabs(htb->q))  htb->q = dg->charge;
	}

        if(dg->mimp) 
           createRelation (ArtePointer<MIMP>(dg->mimp),htb,&MIMP::hitb);
	createRelation (htb,dgb,&HITB::digb);
     }         //======================== end of loop on digitized hits

     if(dg_srt != NULL) { delete dg_srt;  dg_srt = NULL;} 
     if(ind    != NULL) { delete ind;     ind = NULL;} 
     if(ind1   != NULL) { delete ind1;    ind1= NULL;}

   }         //======================== end of loop on sectors

   for(int i=0; i<4; i++) { 
       if(dg_otr[i] != NULL) { delete dg_otr[i];  dg_otr[i] = NULL;}
   } 
   ipl += nsec -1;

   OtrMcKillStatistics("Tstop", 0);
}




