/*
========================================================================
=  msgc.h :  definitions for the msgc simulation program               =
=            -------------------------------------------               =
=                                                                      =
=  last update : 05.11.98                                              =
========================================================================
*/
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>                include files                                >>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#include <ctype.h>
#include <float.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>                physical parameter                           >>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#define  taua 7.14
#define  taub 64.3
#define  tauc 5300

#define  u0a 0.07
#define  u0b 0.0056
#define  u0c 0.000026

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>                miscellous                                   >>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#define mxwire     780
#define mx_mimp     80
#define p_length   200 
#define wire_off    10

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>               macro definitions                             >>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#define set_array(a,n1,n2,c)  for(i=n1;i<=n2;i++) a[i]=c
#define round(a)              floor((a)+.5)
#define nint(a)               ((a) -floor(a))<0.5 ? (int) floor(a) : (int) ceil(a)
#define frac(a)               ((a) - (int)(a))
#define max_(a,b)             (((a) < (b)) ? (b) : (a))
#define min_(a,b)             (((a) > (b)) ? (b) : (a))
#define in_(c,a,b)            (((a) > (c)) ? (a) : ((b) < (c)) ? (b) : (c))
#define signum(a)             (((a)==0) ? (a) : ((a) < 0) ? -1 : 1 )

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>             structure definitions                           >>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
struct clster 
{ float x,y,z;
  int   nprim,ntot;
  int   mimp,mtra;
  float gampl;
  float t;
  float diff_x;
};
typedef struct clster Cluster;

struct _hit
{ float center, width, charge; 
  int   w_n0, w_n1;
  vector<int> *hit2mimp;
};
typedef struct _hit Hit;

struct _mcx
{ float x, tgx; 
  int   flg;
};
typedef struct _mcx MCx;

struct _wire
{ int   iwir[mxwire],digno[mxwire];
  float cwir[mxwire],dwir[mxwire];
};
typedef struct _wire Wire;





