// ******************************************************************
//
// 
// M. Braeuer / D. Samtleben
// Filling of histograms for VDS
//
// changes:
// 28 Oct 1999 (AG)  use defines from DQ.h
// 01 Nov 1999 (AG)  get rid of dq_hid.h
// 03 Nov 1999 (AG)  repair hist id
// 10 Dez 1999 (MBr) update/major revision
// 19 Apr 2000 (AG)  polish
// 16 May 2000 (GS)  vertexing added
// 25 May 2000 (GS)  bug fix in filling of residual histograms
// 29 May 2000 (GS)  target spots added
// 27 Jun 2000 (GS)  bug fix in residual calculation
// 28 Jun 2000 (GS)  signal / noise added
// 19 Jul 2000 (GS)  wrong histo id (203) fixed
// 26 Sep 2000 (GS)  bug fix in filling of VDS Hits/plane tracks histogram
// 19 Oct 2000 (GS)  unbiased residuals added
// 08 Nov 2000 (GS)  bug fix in unbiased residual calculation
//  ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "cats/hbook.h"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"

// from Alignment stuff:
inline int fUnsin(int sl,int sec,int lay) { 
  int i = (lay-1)+(4*(sec-1))+(16*(sl-1));
  return(i); 
}

extern int RoverHidden;
extern int RoverNver;
extern int rver_per_wnr[10];
extern int ntr_per_rver[10];

extern "C"{
  void prima_love(void);
}
extern "C"{
  void dinv_(int*, double*, int*, double*, int*);
}

int DQ_SVD_vertex_evnt();
int DQ_SVD_unbiasres_evnt(int, int, float*);
//
// function code
//
int
DQ_SVD_rec_evnt()
{
  //cout << "DQ_SVD_rec_evnt()" << endl;

  int VDShitsontrack=0;      
  int nrSVDtracks=0;
  int nrVDStrackhits=0; // no of track associated hits/event

    /***********************************************************
     *                   TARGET  INFORMATION                   *
     ***********************************************************/
          
  float targetz= 0.;
  int ntargets = 0;
  int usedwire;
        
  for( ArteTable<GTAR>::iterator target = ArteTable<GTAR>::begin();
     target!=ArteTable<GTAR>::end(); ++target ) {
 
     if((target->x*target->x+target->y*target->y)<1.0){
        ntargets++;
        usedwire = target->wnr;
        targetz = targetz + target->z;
      }
  }
           
  if(ntargets!=0){
    if(ntargets>1){
      targetz = targetz/ntargets;
    } 
  }
            
    /***********************************************************
     *               INFORMATION  ABOUT  SEGMENTS              *
     ***********************************************************/

  // loop over all tracks, track=pointer to RSEG information
  for(ArteTableIterator<RSEG> track;track.valid();track.next()){
    float tx=track->txf;  // x-slope at first hit (smallest z)
    float ty=track->tyf;
    
    /* VDS Modified !!================================== */
    // VDS segments only 65537=CATS  1=HOLMES
    if(track->cmp ==1||track->cmp==65537){ 
      VDShitsontrack = 0;

    //cout << "VDShitsontrack" << VDShitsontrack <<endl;

      float VDSxp1,VDSyp1,VDSxp2,VDSyp2,VDSres, VDSunbiasres, VDSocc, VDSusid, VDSnoise;
      VDSxp1 = VDSyp1 = VDSxp2 = VDSyp2 = VDSres = VDSunbiasres = VDSocc = VDSusid = VDSnoise = 0;
      float VDSx = track->xf;
      float VDSy = track->yf;
      float VDSz = track->zf;
            
      // calculate point at target plane: 
      float xx = VDSx - tx*(VDSz-targetz);
      float yy = VDSy - ty*(VDSz-targetz);
         
      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,211),xx,yy,1.);
      rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,212),xx,1.);
      rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,213),yy,1.);   

      int sect = 0;
      int prevsect = 0;
      int first = 0;
	
      // loop over the hits of track      
      int hit=0;	//hit counter
      for ( ArteRelationIterator<HITB> hitbIt = track->hitb.begin();
	    hitbIt != track->hitb.end(); ++hitbIt ) {
	hit += 1;
	ArtePointer<HITB> hitb(hitbIt);     // hitb = pointer to HITB 
	ArtePointer<GEDE> gede(hitb->gede); // gede = pointer to GEDE
	
	// all hits in the same sector?
        if(sect >= 0) {
        sect = gede->sec;
	  if(!first) {		//first hit of track?
	    prevsect = sect;
	    first = 1;
	  }
	  if(sect != prevsect)
	     sect = -1;		//flag that NOT all hits in same sector
	}	

 	// calculate the plane ID:
	VDSusid = float(fUnsin(gede->gesl->nsl,gede->sec,gede->lay) ); 

	++VDShitsontrack;		// increment number of hits per track:
	++nrVDStrackhits;		// increment total number of hits

	//cout << "VDShitsontrack" << VDShitsontrack <<endl;
	
	// book residuals: VDSxp1 = extrapolated track position at 
	// the plane of hit, VDSxp2 taking rotation from alignment into account
	VDSxp1 = VDSx + tx*( (gede->zmax+gede->zmin)/2 - track->zf ); 
	VDSyp1 = VDSy + ty*( (gede->zmax+gede->zmin)/2 - track->zf );
	VDSxp2 = VDSxp1*gede->rot[0] + VDSyp1*gede->rot[3];
	VDSyp2 = VDSxp1*gede->rot[1] + VDSyp1*gede->rot[4];
	VDSres = 10000 * (hitb->x - ((VDSxp2*gede->cos) - (VDSyp2*gede->sin) ));

	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,204),VDSres,1); // to total residual
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,203),VDSusid,VDSres,1); // to residual
	hfill(DQ_ID(COMP_SVD,LVL_REC,209),VDSusid,VDSres,1); // to residual (profile)
	
	double             x, y, txx, tyy,um,u,utrack;
	double             da[16], db[4]; 
	double             proj_tr[4], proj_vv[16], proj_trConf[4];
	double             dIntWork[16];
	int iFail;
	int n,k;

	um = -1.;
	// fill weight matrix, constraint vector, projectors
	for(int i=0; i<16; i++) da[i] = 0.0;
	for(int i=0; i<4;  i++) db[i] = 0.0;
	for(int i=0; i<16; i++) proj_vv[i] = 0.0;
	for(int i=0; i<4;  i++) proj_tr[i] = 0.0;

	//store projector to confirming plane (with stereo and global rotations)
	proj_trConf[0] =  gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
	proj_trConf[1] =  gede->cos*gede->rot[3] - gede->sin*gede->rot[4];
	proj_trConf[2] = (gede->cos*gede->rot[0] - gede->sin*gede->rot[1])*((gede->zmax+gede->zmin)/2);
	proj_trConf[3] = (gede->cos*gede->rot[3] - gede->sin*gede->rot[4])*((gede->zmax+gede->zmin)/2);

	//loop over hits of track and refit track excluding current hit (confirming plane) 
	k = 0;	//hit counter
	for ( ArteRelationIterator<HITB> hitbIter = track->hitb.begin();
            hitbIter != track->hitb.end(); ++hitbIter ) {

	    ArtePointer<HITB> hitbP(hitbIter);		// hitbP = pointer to HITB

	    if (hitbP->gede->cmp == 1) {       //VDS hit?
	      u = hitbP->x;
	      k += 1;

	      // exclude confirming plane from tracking
	      if (hit == k) {
	        um = hitbP->x;	//track coordinate of confirming plan
	        continue;		// exclude
	      }
	      proj_tr[0] =  hitbP->gede->cos*hitbP->gede->rot[0] - hitbP->gede->sin*hitbP->gede->rot[1];
	      proj_tr[1] =  hitbP->gede->cos*hitbP->gede->rot[3] - hitbP->gede->sin*hitbP->gede->rot[4];
	      proj_tr[2] = (hitbP->gede->cos*hitbP->gede->rot[0] - hitbP->gede->sin*hitbP->gede->rot[1])*((hitbP->gede->zmax+hitbP->gede->zmin)/2);
	      proj_tr[3] = (hitbP->gede->cos*hitbP->gede->rot[3] - hitbP->gede->sin*hitbP->gede->rot[4])*((hitbP->gede->zmax+hitbP->gede->zmin)/2);

	      // : vv is vector_v dot Transpose(vector_v) == sym. matrix!
	      for(int i=0; i<4; i++)
	       for(int j=0; j<4; j++)
	        proj_vv[4*i+j] = proj_tr[i]*proj_tr[j];

	      for(int j=0; j<4;  j++) db[j] += proj_tr[j] * u;
	      for(int j=0; j<16; j++) da[j] += proj_vv[j];

	    } // if VDS hit
	} //end of loop over hits

	x = y = txx = tyy = utrack = 0.;
	// get the covariance matrix
	n=4;

	// CALL to Cern-Lib:
	dinv_(&n, da, &n, dIntWork, &iFail);
	if(iFail) {
	  printf("refit inversion failed \n");
	  return(-1); // Do NOT contribute with shit to any reasonable measurement!
	}

	// determine the fit parameters
	for(int i=0; i<4; i++) {
	  x  += da[4*0+i]*db[i];
	  y  += da[4*1+i]*db[i];
	  txx+= da[4*2+i]*db[i]; 
	  tyy+= da[4*3+i]*db[i];
	}

	utrack =	x*(proj_trConf[0])	+	//
			y*(proj_trConf[1])	+	//
			txx*(proj_trConf[2])	+	//
			tyy*(proj_trConf[3])  ;		//

	VDSunbiasres = (um - utrack)*10000;

	//unbiased residuals	
	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,208),VDSunbiasres,1); // to total residual
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,207),VDSusid,VDSunbiasres,1); // to residual
	hfill(DQ_ID(COMP_SVD,LVL_REC,210),VDSusid,VDSunbiasres,1); // to residual (profile)

	// the occupancy:
	VDSocc = (hitb->x - gede->xfw[0]) / gede->pit;
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,202),VDSusid,VDSocc,1); // to occupancy
        rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,201),VDSusid,1);

	// S/N, signal, noise
        VDSnoise = (hitb->q)/(hitb->y);
        if(hitb->q > 0) {    // N-sides
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,220), VDSusid,VDSocc, hitb->y);
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,221), VDSusid,VDSocc, hitb->q);
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,222), VDSusid,VDSocc, VDSnoise);
        }
        else {                // P-sides 
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,220), VDSusid,VDSocc, -hitb->y);
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,221), VDSusid,VDSocc, -hitb->q);
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,222), VDSusid,VDSocc, VDSnoise);
        }

      } // end of loop over hits of track
  

      //fill target spots only if all hits in same sector
      if(sect!=-1) {	
        if(sect==1){
          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,214),xx,yy,1.);   
        }
        else if(sect==2){
          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,215),xx,yy,1.);
        }
        else if(sect==3){
          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,216),xx,yy,1.);
        }
        else if(sect==4){
          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,217),xx,yy,1.);
        }
        else{
          cout << "not used sector: " << sect << endl;
        }
      }			//end of fill target spots

      VDSxp1 = VDSx + (tx * (100 - track->zf));
      VDSyp1 = VDSy + (ty * (100 - track->zf));
      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,205),10*VDSxp1,10*VDSyp1,1); // to refplane
      nrSVDtracks++;

      //cout << "Filling VDShitsontrack" << VDShitsontrack <<endl;

      if ( VDShitsontrack > 0 )
	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,206),VDShitsontrack,1);

    } // if VDS-track (rseg->cmp==1)
    
    /* End of VDS modified ============================= */
  }
  rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,200),nrSVDtracks,1);
  
  /* Do the Vertexing stuff ============================ */
  DQ_SVD_vertex_evnt();

  return 0; // return error code (0 = okay)
}

int
DQ_SVD_vertex_evnt()
{ 

  RoverHidden = 1;
  
  prima_love();

  int nwires = 0;   
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,300), float(RoverNver), 0.0, 1.0);
  //cout<<"DQ rover => "<<RoverNver<<" : ";
  for(int itar=1;itar<=8;itar++){
    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,300+itar), float(rver_per_wnr[itar]), 0.0, 1.0);
    //cout<<rver_per_wnr[itar]<<" ";
    if(rver_per_wnr[itar] != 0){
      nwires++;   
    }
  }
  //cout << "Total wires used: " <<nwires<<endl;
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,310),float(RoverNver),nwires,1.0);
  //cout<<endl;

  //cout<<"Tracks per vertex : ";
  for(int iver=1;iver<=RoverNver;iver++){
    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,309), float(ntr_per_rver[iver]), 0.0, 1.0);
    //cout<<ntr_per_rver[iver]<<" ";
  }
  //cout<<endl;

  RoverHidden = 0;

  return 0; // return error code (0 = okay)
}

