
// ******************************************************************
//
// M. Braeuer/ D. Samtleben
//
// Declaration of Histograms and Initialization
// Histogram ID:     32 bits
//
// changes:
// 28 Oct 1999 (AG) use defines from DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 19 Nov 1999 (AG) show up
// 23 Feb 2000 (MB) Macro bug fixed
// 16 May 2000 (GS) Vertexing histograms added
// 29 May 2000 (GS) Target pictures added
// 30 May 2000 (GS) Profile Hist for residuals added (private use only)
// 28 Jun 2000 (GS) Signal / Noise histograms added
//  6 Jul 2000 (AG) polished printout
// 19 Jul 2000 (GS) Incorrect histo id (203) fixed
//  5 Oct 2000 (GS) Unbiased residual histograms added
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"
#include "cats/hbook.h"

//
// function code
//
int
DQ_SVD_rec_init()
{
#define VDS_MAXPLANES 128
#define VDS_plane_l  -0.5  // lower bound for plane
#define VDS_plane_h 127.5  // upper bound for plane

char title[100];

  // show up
  ARTE_mark( "DQ_SVD_rec_init 0", 0 ); 

  cout << "DQ_VDS_rec_init" << endl;
  
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,200),"VDS tracks/event",
	     200, -0.5, 199.5 );

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,201),"VDS Hits/plane tracks",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,202),"VDS occupancy tracks",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,203),"VDS residuals",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,500,-1490,1490);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,204),"VDS total residual",
	     500,-1490,1490);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,205),"VDS Angular coverage tracks",
	     200,-249.9,249.9, 200,-249.9,249.9);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,206),"VDS hits/track",
	     30,-.5, 29.5);

// Unbiased residuals
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,207),"VDS unbiased residuals",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,1000,-2990,2990);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,208),"VDS total unbiased residual",
	     1000,-2990,2990);

//  for (int i=0; i<128; i++ ) {
//   sprintf(title, "VDS unbias residual Side %d", i);
//    rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,230+i),title,
//	     500,-1490,1490);
//  }

// Profile histogram for residuals 
  hbprof(DQ_ID(COMP_SVD, LVL_REC,209),"VDS residuals",
             VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,-149,149, "");
  hbprof(DQ_ID(COMP_SVD, LVL_REC,210),"VDS unbiased residuals",
             VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,-149,149, "");

// Target spot histograms
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,211),"VDS Target spot all tracks",200,-1., 1., 200, -1.,1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,212),"VDS Target profile in x",200,-1, 1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,213),"VDS Target profile in y",200,-1, 1.);  
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,214),"VDS Sect 1 tracks only",200,-1., 1., 200, -1.,1.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,215),"VDS Sect 2 tracks only",200,-1., 1., 200, -1.,1.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,216),"VDS Sect 3 tracks only",200,-1., 1., 200, -1.,1.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,217),"VDS Sect 4 tracks only",200,-1., 1., 200, -1.,1.);

// Signal / Noise histograms
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,220),"VDS accumulated S/N",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,221),"VDS accumulated Charge",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,222),"VDS accumulated Noise",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);


// Vertexing Histograms
//======================================================================
//      now GEDE-independent booking
//======================================================================

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,300),"VDS vertex/event",20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,301),"VDS vertex/ev wire 1", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,302),"VDS vertex/ev wire 2", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,303),"VDS vertex/ev wire 3", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,304),"VDS vertex/ev wire 4", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,305),"VDS vertex/ev wire 5", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,306),"VDS vertex/ev wire 6", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,307),"VDS vertex/ev wire 7", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,308),"VDS vertex/ev wire 8", 20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,309),"VDS tracks/vertex",    50, -0.5,49.5);

  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,310),"VDS vertex/event vs n.wires",
	     20, -0.5,19.5,9,-0.5,8.5);

  // show up
  ARTE_mark( "DQ_SVD_rec_init 1", 0 ); 

  return 0; // return error code (0 = okay)
}
