
// *****************************************************************************
//
// source:    faRead.C
//
// type:      source file
//
// created:   15 Mar 1996
//
// Author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   
//                  delayed reading (AG: 26-Nov-98)
//                  fill exp number form DAQ record (AG: 26-Nov-98)
//                  max number of slbs changed (AG: 26-Nov-98)
//                  multiple file input (AG: 27-Nov-98)
//                  bx number from DAQ record (AG: 27-Nov-98)
//                  bug when filling event number in  emg mode cured (AG: 11 Dec 98)
//                  removed hbook writing (AG: 14 Dec 99)
//                  obey ANSI rules (AG: 15 Jan 99)
//                  use sltevt exclusively for event numbering (AG: 15 Jan 99)
//                  some more polishing (AG: 21 Jan 99)
//                  avoid memory leak (AG: 27 Jan 99)
//                  farm_node filling (AG: 10 Feb 99)
//                  filling daq time into EVHD (AG: 10 Feb 99)
//                  introduce handler for online event display (AG: 12 Feb 99)
//                  removed Arte include (AG: 23 Feb 99)
//                  removed faUtil.hh (AG: 23 Feb 99)
//                  polishing (AG: 2 Mar 99)
//                  polishing (AG: 11 Mar 99)
//                  polishing (AG: 29 Mar 99)
//                  read event into DEVT (AG: 11 May 99)
//                  fill DB key in EVHD (AG: 26 May 99)
//                  removed dbKey and dbVersion for the time being (AG: 31 May 99)
//                  put includes (AG: 1 Jun 99)
//                  polishing (AG: 3 Jun 99)
// 19 Jul 1999 (AG) enable database key in EVHD
// 09 Sep 1999 (AG) always use cout
// 24 Jan 2000 (AG) introduce cycling without input file
// 27 Jan 2000 (AG) fix end of file problem
// 10 Apr 2000 (AG) use ARTE_error()
// 27 Apr 2000 (AG) use ARTE_mark()
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes
#include "event.h"
#include "etb.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"

#include "farm/fa.hh"
#include "farm/farm.hh"

// C prototpyes
extern "C" void mmbook_( int & , const int & );
extern "C" void utdevt_();

// externals
extern int  CYCLE;   // cycle flag
extern int  FAON;    // farm online flag
extern int  FADEB;   // debug flag
extern char *evtBuf; // event buffer

//
// program code
//
int
faread_()
{
  ARTE_mark( "faread", 0 );

  // inits
  int        rc    = GRR_OK;
  static int count = 0;
  count++;

  // cycling
  faCycle();

  //
  // dummy header table
  //
  if( CYCLE ){
    // build dummy header table
    mmbook_( ArtePointer<EVHD>::getLink().getRef(), 1 );
    ArtePointer<EVHD> evhd( 1 );
    evhd->evt     = arteflag_.numevt = count; // default event number
    evhd->run     = arteflag_.numrun = -1;    // default run number
    evhd->exp     = arteflag_.numexp = -1;    // default exp number
    evhd->date    = faDate( (time_t *) 0 );   // current date
    evhd->time    = faTime( (time_t *) 0 );   // current time
    evhd->vfld    = 0;  // magnetic field
    evhd->flagdst = 0;  // event classification
    evhd->dbmajor = -1; // DB key major number
    evhd->dbminor = -1; // DB key minor number
    evhd->flag    = 0;  // event classification

    static int first = 1;
    if( first ){
      first = 0;
      return ARTE_GOOD;
    }
    else {
      return ARTE_END;
    }
  }

  //
  // read event to buffer
  //

  // read
  if( FAON ){
    // read emg
    if( ( rc = faReadEmg() ) != GRR_OK ){
      return ARTE_END;
    }
  }
  else {
    // read file
    if( ( rc = faReadFile() ) != ETB_ok ){
      return ARTE_END; // try next file
    }
  }
  
  // event header from buffer
  EVENT_HDR evtHdr;
  ESP_get_event_hdr( (EVENT_HDR *) evtBuf, &evtHdr );

  // print
  if( FADEB ){
    cout << endl;
    cout << "faRead(): READ EVENT: *********************************************" << endl;
    cout << "length="            << evtHdr.length          << " "
	 << "recid=0x"    << hex << evtHdr.recid    << dec << " "
	 << "format=0x"   << hex << evtHdr.format   << dec << " "
	 << "sltevt="            << evtHdr.sltevt          << " "
	 << "compmask=0x" << hex << evtHdr.compmask << dec << " "
	 << "evtmask=0x"  << hex << evtHdr.evtmask  << dec << endl;
    cout << "*******************************************************************" << endl;
    cout << endl;
  }

  //
  // EVHD table
  //
  mmbook_( ArtePointer<EVHD>::getLink().getRef(), 1 );
  ArtePointer<EVHD> evhd( 1 );
  evhd->evt     = arteflag_.numevt = evtHdr.sltevt; // event number filled by l4s
  evhd->run     = arteflag_.numrun = -1;            // default run number
  evhd->exp     = arteflag_.numexp = -1;            // default exp number
  evhd->date    = faDate( (time_t *) 0 );           // current date
  evhd->time    = faTime( (time_t *) 0 );           // current time
  evhd->vfld    = 0;  // magnetic field will be filled later ...
  evhd->flagdst = 0;  // event classification ...
  evhd->dbmajor = -1; // DB key major number
  evhd->dbminor = -1; // DB key minor number
  evhd->flag    = 0;  // event classification ...

  // 
  // DIHD table
  //
  mmbook_( ArtePointer<DIHD>::getLink().getRef(), 1 );
  ArtePointer<DIHD> dihd( 1 );
  dihd->bx   = -1; // from EVT_DATA
  dihd->flt  = -1; // from EVT_DATA
  dihd->slt  = evtHdr.sltevt; // event number
  dihd->slb  = -1;  // ???
  dihd->cmsk = evtHdr.compmask; // component mask
  dihd->emsk = evtHdr.evtmask;  // event mask

  // inits
  const int       maxDaq = 1;   // maximal SLB records
  EVT_DATA_RECORD daqs[maxDaq]; // array of SLB records

  // get DAQs
  int m = 0;
  if( ( rc = ETB_get_daqs( (EVENT_HDR *) evtBuf, maxDaq, daqs, &m ) ) == ETB_ok ){
    if( FADEB ){
      cout << "faRead(): ETB_get_daqs() m=" << m << endl;
    }
  }
  else {
    ARTE_error( GRR_SFATAL, "READ: ETB_get_daqs() FAILED.", rc );
    return ARTE_ERR;
  }

  // EVT_DATA
  evhd->run     = daqs[0].data.run_number; // run number
  evhd->exp     = daqs[0].data.exp_number; // exp number
  evhd->date    = faDate( (time_t *) &daqs[0].data.event_time ); // date
  evhd->time    = faTime( (time_t *) &daqs[0].data.event_time ); // time
  evhd->dbmajor = daqs[0].data.dbKey;     // DB key
  evhd->dbminor = daqs[0].data.dbVersion; // DB version
  dihd->bx      = daqs[0].data.physBX; // physical bunch crossing number
  dihd->flt     = daqs[0].data.fltevt; // FLT event number
  dihd->slb     = -1;

  // print
  if( FADEB ){
    cout << "faRead(): EVHD: "
	 << "evhd->evt="     << evhd->evt     << " "
	 << "evhd->run="     << evhd->run     << " "
	 << "evhd->exp="     << evhd->exp     << " "
	 << "evhd->date="    << evhd->date    << " "
	 << "evhd->time="    << evhd->time    << " "
	 << "evhd->dbmajor=" << evhd->dbmajor << " "
	 << "evhd->dbminor=" << evhd->dbminor << endl;
    cout << "faRead(): DIHD: "
	 << "dihd->bx=0x"  << hex << dihd->bx         << " "
	 << "dihd->flt=0x"        << dihd->flt        << " "
	 << "dihd->slb=0x"        << dihd->slb << dec << endl;
  }
  
  // Arte flags
  arteflag_.numrun = evhd->run;
  arteflag_.numexp = evhd->exp;
    
  //
  // print run, event
  //
  if( FADEB ){ 
    faPrintEvtCount( faEVHDrun(), faEVHDevt(), faEVHDexp() );
  }

  //
  // fill event into DEVT
  //

  // build table DEVT
  int nrow = evtHdr.length / sizeof( int ) + 1;           // rows needed
  mmbook_( ArtePointer<DEVT>::getLink().getRef(), nrow ); // book nrow rows
  if( ArteTable<DEVT>::size() == 0 ){
    ARTE_error( GRR_SFATAL, "Creation of DEVT FAILED." , 0 );
    return ARTE_ERR;
  }
  
  // Arte pointer
  ArtePointer<DEVT> devt( 1 );
  
  // put recid
  int recid = int( evtHdr.recid );                          // convert recid
  ESP_put_int( &recid, (int *) &devt->bits, 1 );            // put recid

  // copy data
  memcpy( (int *) &devt->bits + 1, evtBuf, evtHdr.length ); // copy data
  if( FADEB ){
    cout << "faRead(): "
	 << "evtHdr.length=" << evtHdr.length << " "
	 << "devt.nrow()="   << devt.nrow()   << endl;
  }

  //
  // create DNNN tables
  //
  utdevt_();
  
  // return
  return ARTE_GOOD;
}
