/***********************************************************************
* Author :    Siegmund Nowak   07/96
* Description: generate one phase space particle
* Called from: QNEXT
* External references: LUND
***********************************************************************/
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "gean/phaspa.h"

extern "C" {
  void phevnt_();
  void atevhd_();
  void alrini_();
  void mmbook_(int &, const int &);
  void crlosh_(char *,char *);
  void ranecu_(float *,const int &,const int &);
  void ranecq_(int *, int *, const int &, char *, const int);
  void lu1ent_(const int &, int &, float &, float&, float &);
  void alfsto_();
  int crwire_(int *);
  int tpname_(char *, const int);
}

void phevnt_(){

  // event header bank EVHD
  atevhd_();

  // set seed
  if (arteflag_.lundseed[0] == 0 || arteflag_.lundseed[1] == 0)  alrini_();
  ranecq_(&arteflag_.lundseed[0], &arteflag_.lundseed[1], 1, "S",1);

  // book MCIN table
  if(!ArteTable<MCIN>::booked()) 
                             mmbook_(ArtePointer<MCIN>::getLink().getRef(), 1);
  ArtePointer<MCIN> mcn(ArteTable<MCIN>::size()); 

  // set version numbers     
  mcn->vgea = arteflag_.geanvers;
  mcn->vgeo = arteflag_.geomvers;

  // set short/long detector configuration
  crlosh_(&mcn->fmcl[0],&mcn->fmcs[0]);

  // book and fill MCME
  mmbook_(ArtePointer<MCME>::getLink().getRef(), 1);
  ArtePointer<MCME> mcm(ArteTable<MCME>::size());
  mcm->evt = arteflag_.numevt;    mcm->run = arteflag_.numrun;   
  mcm->exp = arteflag_.numexp;

  // further specification necessary in future:
  mcm->phy = 0;         mcm->vlun = 0;

  // lund random seed for this event
  mcm->rlun[0] = arteflag_.lundseed[0];  mcm->rlun[1] = arteflag_.lundseed[1];
  float rx[3];
  ranecu_(rx,3,1);
  float pa = rx[0]*(phaspa_.pmax - phaspa_.pmin) +  phaspa_.pmin;
  float th = rx[1]*(phaspa_.thmax - phaspa_.thmin) + phaspa_.thmin;
  float phi= rx[2]*(phaspa_.phimax - phaspa_.phimin) + phaspa_.phimin;

  // call generator and store event into MTRA/MVER
  ArtePointer<TOPP> tp(tpname_(phaspa_.cpart,12));
  float edky = sqrt(pa*pa + tp->mass*tp->mass);
  int iplund = tp->lund;
  if(tp->deca == 0) lu1ent_(1,iplund,edky,th,phi);
  else              lu1ent_(0,iplund,edky,th,phi);

  alfsto_();

  // define wire
  int nwi = 1;
  int iwire = crwire_(&nwi);
  mcm->wire = iwire;
  ArtePointer<GMAT> gm(ArtePointer<GTAR>(iwire)->cmp);
  if     (!strncmp(gm->name,"ALUM",4)) strncpy(mcm->mat,"AL  ",4);
  else if(!strncmp(gm->name,"CARB",4)) strncpy(mcm->mat,"C   ",4);
  else if(!strncmp(gm->name,"COPP",4)) strncpy(mcm->mat,"CU  ",4);
  else if(!strncmp(gm->name,"TUNG",4)) strncpy(mcm->mat,"W   ",4);
  else if(!strncmp(gm->name,"TITA",4)) strncpy(mcm->mat,"TI  ",4);
  else if(!strncmp(gm->name,"BERY",4)) strncpy(mcm->mat,"BE  ",4);

  // store random geant seed into MCME
  ranecq_(&arteflag_.lundseed[0], &arteflag_.lundseed[1], 1, "R",1);
  mcn->rgea[0] = arteflag_.lundseed[0];
  mcn->rgea[1] = arteflag_.lundseed[1];
}
