/***********************************************************************
* showGeometry
* written by Siegmund Nowak
*
* first step for a better output for description of subdetectors
* in GEANT/Geometry installation
* The SVD output is taken from U.Schwanke's tlt/src/L3_Geometry.c
*
* Arguments  kind         description
*-----------------------------------------------------------------------
* cmp        in           Detector component (LGESL_SVD ..)
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "arte/arte.hh"
#include "gean/ZGESL.h"

void showGeoSVD();
void showGeoTarget();

void showGeometry(const int cmp){
  if( cmp == LGESL_TAR) showGeoTarget();  
  if( cmp == LGESL_SVD)  showGeoSVD();
}   

void showGeoTarget(){
    char bufnam[128],bufmat[128],bufeff[128],bufpos[128];
    int card = 0;

    memset(bufnam,' ',sizeof(bufnam));
    memset(bufmat,' ',sizeof(bufmat));
    memset(bufeff,' ',sizeof(bufeff));
    memset(bufpos,' ',sizeof(bufpos));
    strncpy(bufnam,"TARGET   ",9);
    strncpy(bufmat,"Material ",9);
    strncpy(bufeff,"eff.     ",9);
    strncpy(bufpos,"z-pos.   ",9);

    card = 9;
    for (ArteTable<NTAR>::iterator gs=ArteTable<NTAR>::begin();
                         gs!=ArteTable<NTAR>::end(); ++gs) {
       if ((*gs)->eff != 0.) {
          strncpy(&bufnam[card+2],(*gs)->name,4);
          ArtePointer<GMAT> gm((*gs)->cmp);
          if(!strncmp(gm->name,"CA",2)) strncpy(&bufmat[card+3],"C ",2);  
          else if(!strncmp(gm->name,"TU",2)) strncpy(&bufmat[card+3],"W ",2);  
          else strncpy(&bufmat[card+3],gm->name,2);
          sprintf(&bufeff[card+2],"%.02f",(*gs)->eff);           
          if((*gs)->shap == 1)   sprintf(&bufpos[card+2],"%.02f",(*gs)->z);
          else if((*gs)->shap == 5) {
             float zp=(*gs)->rot[6]*(*gs)->x + (*gs)->rot[7]*(*gs)->y 
                                             + (*gs)->rot[8]*(*gs)->z;
             sprintf(&bufpos[card+2],"%.02f",zp);
          }
          bufeff[strlen(bufeff)] = ' ';
          bufpos[strlen(bufpos)] = ' ';  
          card=card+8;
       }
    }
    bufnam [card-8] = '\0';
    bufmat [card-8] = '\0';
    bufeff [card-8] = '\0';
    bufpos [card-8] = '\0';
    int len = strlen(bufnam); 
    for(int i=0;i<len;i++) cout << "-"; cout << endl;
    cout << "TARGETS"; cout << endl; 
    for(int i=0;i<len;i++) cout << "-"; cout << endl;
    cout << bufnam;                     cout << endl;
    cout << bufmat;                     cout << endl;
    cout << bufeff;                     cout << endl;
    cout << bufpos;                     cout << endl;

    for(int i=0;i<len;i++) cout << "-"; cout << endl;
}


void showGeoSVD(){
    char buf[128];
    char *card;

    memset(buf,' ',sizeof(buf));
    strncpy(buf,"Sector ",7);
    card = &buf[7];
    for (ArteTable<NESL>::iterator gs=ArteTable<NESL>::begin();
                         gs!=ArteTable<NESL>::end(); ++gs) {
       if ((*gs)->cmp== LGESL_SVD) {
	  card = card+2;
          strncpy(card,(*gs)->name,4);
          card=card+7;
       }
    }
    *card ='\0';
    int len = strlen(buf); 
    for(int i=0;i<len;i++) cout << "-"; cout << endl;
    cout << "SVD orientation"; cout << endl; 
    for(int i=0;i<len;i++) cout << "-"; cout << endl;
    cout << buf; 
    cout << endl;

    for(int sec=1;sec<=4;sec++)  {
      memset(buf,' ',sizeof(buf));
      sprintf(&buf[3],"%01i",sec); 
      buf[4] = ' ';
      card = &buf[7]; 

      for (ArteTable<NESL>::iterator gs=ArteTable<NESL>::begin();
                               gs!=ArteTable<NESL>::end();gs++) {
         if((*gs)->cmp == LGESL_SVD) { 
            for(int pl=0;pl<4;pl++){ 
              int found = 0;
	      for (int i = (*gs)->fpl;i<(*gs)->fpl+(*gs)->npl;i++) {
                  ArteTable<NEDE>::iterator gd=ArteTable<NEDE>::begin()-1 + i;
                  if((*gd)->sec == sec) {
                     if((*gd)->lay == pl+1) {
                        *card = (atan((*gd)->sin/(*gd)->cos)>0) ? '+' : '-';
                        *(card+1) = (fabs((*gd)->cos)>0.5) ? 'y' : 'x';
                        found = 1;
                        break;

                     }
                  }
              }
              if(found == 0) {
		*card = '_';
		*(card+1) = '_';
              }                
              card = card + 2; 
           }
           card = card+1;
         }
       }
       *card='\0';
       cout << buf;
       cout << endl;
   }
    
    for(int i=0;i<len;i++) cout << "-"; cout << endl;
}
