#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <vector.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgmed.hh"
#include "gean/hgdat.h"
#include "gean/gclink.h"
#include "gean/gcnum.h"
#include "gean/gccuts.h"
#include "gean/gcphys.h"
#include "gean/gcmzfo.h"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"

extern "C" {
   void gsmate_(int*,char*,float*,float*,float*,float*,float*,float*,int*,int);
   void gsmixt_(int&,char*,float[],float[],float&,int&,float[],int);
   void mmbook_(int &, const int &); 
   void mzbook_(int*, int*, int*, int*, char*, int*, int*, int*, int*, int*,const int);
   void mzpush_(int*, int*, int*, int*, char*,const int);
}

int zgmed::maxmat ;
int zgmed::nmat;

void  zgmed::Set_nmat(int n) {nmat = n;}

void  zgmed::Readmed(char *card, int &nmed) {
      char taga[16]; int flag; int nmix;
      int nitem=sscanf(card,"%s %d %d",taga,&flag,&nmix); 
      if (nitem!=3) {
          arteMsg <<"check data on medium cards" << taga  ;
          arteMsg.error();} 
      fillstr(taga,12);             
      nmed = Check_Gmat(taga);
      if (nmed !=0 ) {
         arteMsg << "material "<<taga<<" exists already - see entry"<< nmed;
         arteMsg.warn();
         if(abs(nmix) > abs(Getncomp(zgmed::nmat))) {
             arteMsg << "number of components in new bigger than in old " 
                   << " define another material name ";
             arteMsg.error();
         } 
      }
      else {
         mmbook_(ArtePointer<GMAT>::getLink().getRef(), 1);
         nmed = ArteTable<GMAT>::size();
      }
      Set_nmat(nmed);
      ArtePointer<GMAT> gmat(nmat); 
      gmat->flag = flag;       
      gmat->nmix  = nmix;
      strncpy(gmat->name,taga,12);
      unsigned int nim = abs(nmix);
      if(nim <= 1) gmat->gmix = 0;
      else gmat->gmix = ArteTable<GMIX>::size() + 1 ; 
}

void  zgmed::Readparam(char *card) {
      char taga[16];
      ArtePointer<GMAT> gmat(nmat); 
      int nitem=sscanf(card,"%f %f %f %f  %f ",&gmat->a,&gmat->z,
                  &gmat->dens, &gmat->radl, &gmat->absl);
      if (nitem!=5) {
         arteMsg <<"check data on medium/parameter cards"  ;
         arteMsg.error();
      } 
      card+=itemlength(card,nitem);
      nitem = sscanf(card," %s ",taga);
      if(nitem== -1 || !strncmp(taga,"!",1) || !strncmp(taga,"//",2)) 
                                         gmat->loss = 0;
      else  nitem = sscanf(card," %f ",&gmat->loss);
}


void zgmed::Readmcomp(char *card){
  char taga[16]; float mix_prop;
  int nitem=sscanf(card,"%s %f ",taga,&mix_prop); 
  fillstr(taga,12);
  if (nitem!=2) {
     arteMsg <<"check data on medium/mcomp cards"  ;
     arteMsg.error();
  }
  Fill_GMIX(taga,mix_prop);  
}

void zgmed::Fill_GMIX(char* name, float &volperc) {
  int nm = Check_Gmat(name);
  if(nm == 0){
    arteMsg << "mixture component" << name 
            << "with volume weights must be defined before";
    arteMsg.error();
  }
  ArtePointer<GMAT> gm(nm);
  GMIX gx;
  strncpy(gx.name,name,12);  
  gx.a = gm->a;      gx.z = gm->z;       gx.w = volperc;
  ArteTable<GMIX>::push_back(gx);
}


void zgmed::Create_Gmed() {
      char gname[20];
      float a_mix[10], z_mix[10], w_mix[10], d_mix;
      ArtePointer<GMAT> gmat(nmat);
      if(gmat->flag == 0) return;

      memset(gname,' ',20);
      strncpy(gname,gmat->name,12);
      gname[12] ='$';
      float dens = gmat->dens; 
      unsigned inmix = abs(gmat->nmix);

      if(inmix == 1) {
        float ubuf; int i0=0;         
        gsmate_(&nmat,gname,&gmat->a,&gmat->z,&dens,&gmat->radl,
                &gmat->absl,&ubuf,&i0,20);
      }
      else {
        float weight = 0.;  float vol = 0.;  
        ArtePointer<GMIX> gm0 = gmat->gmix;
        int istart_mix = gm0.index();
        for (unsigned int i=0; i<inmix; i++)  {
          ArtePointer<GMIX> gmix(istart_mix+i); 
          a_mix[i] = gmix->a;
          z_mix[i] = gmix->z;
          w_mix[i] = gmix->w;
          if (gmat->nmix > 1)  { 
            int nm = Check_Gmat(gmix->name);
            if(nm ==0 ) {
               arteMsg << "mixture component" << gmix->name <<
                   "with volume weights must be defined before";
               arteMsg.error();
            }    
            ArtePointer<GMAT> gm(nm);
            d_mix = gm->dens;
            vol += w_mix[i]; 
            w_mix[i] = w_mix[i]* d_mix;
            weight += w_mix[i];
          }
        }
        if(gmat->nmix>1) {
           for (unsigned int i=0;i<inmix;i++) w_mix[i]=w_mix[i]/weight;
            if(gmat->dens == 0.) {
               dens = weight/vol; gmat->dens = dens;
            }
        }
        gsmixt_(nmat,gname,a_mix,z_mix,dens,gmat->nmix,w_mix,20);
        Set_mixpar();
      }
      crtmed(nmat);  
   }


void  zgmed::Set_mixpar() {
      float aeff, zeff, dens, reff, aleff;
      char taga[13];
      ArtePointer<GMAT> gmat(nmat);
      strncpy(taga,gmat->name,12);
      taga[12] = '\0';
      int jma = *(gcbank_.lq + gclink_.jmate -1 - nmat);
      memcpy(&aeff ,gcbank_.iq + jma + 5,4);
      memcpy(&zeff ,gcbank_.iq + jma + 6,4);
      memcpy(&dens ,gcbank_.iq + jma + 7,4);
      memcpy(&reff ,gcbank_.iq + jma + 8,4);
      memcpy(&aleff,gcbank_.iq + jma + 9,4);
      if((fabs(aeff-gmat->a)/(aeff+gmat->a) > .001 && gmat->a != 0.) ||
         (fabs(zeff-gmat->z)/(zeff+gmat->z) > .001 && gmat->z != 0.) ||
         (fabs(reff-gmat->radl)/(reff+gmat->radl) > .001 
                                                && gmat->radl != 0.) ||
         (fabs(aleff-gmat->absl)/(aleff+gmat->absl) > .001 
                                                && gmat->absl != 0.) ||
         (fabs(dens-gmat->dens)/(dens+gmat->dens) > .001 
                                                && gmat->dens != 0.)) {
         arteMsg <<"for "<< taga <<" new mixture parameters were calculated:";
         arteMsg.warn();
         arteMsg <<"old: " << gmat->a << " " << gmat->z << " " << gmat->dens 
                           << " " << gmat->radl <<  " " << gmat->absl;  
         arteMsg.warn();
         arteMsg <<"new: " << aeff << " " << zeff << " " << dens << " " <<
                              reff << " " << aleff;  
         arteMsg.warn();
      }    
      gmat->a = aeff;
      gmat->z = zeff;
      gmat->radl = reff;
      gmat->absl = aleff;
      gmat->dens = dens; 
}

int   zgmed::Getncomp(int &nr){
      if (nr > nmat) nr = nmat;
      ArtePointer<GMAT> gmat(nmat); 
      return gmat->nmix;
}

int zgmed::Check_Gmat(char *cinp){
    int nm=0;
    for(ArteTableIterator<GMAT> gmat; gmat.valid(); gmat.next()){
       if(!strncmp(gmat->name,cinp,12)){
          nm = gmat.index();
          break;
       }
    }
    return nm;  
}

void zgmed::crtmed(const int &imat) { 
    extern vector<int*>    hg_tmed;
    static int crtmed_nmed ;
    ArtePointer<GMAT> gm(imat);
    if(gm->flag == 0) return;

    int i0=0, i1=1, i3=3, i10=10, i14=14, i40=40;
    int jtm=0, isvol=0, ifield=0;
    float stemax =10.;
    char cname[13];
    float jx;

    unsigned int len = 12;
    if(len >  strlen(gm->name)) len = strlen(gm->name);
    strncpy(cname,gm->name,len);
    for(unsigned int k=0; k<len; k++) {
      if(cname[k] == ' ') { len=k;  break;
      }
    }    
    cname[len] = '\0';

    char sl[4]; char si[1];
    strncpy(sl,"TMED",4);
    strncpy(si,"I",1);

    if(gclink_.jtmed <= 0){
       
       mzbook_(&gcbank_.ixcons,&gclink_.jtmed,&gclink_.jtmed,
                &i1,sl,&gcnum_.ntmed,&gcnum_.ntmed,&i40,&i3,&i0,4);

       memcpy(gcbank_.iq+gclink_.jtmed,&gccuts_.cutgam,40);
       for(int k=0; k< 13; k++) { 
          jx =float(*(&gcphys_.ipair+5*k)); 
          memcpy(gcbank_.iq+gclink_.jtmed+ k+10,&jx,4);
       }  

       *(gcbank_.iq + gclink_.jtmed - 6 ) = 0;
       jx =float(gcphlt_.ilabs); memcpy(gcbank_.iq + gclink_.jtmed +30,&jx,4);
       jx =float(gcphlt_.isync); memcpy(gcbank_.iq + gclink_.jtmed +31,&jx,4);
       jx =float(gcphlt_.istra); memcpy(gcbank_.iq + gclink_.jtmed +32,&jx,4);
       crtmed_nmed = 0;
    }

    int dnew = imat- hg_tmed.size();
    int *ptr;
    if(dnew >0 ){
       for (int k = 0; k< dnew; k++) {
	  ptr = new int[4];
          *ptr = 0;   *(ptr+1) = 0;   *(ptr+2) = 0;   *(ptr+3) = 0; 
          hg_tmed.push_back(ptr);
       }
    }					     
    else  ptr = hg_tmed[imat-1];

    char cnam[20];
    int medmask[4] = {1, 2, 4, 8};

    for(int j=0; j<4; j++) {
       memset(cnam,' ',20);
       int ia = gm->flag & medmask[j];
       if(ia != 0 ) {

         crtmed_nmed = crtmed_nmed + 1;
         *(ptr+j) = crtmed_nmed;

         strcpy(cnam, cname);
         if(j == 0) {
            isvol = 0;
            ifield = 0;
            stemax = 1.e+10;
         }
         else if(j == 1) {
            strncat(cnam,"_magn",5);
            isvol = 0;
            if(zgdetv_.DETV[LGESL_FLD-1] == 'S') ifield = 0;
            else if(zgdetv_.DETV[LGESL_FLD-1] == 'L'  ||
                    zgdetv_.DETV[LGESL_FLD-1] == 'R') ifield = 1;
            stemax = 10.;
         }
         else if(j == 2) {
            isvol = 1;
            ifield = 0;
            stemax = 1.e+10;
         }
         else if (j == 3) {
            strncat(cnam,"_magn",5);
            isvol = 1;
            if(zgdetv_.DETV[LGESL_FLD-1] == 'S') ifield = 0;
            else if(zgdetv_.DETV[LGESL_FLD-1] == 'L'  ||
                    zgdetv_.DETV[LGESL_FLD-1] == 'R') ifield = 1;
            stemax = 10.;
         }
         cnam[strlen(cnam)] = ' ';

         if(crtmed_nmed > gcnum_.ntmed) {
            int dnmed = crtmed_nmed - gcnum_.ntmed; 
            mzpush_(&gcbank_.ixcons,&gclink_.jtmed,&dnmed,&i0,si,1);
            gcnum_.ntmed=crtmed_nmed;
         }
         int mnmed = -crtmed_nmed;
         mzbook_(&gcbank_.ixcons,&jtm,&gclink_.jtmed,&mnmed,sl,&i10,&i10,&i14,
                 &gcmzfo_.iotmed,&i0,4);

         memcpy(gcbank_.iq + jtm,cnam,20);

         jx = float(imat);     memcpy(gcbank_.iq + jtm + 5,&jx,4);
         jx = float(isvol);    memcpy(gcbank_.iq + jtm + 6,&jx,4);
         jx = float(ifield);   memcpy(gcbank_.iq + jtm + 7,&jx,4);
         jx = 10.;             memcpy(gcbank_.iq + jtm + 8,&jx,4);
         jx = 10.;             memcpy(gcbank_.iq + jtm + 9,&jx,4);
         jx = stemax;          memcpy(gcbank_.iq + jtm +10,&jx,4);
         jx = -1.;             memcpy(gcbank_.iq + jtm +11,&jx,4);
         jx = .01;             memcpy(gcbank_.iq + jtm +12,&jx,4);
         jx = -1.;             memcpy(gcbank_.iq + jtm +13,&jx,4);

       }
    }
}











void  zgmed::Fill_hg_tmed() {
      extern vector<int*>    hg_tmed;
      int nmat = ArteTable<GMAT>::size();
      hg_tmed.reserve(nmat);
      for (int k = 0; k< nmat; k++) {
         int *ptr = new int[4];
         *ptr = 0;   *(ptr+1) = 0;   *(ptr+2) = 0;   *(ptr+3) = 0; 
         hg_tmed.push_back(ptr);
      }

      for(int imed =1;imed <= gcnum_.ntmed;imed++) {
        int* pjmd = gcbank_.lq + gclink_.jtmed -1 - imed;
        int *pji = gcbank_.iq + *pjmd - 1;
        float *pjx = (float*)pji;
        int imat = int(*(pjx+6));
        vector<int*>::iterator tm = hg_tmed.begin() + imat -1;
        int isvol = int(*(pjx+7));
        if(isvol>0) isvol = 2;
        int ifield = int(*(pjx+8));
        if(ifield > 0) ifield = 1;
        (*tm)[isvol+ifield] = imed;
      }
}

