#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgeom.h"
#include "gean/gclink.h"

  unsigned int zgeom::ind_pt;
  vector<hgvol> *zgvol::ptvol[33];
  extern vector<int*>    hg_tmed;

extern "C" {
    int mednum_(int&,const char*,int);
    void gsvolu_(char*, char*, int*, float [], int*, int*);
    void gsatt_(char*,char*,int*); 
}


void zgvol::ini(int n){ 
        ptvol[ind_pt] = new vector<hgvol>;
        if(ptvol[ind_pt]==NULL) {
               arteMsg << "no hgvol[n] allocated"; arteMsg.error();
        }
        ptvol[ind_pt]->reserve(n); 
	zguser::hg_user[ind_pt] = NULL;
}

void zgvol::fin() {   
        delete ptvol[ind_pt];
        if(zguser::hg_user[ind_pt] != NULL) {
	  for (unsigned int i=0; i<zguser::hg_user[ind_pt]->size();i++)
                       delete [] *(zguser::hg_user[ind_pt]->begin()+i);
          delete zguser::hg_user[ind_pt];
          zguser::hg_user[ind_pt] = NULL;
        }     
}   


void zgvol::Set_default() {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        memset(vol,0,sizeof(hgvol));
        vol->seen = vol->color = 1; 
}

void zgvol::Copy_previous() {
        if (ptvol[ind_pt]->size() > 1) {
           vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
           vector<hgvol>::iterator old = vol -1;       
           memcpy(vol,old,sizeof(hgvol));
           vol->ncomp = 0;
           if(vol->sens_flag >2 ) { 
              zgdet ZGDET;
              ZGDET.increase();
              ZGDET.Copy_previous();
              ZGDET.Set_vol_det_Rel(); 
           }
        }
        else { 
           arteMsg <<"zgvol::Copy previous: previous does not exist";
           arteMsg.error();
        }
}

void zgvol::Read_vol(char *card,int  &nr) {
        hgvol HGVOL;
        ptvol[ind_pt]->push_back(HGVOL);
        nr = ptvol[ind_pt]->size();
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        char taga[41], tagb[16];
        int nitem=sscanf(card,"%s %s ",taga,tagb);
        if (nitem!=2) {
            arteMsg <<"check data on volume card"; arteMsg.error();
        }
        char *ptr = strstr(tagb,"*");
        if(ptr == NULL) Set_default();    
        else Copy_previous();
        fillstr(taga,40);
        fillstr(tagb,4);
        strncpy(vol->long_name,taga,40);
        strncpy(vol->name,tagb,4);  
}

void  zgvol::Read_shape(char *card) {
        char taga[16];
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        int nitem=sscanf(card,"%s ",taga);
        if(nitem!=1) {arteMsg <<"check data on shape card"; arteMsg.error();}
        fillstr(taga,4);            
        strncpy(vol->shape,taga,4);
        if(!strncmp(taga,"BOX ",4))      vol->npar =3;
        else if(!strncmp(taga,"TUBE",4)) vol->npar =3;
        else if(!strncmp(taga,"ELTU",4)) vol->npar =3;
        else if(!strncmp(taga,"TRD1",4)) vol->npar =4;
        else if(!strncmp(taga,"TRD2",4)) vol->npar =5;
	else if(!strncmp(taga,"TUBS",4)) vol->npar =5;
        else if(!strncmp(taga,"CONE",4)) vol->npar =5;
        else if(!strncmp(taga,"SPHE",4)) vol->npar =6;
        else if(!strncmp(taga,"PARA",4)) vol->npar =6;
        else if(!strncmp(taga,"CONS",4)) vol->npar =7;
        else if(!strncmp(taga,"TRAP",4)) vol->npar =11;
        else { 
            arteMsg <<"shape" << vol->shape << "not foreseen" ;
            arteMsg.error();
        }
}

void  zgvol::Read_dimension(char *card) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        if(vol->npar == 0) {
            arteMsg <<"in volume" << vol->name << "define first the shape" ;
            arteMsg.error();
        } 
        else {
           for (int i=0; i<vol->npar; i++) {  
              int nitem=sscanf(card,"%f",&vol->dimension[i]); 
              if (nitem!=1) { 
                 arteMsg <<"check data on dimension cards";
                 arteMsg.error();
              } 
              card+=itemlength(card,nitem);
           } 
        }
}

void  zgvol::Read_type(char *card) { 
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1; 
        int nitem=sscanf(card,"%d",&vol->type);
        if (nitem!=1) {
            arteMsg <<"check data on type cards";
            arteMsg.error();
        }      
}

void  zgvol::Read_sens_fl(char *card) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        int sflag = vol->sens_flag; 
        int nitem=sscanf(card,"%d",&vol->sens_flag);
        if (nitem!=1) {
            arteMsg <<"check data on sens_flag cards";
            arteMsg.error();}
        if(vol->sens_flag > 2 && sflag == 0) {
          zgdet ZGDET;
          ZGDET.increase();
          ZGDET.Set_default();
          ZGDET.Set_vol_det_Rel();
        } 
}  

void  zgvol::Read_material(char *card) {
        char taga[16];
        int nitem=sscanf(card,"%s",taga);
        if (nitem!=1) {
            arteMsg <<"check data on material cards";
            arteMsg.error();
        }
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        fillstr(taga,12);       
        for (ArteTableIterator<GMAT> gm; gm.valid(); gm.next() ) {
           if(!strncmp(gm->name,taga,12)) {
              vol->imat = gm.index();
              return;
           }   
        }
        arteMsg << "Material not found in GMAT list" << taga;
        arteMsg.error();
        return;      
}

void  zgvol::Read_rel_point(char *card) {
        char taga[16];
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        sscanf(card,"%s",taga);
        for (int m=0;m<3;m++) taga[m] = toupper(taga[m]);
        if(!strncmp(taga,"DET",3)) {
          for(unsigned int isl=1;isl<=ArteTable<NESL>::size();isl++) {
               ArtePointer<NESL> gs(isl);
               if(!strncmp(gs->name,vol->name,4)) {
                  vol->rel_px = -0.5*(gs->x1 + gs->x2);
                  vol->rel_py = -0.5*(gs->y1 + gs->y2);
                  vol->rel_pz = -0.5*(gs->z1 + gs->z2);
                  return;
               }
          }
        }
        else {  
           int nitem=sscanf(card,"%f %f %f",
                                &vol->rel_px,&vol->rel_py,&vol->rel_pz); 
           if (nitem!=3) {
               arteMsg <<"check data on rel_point cards";
               arteMsg.error();
           }
        }
}

void  zgvol::Read_color(char *card) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        int nitem=sscanf(card,"%d",&vol->color);
        if (nitem!=1) {
            arteMsg <<"check data on color cards";
            arteMsg.error();
        }      
        if(vol->color!=0) vol->seen=1;              
} 

void  zgvol::Read_User(char *card) { 
        if(zguser::hg_user[ind_pt] == NULL) 
	zguser::hg_user[ind_pt] = new vector<char *>;
        zguser ZGUSER;
        int lfd = ZGUSER.increase(card);
        vector<hgvol>::iterator vol = ptvol[ind_pt]->end() -1;
        if(!vol->index_hguser) vol->index_hguser = lfd;
        vol->nuser ++;
}

char* zgvol::Get_volname(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->name; 
}

char*  zgvol::Get_fullname(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->long_name; 
}

int   zgvol::Get_volnr(char* volname,int voltype) {
        for(unsigned int n=0; n< ptvol[ind_pt]->size(); n++) {
           vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + n;
           if(!strncmp(vol->name,volname,4) && vol->type==voltype) 
                                                         return n+1;
        }            
        return 0;
} 

char* zgvol::Get_shape(int &nr)    { 
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->shape;}

int  zgvol::Get_dimension(int &nr,float dim[]) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
        return vol->npar;
}

int  zgvol::Get_par(int &nr,float dim[]) {
       vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
       if(!strncmp(vol->shape,"BOX ",4) || !strncmp(vol->shape,"TRD1",4) ||
          !strncmp(vol->shape,"TRD2",4)) { 
            for (int i=0; i<vol->npar; i++) dim[i] = 0.5*vol->dimension[i];
        }
       else if(!strncmp(vol->shape,"TRAP",4)) {
          for (int i=0; i<vol->npar; i++) dim[i] = 0.5*vol->dimension[i];
          dim[1] = vol->dimension[1];
          dim[2] = vol->dimension[2];
          dim[6] = vol->dimension[6];
          dim[10] = vol->dimension[10]; 
        }
        else if(!strncmp(vol->shape,"TUBE",4) || 
                !strncmp(vol->shape,"TUBS",4) ||
                !strncmp(vol->shape,"ELTU",4))   { 
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
          dim[2] = 0.5 * vol->dimension[2];
       }
       else if(!strncmp(vol->shape,"CONE",4) || !strncmp(vol->shape,"CONS",4)){
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
          dim[0] = 0.5*dim[0]; 
       }     
       else if(!strncmp(vol->shape,"SPHE",4)) { 
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
       }
       else if(!strncmp(vol->shape,"PARA",4)) {  
          for (int i=0; i< 3; i++) dim[i] = 0.5*vol->dimension[i];
          for (int i=3; i<vol->npar; i++) dim[i] = vol->dimension[i];
       }
       return vol->npar;
}

int  zgvol::Get_type(int &nr) {  
       vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
       return vol->type;   
}

int   zgvol::Get_sens_flag(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->sens_flag;
}
     
int zgvol::Get_imat(int &nr) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->imat;
}

int  zgvol::Get_imed(int &nr) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        vector<int*>::iterator tm = hg_tmed.begin() + vol->imat -1;
        return (*tm)[vol->sens_flag-1];        
}

void  zgvol::Get_rel_point(int &nr,float *xp, float *yp, float *zp) {
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        *xp = vol->rel_px;
        *yp = vol->rel_py;
        *zp = vol->rel_pz;
}

int   zgvol::Get_color(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->color;
}

int   zgvol::Get_ncomp(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->ncomp;
}

int   zgvol::Get_nuser(int &nr) {  
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + nr -1;
        return vol->nuser;
}

void  zgvol::Create_Gvol(int n) {
        int ivol = 0, zero=0;    float par[11];
        vector<hgvol>::iterator vol = ptvol[ind_pt]->begin() + n -1;
	//        char colo[5];
	//        strcpy(colo,"COLO"); 
        vector<int*>::iterator tm = hg_tmed.begin() + vol->imat -1;  
        int imed = (*tm)[vol->sens_flag-1];
	gsvolu_(vol->name,vol->shape, &imed, par, &zero, &ivol);
        gsatt_(vol->name,"COLO", &vol->color);  
}


int   zgvol::Check_Gvol(char *volname) {
//      check whether the volume already exists
        int nvol=*(gcbank_.iq + gclink_.jvolum -2);
        for(int i=0; i<nvol; i++) {
          if(!memcmp((gcbank_.iq+gclink_.jvolum+i),volname,4)) return i+1;
        }
        return 0;  
}
  







