#include <iostream>
#include "grover/DumpFormatter.hh"
#include "grover/grover.hh"

///////////////////////////////////////////////////////////////////////
//  GroverCuts class definition
//  ---------------------------
//
//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// --------------------------------------------------------------------------
// 00-10-01  M.Sang   Secondary finders not called by default
// 00-10-11  M.Sang   3D display cuts added
// 00-11-08  M.Sang   added a few more cuts (GroverEvent() steering etc)


GroverCuts::GroverCuts(void) // Default constructor
  : VertexingPackage(1), GroverDraw(0),
    FindJpsiPlusX(0), FindUnbiased(0), FindCharm(0),
    Verbose(0),       GPEActivate(0), Paw(0),          Root(0),         
    MassConstraint(0.0),
    CheatMass(0),     CheatMomentum(0),                CheatSmearing(0),
    CheatSmearConst(0.03), CheatSmearVar(0.0),
    VTmPsi(3.09693),  VTmKs(0.4977),  VTmB(5.2794), 
    VTmpi(0.1396),    VTmmu(0.1057),  VTme(0.000511),
    VTch2wtr(10.),    VTch2wB(20.),   VTxlmin(0.015),  VTxlwid(0.050),
    VTntmin(3),       VTch2vtr(14.),  VTch2stt(6.),    VTch2stv(8.),
    VTch2svw(1.),     VTBvimpc(0.1),  VTnmix(1),       VTch2vv(6.),
    VTtweak(999),     VTDraw(0),      RPTrackChoice(1),
    RPTrackQualNH(5),
    RPMinTrksInVtx(3),RPTrkWireSigmaMax(3.5),
    GPEUpdateFreq(100),               GPEMinHitsInTrack(5), 
    GPEMinTracksInVertex(3),          GPETransverseWindow(0.03), 
    GPELongitudinalWindow(0.2),       GPEChiWindow(6.0),
    RSTrackQualNH(7),
    RSJpsiSeedMaxChi2(10.0),          RSJpsiSeedMaxDist(0.05),
    //    RSVtxVtxMinChi2(3.0),
    //    RSTrkVtxMaxChi2(5.0),             RSTrkVtxMaxDist(0.2),
    //RSTrkVtxDistRatio(1.0/3.0)
    RSMinSeedIPS(5.0),               RSMaxChi2Contrib(3.0), 
    RSMaxTrkVtxChi2(10.0),            RSMaxTrkTrkChi2(5.0),
    RSMax2TVtxChi2(5.0),             RSMaxChi2PDOF(3.0),
    GroverDraw3D(0),
    TDX3D(1),
    TDTargetRegion(1),
    TDAxes(0),
    TDWires(1),
    TDPrimaries(1),
    TDSecondaries(1),
    TDMCPrimaries(1),
    TDMCSecondaries(1),
    TDPrimaryTracks(0),
    TDSecondaryTracks(0),
    TDUnusedTracks(0),
    TDSpecialTracks(0),
    TDWireColour(5),
    TDPrimaryColour(2),
    TDSecondaryColour(4),
    TDMCPrimaryColour(3),
    TDMCSecondaryColour(6),
    TDPrimaryTrackColour(15),
    TDSecondaryTrackColour(7),
    TDUnusedTrackColour(28),
    TDSpecialTrackColour(41),
    TDTrackSmoothness(8),
    TDPrimaryTrackLength(1.0),
    TDSecondaryTrackLength(1.0),
    TDUnusedTrackLength(10.0),
    TDSpecialTrackLength(10.0),
    TDXAxisLength(3.0),
    TDYAxisLength(3.0),
    TDZAxisLength(15.0),
    TDAxisRadius(0.01),
    TDTrackMinP(0.0),
    TDTrackMinPt(0.0),
    TDTrackMaxChi2(1000000.0),
    TDTrackMinNh(0),
    GETrackChoice(1),
    GERefillTracks(0),
    GEFindPrimaries(1),
    GERedoPrimaries(1),
    GEFindSecondaries(0),
    GERedoSecondaries(0),
    GERefitRSEGTable(0),
    GEWriteRTAR(0),
    GEWriteRVER(1),
    GPWireFollowing(0),
    GPNewWires(0)
{
  // If possible, you should update the particle masses (VTmXX) from TOPP
  // to avoid nasty surprises which can mess up your mass-constrained fits!
}

GroverCuts::~GroverCuts(void)
{ // Default destructor
}

void GroverCuts::Dump(void) { // Dump members to stdout
  cout << "*** Grover: Dump() of GroverCuts object" << endl;
  DumpFormatter Nice;
  Nice.DumpClassName("GroverCuts");
  Nice.DumpComment("Part of the Grover vertexing package");
  Nice.DumpComment("Please send questions about this class "
		    "to the Grover authors");
  Nice.DumpHeader();
  Nice.DumpPrint("VertPackage",&VertexingPackage);
  Nice.DumpPrint("GroverDraw",&GroverDraw);
  Nice.DumpPrint("FindJpsiPlusX",&FindJpsiPlusX);
  Nice.DumpPrint("FindUnbiased",&FindUnbiased);
  Nice.DumpPrint("FindCharm",&FindCharm);
  Nice.DumpPrint("Verbose",&Verbose);
  Nice.DumpPrint("GPEActivate",&GPEActivate);
  Nice.DumpPrint("Paw",&Paw);
  Nice.DumpPrint("Root",&Root);
  Nice.DumpPrint("MassConstraint",&MassConstraint);
  Nice.DumpPrint("CheatMass",&CheatMass);
  Nice.DumpPrint("CheatMomentum",&CheatMomentum);
  Nice.DumpPrint("CheatSmearing",&CheatSmearing);
  Nice.DumpPrint("CheatSmearConst",&CheatSmearConst);
  Nice.DumpPrint("CheatSmearVar",&CheatSmearVar);
  /*
    Nice.DumpPrint("VTch2wtr",&VTch2wtr);
    Nice.DumpPrint("VTch2wB",&VTch2wB);
    Nice.DumpPrint("VTxlmin",&VTxlmin);
    Nice.DumpPrint("VTxlwid",&VTxlwid);
    Nice.DumpPrint("VTntmin",&VTntmin);
    Nice.DumpPrint("VTch2vtr",&VTch2vtr);
    Nice.DumpPrint("VTch2stt",&VTch2stt);
    Nice.DumpPrint("VTch2stv",&VTch2stv);
    Nice.DumpPrint("VTch2svw",&VTch2svw);
    Nice.DumpPrint("VTBvimpc",&VTBvimpc);
    Nice.DumpPrint("VTnmix",&VTnmix);
    Nice.DumpPrint("VTch2vv",&VTch2vv);
    Nice.DumpPrint("VTDraw",&VTDraw);
  */
  Nice.DumpPrint("RPTrackChoice",&RPTrackChoice);
  Nice.DumpPrint("RPTrackQualNH",&RPTrackQualNH);
  Nice.DumpPrint("RPMinTrks/Vtx",&RPMinTrksInVtx);
  Nice.DumpPrint("RPTrkWireSMax",&RPTrkWireSigmaMax);
  Nice.DumpPrint("GPEUpdateFreq",&GPEUpdateFreq);
  Nice.DumpPrint("GPEMinHits/Trk",&GPEMinHitsInTrack);
  Nice.DumpPrint("GPEMinTrks/Vtx",&GPEMinTracksInVertex);
  Nice.DumpPrint("GPEMaxChi2",&GPEMaxChi2);
  Nice.DumpPrint("GPEMaxChi2/Trk",&GPEMaxChi2PerTrack);
  Nice.DumpPrint("GPELong..Window",&GPELongitudinalWindow);
  Nice.DumpPrint("GPETransWindow",&GPETransverseWindow);
  Nice.DumpPrint("GPEChiWindow",&GPEChiWindow);
  Nice.DumpPrint("RSTrackQualNH",&RSTrackQualNH);
  Nice.DumpPrint("RSJpSeedMaxC2",&RSJpsiSeedMaxChi2);
  Nice.DumpPrint("RSJpSeedMaxD",&RSJpsiSeedMaxDist);
  /*
    Nice.DumpPrint("RSVtxVtxMinC2",&RSVtxVtxMinChi2);
    Nice.DumpPrint("RSTrkVtxMaxC2",&RSTrkVtxMaxChi2);
    Nice.DumpPrint("RSTrkVtxMaxD",&RSTrkVtxMaxDist);
    Nice.DumpPrint("RSTVDistRat",&RSTrkVtxDistRatio);
  */
  Nice.DumpPrint("RSMinSeedIPS",&RSMinSeedIPS);
  Nice.DumpPrint("RSMaxC2Contrib",&RSMaxChi2Contrib);
  Nice.DumpPrint("RSMaxTrkVtxChi2",&RSMaxTrkVtxChi2);
  Nice.DumpPrint("RSMaxTrkTrkChi2",&RSMaxTrkTrkChi2);
  Nice.DumpPrint("RSMax2TVtxChi2",&RSMax2TVtxChi2);
  Nice.DumpPrint("RSMaxChi2PDOF",&RSMaxChi2PDOF);
  Nice.DumpPrint("GroverDraw3D",&GroverDraw3D);
  Nice.DumpPrint("TDX3D",&TDX3D);
  Nice.DumpPrint("TDTargetRegion",&TDTargetRegion);
  Nice.DumpPrint("TDAxes",&TDAxes);
  Nice.DumpPrint("TDWires",&TDWires);
  Nice.DumpPrint("TDPrimaries",&TDPrimaries);
  Nice.DumpPrint("TDSecondaries",&TDSecondaries);
  Nice.DumpPrint("TDMCPrimaries",&TDMCPrimaries);
  Nice.DumpPrint("TDMCSecondaries",&TDMCSecondaries);
  Nice.DumpPrint("TDPrimaryTracks",&TDPrimaryTracks);
  Nice.DumpPrint("TDSecondaryTrks",&TDSecondaryTracks);
  Nice.DumpPrint("TDUnusedTracks",&TDUnusedTracks);
  Nice.DumpPrint("TDSpecialTracks",&TDSpecialTracks);
  Nice.DumpPrint("TDWireColour",&TDWireColour);
  Nice.DumpPrint("TDPrimaryColour",&TDPrimaryColour);
  Nice.DumpPrint("TDSecondaryCol",&TDSecondaryColour);
  Nice.DumpPrint("TDMCPrimaryCol",&TDMCPrimaryColour);
  Nice.DumpPrint("TDMCSecColour",&TDMCSecondaryColour);
  Nice.DumpPrint("TDPrimaryTrkCol",&TDPrimaryTrackColour);
  Nice.DumpPrint("TDSecTrkColour",&TDSecondaryTrackColour);
  Nice.DumpPrint("TDUnusedTrckCol",&TDUnusedTrackColour);
  Nice.DumpPrint("TDSpecialTrkCol",&TDSpecialTrackColour);
  Nice.DumpPrint("TDTrackSmooth",&TDTrackSmoothness);
  Nice.DumpPrint("TDPrimaryTrkLen",&TDPrimaryTrackLength);
  Nice.DumpPrint("TDSecTrkLen",&TDSecondaryTrackLength);
  Nice.DumpPrint("TDUnusedTrkLen",&TDUnusedTrackLength);
  Nice.DumpPrint("TDSpecialTrkLen",&TDSpecialTrackLength);
  Nice.DumpPrint("TDXAxisLength",&TDXAxisLength);
  Nice.DumpPrint("TDYAxisLength",&TDYAxisLength);
  Nice.DumpPrint("TDZAxisLength",&TDZAxisLength);
  Nice.DumpPrint("TDAxisRadius",&TDAxisRadius);
  Nice.DumpPrint("GETrackChoice",&GETrackChoice);
  Nice.DumpPrint("GERefillTracks",&GERefillTracks);
  Nice.DumpPrint("GEFindPrims",&GEFindPrimaries);
  Nice.DumpPrint("GERedoPrims",&GERedoPrimaries);
  Nice.DumpPrint("GEFindSecs",&GEFindSecondaries);
  Nice.DumpPrint("GERedoSecs",&GERedoSecondaries);
  Nice.DumpPrint("GERefitRSEGTab",&GERefitRSEGTable);
  Nice.DumpPrint("GEWriteRTAR",&GEWriteRTAR);
  Nice.DumpPrint("GEWriteRVER",&GEWriteRVER);
  Nice.DumpPrint("GPWireFollow",&GPWireFollowing);
  Nice.DumpPrint("GPNewWires",&GPNewWires);
  Nice.DumpFooter();
}

///////////////////////////////////////////////////////////////////////
