#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "grover/grover.hh"

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

ArtePointer<RSEG> RSEGCausedBy(ArtePointer<MTRA> mtra)
{
  // Function to decide which RSEG (if any) was caused
  // by a given MTRA. 
  // I.Kisel 13/06/00

  // Fraction of correct hits required
  const float threshold = 0.70;

  ArtePointer<RSEG> rseg = 0;
  bool successful = false;
  int maxhits = 0;

  for( ArteTable<RSEG>::iterator rsegIt = ArteTable<RSEG>::begin();
       rsegIt != ArteTable<RSEG>::end(); ++rsegIt ) {
    if (rsegIt->cmp  != (1 | Rsegc::bit16)) continue; // not CATS
    // count hits -> mimps -> mtra
    int nhits = 0;
    for( ArteRelationIterator<HITB> hitbIt = rsegIt->hitb.begin();
	 hitbIt != rsegIt->hitb.end(); ++hitbIt ){
      for( ArteRelationIterator<MIMP> mimpIt = (*hitbIt)->mimp.begin();
	   mimpIt != (*hitbIt)->mimp.end(); ++mimpIt) {
	if ((*mimpIt)->mtra == mtra) nhits++;
      }
    }
    if (static_cast<float>(nhits)/static_cast<float>(rsegIt->nhit)
	>= threshold) {
      // associated to MTRA
      successful = true;
      if (nhits > maxhits){ 
	// choose longest track to reject clones
	maxhits = nhits;
	rseg = *rsegIt;
      }
    }
  }
  

  if (successful) return rseg;
  else return 0;
}
