//
// Now part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

class DumpFormatter {

  ////////////////////////////////////////////////////////////////////////
  //
  // Class DumpFormatter
  // Author(s): Max Sang, MPI Muenchen (Max.Sang@cern.ch)
  //            Sebastian Slama, ETHZ (Sebastian@Slama.de)
  // Written: 10 March 2000 MS Basic structure
  // Updated: 13 March 2000 SS Added lots more DumpPrint()s
  //
  // Lets you dump the private members of your class to stdout in a 
  // controlled way. If users want to see the contents of the private 
  // members, they should be able to call YourClass->Dump();
  // 
  // YourClass::Dump() should look something like this:
  //
  //    void YourClass::Dump()
  //    {
  //     Util::DumpFormatter Nice;
  //     Nice->DumpClassName("YourClass")
  //     Nice->DumpComment("Class written by John Smith (smith@foobar.net)");
  //     Nice->DumpHeader();
  //     Nice->DumpPrint("x",&x);         // x is an integer
  //     Nice->DumpPrint("y",y, 200);     // y is an array of 200 floats
  //     Nice->DumpPrint("fred",&fred);   // fred is a Hep3Vector
  //     Nice->DumpPrint("jim",&jim,9,9); // jim is a 9*9 MyMatrix
  //     Nice->DumpFooter();
  //    } 
  //
  // It's so easy you don't have an excuse not to provide it ;-)
  //
  ////////////////////////////////////////////////////////////////////////

public:
   DumpFormatter();
  ~DumpFormatter();
  static int dump_verbosity;                 // default 1
  void DumpClassName(char classname[]);      // For ID purposes
  void DumpComment(char comment[]);          // Print out whatever you like
  void DumpHeader();                         // Print a simple header
  void DumpFooter();                         // Print a simple footer
  void DumpPrint(char varname[], ... );      // DEFAULT (does nothing)
  void DumpPrint(char varname[], int *var);// Print an integer....
  void DumpPrint(char varname[], int var[], int size);//overloaded ad infinitum
  void DumpPrint(char varname[], long *var);
  void DumpPrint(char varname[], long var[], int size);
  void DumpPrint(char varname[], unsigned long *var);
  void DumpPrint(char varname[], unsigned long var[], int size);
  void DumpPrint(char varname[], unsigned int var[], int size);
  void DumpPrint(char varname[], unsigned int *var);
  void DumpPrint(char varname[], long long *var);
  void DumpPrint(char varname[], unsigned long long *var);
  void DumpPrint(char varname[], long long var[], int size);
  void DumpPrint(char varname[], unsigned long long var[], int size);
  void DumpPrint(char varname[], short *var);
  void DumpPrint(char varname[], unsigned short *var);
  void DumpPrint(char varname[], short var[], int size);
  void DumpPrint(char varname[], unsigned short var[], int size);
  void DumpPrint(char varname[], float *var);
  void DumpPrint(char varname[], float var[], int size);
  void DumpPrint(char varname[], double *var);
  void DumpPrint(char varname[], long double *var);
  void DumpPrint(char varname[], double var[], int size);
  void DumpPrint(char varname[], long double var[], int size);
  void DumpPrint(char varname[], char *var);
  void DumpPrint(char varname[], unsigned char *var);
  void DumpPrint(char varname[], char var[], int size);
  void DumpPrint(char varname[], unsigned char var[], int size);
  void DumpPrint(char varname[], bool *var);
  void DumpPrint(char varname[], bool var[], int size);
  /*
  void DumpPrint(char varname[], complex<double> *var);
  void DumpPrint(char varname[], complex<long double> *var);
  void DumpPrint(char varname[], complex<float> *var);
  void DumpPrint(char varname[], Hep3Vector *vec);
  void DumpPrint(char varname[], HepLorentzVector *vec);
  void DumpPrint(char varname[], HepRotation *rot);
  void DumpPrint(char varname[], HepLorentzRotation *lrot);
  void DumpPrint(char varname[], HepPoint3D *pony);
  void DumpPrint(char varname[], HepVector3D *vecty);
  void DumpPrint(char varname[], HepPlane3D *ivan);
  void DumpPrint(char varname[], HepNormal3D *sven);
  */
private:
  int errorflag; // ??
};
