*CMZ :          27/05/96  20.02.58  by  Fedor Ratnikov
*CMZ :  1.01/00 27/12/95  16.43.59  by  Fedor Ratnikov
*CMZ :  1.00/08 23/11/95  21.21.51  by  Fedor Ratnikov
*CMZ :  1.00/01 16/08/95  13.26.17  by  S.B.
*CMZ :  1.00/00 06/01/95  14.57.02  by  Dominik Ressing
*CMZ :  0.01/02 29/08/94  13.10.33  by  Dominik Ressing
*-- Author :    Dominik Ressing   18/08/94
************************************************************************
*
* L1KINEX
* written by Dominik Ressing, 18/08/94 12.14.38
*
* This routines action:
* extrapolate message to next ROI(s). The kinematic limits given by
* the minimal amd the maximal momentum are obeied. If only one Point
* is known it must be Point2 and Point1(L1POz0)<0. If none of the two
* allowed regions (for the two charges) can be excluded two ROIs are returned.
* Inside this routine tan(x) = x !!!!!
*
*
* iAlgo  ==  CurrentSL*100+(Previous-Current)*10+(Current-next)
*-----------------------------------------------------------------------
* iAlgo       in          algorythm we should use for extrapolation
* Point1      in          array of type GEOINF for point at larger z
* Point2      in          array of type GEOINF for point at smaller z
* P           in          nominal momentum for track (for slopes in ROIs)
* Pmin        in          minimal momentum of track
* Pmax        in          maximal momentum of track
* Ptmin       in          minimal transverse momentum of track
* z           in          z of extrapolation
* nROI        out         number of found ROIs : 1 or 2
* ROI1        out         array of type L1ROI (see l1mesdf) for 1st ROI
* ROI2        out         array of type L1ROI for 2nd ROI
*
************************************************************************
c
      SUBROUTINE L1KINEX(iAlgo,Point1,Point2,P,Pmin,Pmax,z,
     &                   nROI,ROI1,ROI2)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
#include "l1si/l1mesdf.fi"
c
      REAL Point1(L1GEOINFlen),Point2(L1GEOINFlen),Pmin,Pmax,z,
     &     ROI1(0:L1ROIlen-1),ROI2(0:L1ROIlen-1),P
      INTEGER nROI,iAlgo
      INTEGER iCurSL,iPrevSL,iNextSL
      REAL dxms,dyms
c
      REAL BendMin,BendMax,AlpMax1,AlpMin1,AlpMax2,AlpMin2,
     &     y0,sy,y0Def,syDef,zDEf,xMax1,xMin1,xMax2,xMin2,
     &     AlpExtMax,AlpExtMin,dz,dx1,dx2,dy,dis,
     &     xl2,xu2,xl1,xu1,BendNom,fac,AlpNom1,AlpNom2,
     &     ymin,ymax,y0store,systore
c
      IF ( Point2(L1GEOINFz).lt.0. ) THEN
       WRITE (L1LUNprn,*) 'L1KINEX==> invalid point 2'
       nROI = 0
       RETURN
      END IF
c
      IF(iAlgo.EQ.0)  THEN
       iCurSL = 0
      ELSE
       iCurSL = iAlgo/100
       iPrevSL = MOD(iAlgo,100)/10+iCurSL
       iNextSL = iCurSL-MOD(iAlgo,10)
      ENDIF
c
c     --- find max/min slopes from kinematic limits (*weighting)
      fac     = 0.003*T1DETmagB*T1DETmagd*T1DETmagz
      BendMin = fac/Pmax
      BendMax = fac/Pmin
      BendNom = fac/P
c

c
      xl2 = Point2(L1GEOINFx0)
      xu2 = xl2 + Point2(L1GEOINFsx)
c     --- here for q = -1: abs(AlpMax2) > abs(AlpMin2)  and
c     AlpMax2>AlpMin2
      AlpMax1 = (BendMax - xu2)/Point2(L1GEOINFz) ! maximal abs(bending) for q=-1
      AlpMin1 = (BendMin - xl2)/Point2(L1GEOINFz) ! minimal abs(bending) for q=-1
      AlpNom1 = (BendNom - xl2)/Point2(L1GEOINFz) ! minimal abs(bending) for q=-1
c     --- here for q = +1: abs(AlpMax2) > abs(AlpMin2)  but
c     AlpMax2<AlpMin2
      AlpMax2 = -1.*( (BendMax + xl2)/Point2(L1GEOINFz) )
      AlpMin2 = -1.*( (BendMin + xu2)/Point2(L1GEOINFz) )
      AlpNom2 = -1.*( (BendNom + xu2)/Point2(L1GEOINFz) )
c
c     For muons system we do not use this information
c
      IF(iAlgo.NE.0) THEN

       AlpMax1 =1000000.
       AlpMin1 =-1000000.
       AlpNom1 =0.

       AlpMax2 =-1000000.
       AlpMin2 =1000000.
       AlpNom2 =0.

      ENDIF

c
c     --- extrapolate using the two points:
      IF ( Point1(L1GEOINFz).gt.0. ) THEN ! both points valid ??
       xl1 = Point1(L1GEOINFx0)
       xu1 = xl1 + Point1(L1GEOINFsx)
       dz  = Point1(L1GEOINFz)-Point2(L1GEOINFz)
       AlpExtMax = ( xu2-xl1 )/dz ! larger bound
       AlpExtMin = ( xl2-xu1 )/dz ! smaller bound
       AlpMax1 = MIN(AlpExtMax,AlpMax1) ! larger bound for q = -1
       AlpMin1 = MAX(AlpExtMin,AlpMin1) ! lower bound for q = -1
       AlpMin2 = MIN(AlpExtMax,AlpMin2) ! larger bound for q = 1
       AlpMax2 = MAX(AlpExtMin,AlpMax2) ! lower bound for q = 1
       ymin = MAX(Point1(L1GEOINFy0),
     &      Point2(L1GEOINFy0)/Point2(L1GEOINFz)*Point1(L1GEOINFz))
       ymax = MIN(Point1(L1GEOINFy0)+Point1(L1GEOINFsy),
     &      (Point2(L1GEOINFy0)+Point2(L1GEOINFsy))/
     &      Point2(L1GEOINFz)*Point1(L1GEOINFz))
       IF(iAlgo.EQ.0)  THEN     ! projective extrapolation
        IF(ymin.LT.ymax)  THEN
         y0Def = ymin           ! overlap of both points
         syDef = ymax-ymin
         zDef  = Point1(L1GEOINFz)
        ELSE                    ! not overlaping points
         nROI = 0
         GOTO 7777              ! return
        ENDIF
       ELSE                     ! muons
        y0Def = Point2(L1GEOINFy0)
        syDef = Point2(L1GEOINFsy)
        zDef  = Point2(L1GEOINFz)
       ENDIF
      ELSE
       y0Def = Point2(L1GEOINFy0)
       syDef = Point2(L1GEOINFsy)
       zDef  = Point2(L1GEOINFz)
      END IF


      dz = Point2(L1GEOINFz) - z
      IF ( AlpMax1.gt.AlpMin1 ) THEN
       xMax1 = dz*AlpMax1 + xu2
       xMin1 = dz*AlpMin1 + xl2
c      dx1 = (AlpMin1+AlpMax1)/2.
       dx1 = AlpNom1
       IF ( ALpMax2.lt.AlpMin2 ) THEN
        xMax2 = dz*AlpMin2 + xu2
        xMin2 = dz*AlpMax2 + xl2
c       dx2 = (AlpMin2+AlpMax2)/2.
        dx2 = AlpNom2
        dis = (xMin1-xMax2)/(xMax1-xMin2) ! relative distance between 2 regions
        IF ( dis.lt.T1DEFroiDisMin .and. ! the 2 overlap   .and.
     &       abs(dx1-dx2).lt.T1DEFroiSloMax ! not too different slopes
     &       ) THEN             !  : -> combine
         xMin1 = xMin2
         dx1 = (AlpNom1+AlpNom2)/2.
         xMax2 = -1.
         xMin2 = 1.
         dx2 = 0.
        END IF
       ELSE
        xMax2 = -1.
        xMin2 = 1.
        dx2 = 0.
       END IF
      ELSE
       xMax2 = -1.
       xMin2 = 1.
       dx2 = 0.
       IF ( ALpMax2.lt.AlpMin2 ) THEN
        xMax1 = dz*AlpMin2 + xu2
        xMin1 = dz*AlpMax2 + xl2
c       dx1 = (AlpMax2+AlpMin2)/2.
        dx1 = AlpNom2
       ELSE
        xMax1 = -1.
        xMin1 = 1.
        dx1 = 0.
       END IF
      END IF
c
      dy = y0Def/zDef
      y0 = z*dy
      sy = syDef*z/zDef
c
      IF( xMin1 .LT. xMax1 )THEN
       IF(iAlgo.NE.0) THEN
        CALL L1MUSCA(iPrevSL,iCurSL,iNextSL,
     &       Point2(L1GEOINFx0)+0.5*Point2(L1GEOINFsx),
     &       Point2(L1GEOINFy0)+0.5*Point2(L1GEOINFsy),
     &       Point2(L1GEOINFz),
     &       dxms,dyms)
        xMin1 = xMin1-dxms
        xMax1 = xMax1+dxms
        y0 = y0-dyms
        sy = sy+2.*dyms
       ENDIF
       ROI1(L1ROIreg+L1GEOINFx0) = xMin1
       ROI1(L1ROIreg+L1GEOINFsx) = xMax1-xMin1
       ROI1(L1ROIreg+L1GEOINFy0) = y0
       ROI1(L1ROIreg+L1GEOINFsy) = sy
       ROI1(L1ROIreg+L1GEOINFz ) = z
       ROI1(L1ROIreg+L1GEOINFsin_al) = 0.
       ROI1(L1ROIreg+L1GEOINFcos_al) = 1. ! rectangular ROI
       IF(dz.NE.0) THEN         ! nonzerro extrapolation
        ROI1(L1ROIdxdz) = 0.5*(xu2+xl2-xMax1-xMin1)/dz ! centers...
       ELSE
        ROI1(L1ROIdxdz) = 0.
       ENDIF
c      ROI1(L1ROIdxdz) = -1.*dx1
       ROI1(L1ROIdydz) = dy
       IF( xMIN2 .LT. xMax2 )THEN
        nROI = 2
        IF(iAlgo.NE.0) THEN
         xMin2 = xMin2-dxms
         xMax2 = xMax2+dxms
        ENDIF
        ROI2(L1ROIreg+L1GEOINFx0) = xMin2
        ROI2(L1ROIreg+L1GEOINFsx) = xMax2-xMin2
        ROI2(L1ROIreg+L1GEOINFy0) = y0
        ROI2(L1ROIreg+L1GEOINFsy) = sy
        ROI2(L1ROIreg+L1GEOINFz ) = z
        ROI2(L1ROIreg+L1GEOINFsin_al) = 0.
        ROI2(L1ROIreg+L1GEOINFcos_al) = 1. ! rectangular ROI
        IF(dz.NE.0) THEN        ! nonzerro extrapolation
         ROI2(L1ROIdxdz) = 0.5*(xu2+xl2-xMax2-xMin2)/dz ! centers...
        ELSE
         ROI2(L1ROIdxdz) = 0.
        ENDIF
c       ROI2(L1ROIdxdz) = -1.*dx2
        ROI2(L1ROIdydz) = dy
       ELSE
        nROI = 1
       END IF
      ELSE
       nROI = 0
      END IF
c
 7777 CONTINUE
      RETURN
c
      END







