//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcSegPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"

#include "marple/marple.hh"

RcSegPoint::RcSegPoint( const ArtePointer <RSEG>  rsegLink,
			int EndPoint)
  : RcPoint(), fSegLink(rsegLink)  
{
  if( !(rsegLink != 0 ) || 
      rsegLink - *ArteTable<RSEG>::begin() >  (int)ArteTable<RSEG>::size() )
    { 
      arteMsg << " MARPLE: RSEG entry number " << rsegLink 
	      << "does not exist";
      arteMsg.allp();
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }
  if( maRichSegment(rsegLink) )  // RICH segments 
    {
      setX(   0  );
      setY(   0  );
      setZ(   0  );
      settx(  rsegLink->txf );
      setty(  rsegLink->tyf );
      setpe(  0 );
      setcov(3,3, rsegLink->cvf[c_matrix_pack[2][2]] ); // remember about index starts from 0 !
      setcov(3,4, rsegLink->cvf[c_matrix_pack[2][3]] );
      setcov(4,4, rsegLink->cvf[c_matrix_pack[3][3]] );
      //setcov(3,3, rsegLink->cvf[c_matrix_pack[3][3]] );
      //setcov(3,4, rsegLink->cvf[c_matrix_pack[3][4]] );
      //setcov(4,4, rsegLink->cvf[c_matrix_pack[4][4]] );
    }
  else if( EndPoint )
    {
      setX(   rsegLink->xe  );
      setY(   rsegLink->ye  );
      setZ(   rsegLink->ze  );
      settx(  rsegLink->txe );
      setty(  rsegLink->tye );
      setpe(  rsegLink->pe );
      setcov( rsegLink->cve );
    }else{
      setX(   rsegLink->xf  );
      setY(   rsegLink->yf  );
      setZ(   rsegLink->zf  );
      settx(  rsegLink->txf );
      setty(  rsegLink->tyf );
      setpe(  rsegLink->pf );
      setcov( rsegLink->cvf );
    }
  checkAndCorrectCov();
}

// returns relation with MTRA 
const std::vector<ArtePointer<MTRA> > RcSegPoint::mtraList() const
{
  std::vector<ArtePointer<MTRA> > mList;
  if( ArteTable<MTRA>::size() == 0 ) return mList;
  for( ArteRelationIterator<MTRA> rsMtra = link()->mtra.begin();
       rsMtra != link()->mtra.end();
       ++rsMtra )
    {
      //ArtePointer<MTRA> mEntry(rsMtra );
      //mList.push_back(mEntry);
      mList.push_back(*rsMtra);
    }
  return mList;
}

// returns relation with HITB 
const std::vector<ArtePointer<HITB> > RcSegPoint::hitbList() const
{
  std::vector<ArtePointer<HITB> > hList;
  if( ArteTable<HITB>::size() == 0 ) return hList;
  for( ArteRelationIterator<HITB> rsHitb = link()->hitb.begin();
       rsHitb != link()->hitb.end();
       ++rsHitb )
    {
      hList.push_back(*rsHitb);
    }
  return hList;
}
//------------------------------------------------------
// update RSEG bank by RcSegPoint information

void RcSegPoint::updateARTE( int & iErr ) const
{
  int EndPoint;
  if( fabs( link()->ze - z() ) <  fabs( link()->zf - z() ) )
    {
      EndPoint = 1;
    }else{
      EndPoint = 0;
    }
    updateARTE( iErr, EndPoint ); 
}

// update RSEG bank by RcSegPoint information
void RcSegPoint::updateARTE( int & iErr, int EndPoint ) const
{
  iErr = 0;
  ArtePointer<RSEG> rseg = link();
  if( EndPoint )
    {
      static int idMess1 = 75673456;
      arteMsg << "MARPLE:  updateARTE: RcSegPoint is saved " 
	      << "as end point of RSEG";
      arteMsg.allp(idMess1,1);
      rseg->xe  = x();
      rseg->ye  = y() ;
      rseg->ze  = z() ;
      rseg->txe = tx() ;
      rseg->tye = ty();
      rseg->pe  = pe();
      for( int i=0; i<5; i++){
	for(int j=i; j<5; j++){
	  rseg->cve[c_matrix_pack[i][j]] = cov()[i][j]; 
	  // start numbering from [0][0] as in struct RSEG
	}
      }
    }else{
      static int idMess2 = 342873456;
      arteMsg << "MARPLE:  updateARTE: RcSegPoint is saved " 
	      << "as first point of RSEG";
      arteMsg.allp(idMess2,1);
      rseg->xf  = x();
      rseg->yf  = y() ;
      rseg->zf  = z() ;
      rseg->txf = tx() ;
      rseg->tyf = ty();
      rseg->pf  = pe();
      for( int i=0; i<5; i++){
	for(int j=i; j<5; j++){
	  rseg->cvf[c_matrix_pack[i][j]] = cov()[i][j]; 
	  // start numbering from [0][0] as in struct RSEG
	}
      }
    }
}

ostream & operator << (ostream & os, const RcSegPoint &rp)
{
  //os << static_cast<RcPoint&>(rp) ;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RSEG link: " << rp.link() << endl ;
  return os ;
}
