
//**********************************************************************/
// Created: 30.07.00 by   P.Conde, W.Hulsbergen & O.Igonkina 
//
// implementation of SegRpntBuilder class
//
//**********************************************************************/

#include <math.h>
#include <arte/arte.hh>
#include "arte/RSEG_cmp.hh"

#include "marple/marple.hh"
#include "marple/RcMcFunction.hh"
#include "marple/RcSegRpntBuilder.hh"

#include "marple/RcHbook.hh"
#include "marple/RcTuples.h"

RcSegRpntBuilder::RcSegRpntBuilder() :
  RcMatchBuilder<RcSegPoint,RcRpntPoint>()
{}

RcSegRpntBuilder::RcSegRpntBuilder( std::vector<RcSegPoint  *> & vecLeft,
				    std::vector<RcRpntPoint *> & vecRight,
				    RcMatchConst & rcConst)
  : RcMatchBuilder<RcSegPoint, RcRpntPoint>(vecLeft, vecRight, rcConst)
{}

RcSegRpntBuilder::~RcSegRpntBuilder()
{
  //RcMatchBuilder<RcSegPoint,RcRpntPoint>::~RcMatchPoint(*this);
}

//------------------------------------------------------
void RcSegRpntBuilder::doMatch()
{
  RcMatchBuilder<RcSegPoint, RcRpntPoint>::doMatch();
}

//------------------------------------------------------
void RcSegRpntBuilder::doMatch(  MatchFunction matchFunction )
{

  // is there anything to match ?
  if( vectorLeft().size() == 0  || vectorRight().size() == 0 )
    return;

  int baseMark = 0;
  RcMatchConst * ptConst =  ptToMatchConst();

  int compareChi2 = 0;
  if( ptConst->chi2 > 0 &&  
      ptConst->chi2 < RcMatchConst::maxConsideredChi2 ) compareChi2 = 1;
  
  if( compareChi2 &&  ptConst->use_pair_cut ) //????? should we add paircut?
    {
      for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
	vectorLeft()[ iLeft ]->setmark( ptConst->n_pair_left);
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
	vectorRight()[ iRight ]->setmark( ptConst->n_pair_right);
    }

  fNrealPair = 0;
  fNfastReal = 0;

  int goodCombCut = 0;
  int goodFastCut = 0;

  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
    {
      RcSegPoint * segment = vectorLeft()[ iLeft ];      
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
        {
	  RcRpntPoint * rpnt = vectorRight()[iRight];

	  // check Monte Carlo
          if( ArteTable<MTRA>::size() )
            {
              ArtePointer<MTRA> imtra = 
                RcMtraPointer( segment->mtraList(), rpnt->mtraList() );
              if( imtra != 0 )
                {  
                  fNrealPair++;            
                  if( RcMtraReference( imtra ) ) { fNfastReal++;} 
                }
            }

	  // do Matching
	  
	  RcMatchPoint * matchPoint = 0;
	  // matchFunction will create 'new' object!! if not 0
	  if(  ((*this).*matchFunction)(segment, rpnt, matchPoint) )
	    {
	      // check in addition chi2
	      if( compareChi2 && matchPoint->chi2() > ptConst->chi2 )
		{
		  delete matchPoint;
		  matchPoint = 0;
		  continue;
		}

	      // suppose all match points of this type
	      // should have the same mark
	      baseMark = matchPoint->mark(); 
	      fmatchList.push_back(matchPoint);
	      matchPoint = 0;

	      // calculate MTRA statistics
	      if( ArteTable<MTRA>::size() )
		{
		  ArtePointer<MTRA> imtra = 
		    RcMtraPointer( segment->mtraList(), rpnt->mtraList() );
		  if( imtra != 0 )
		    { 
		      goodCombCut++ ; 
		      if ( RcMtraReference(imtra ) )goodFastCut++;
		    }     
		}	      

	      // end of matchFunction
	    }
	}
    }	
  
  //**********************************************************************/
  // check the number of combinations with given RcPoints
  //**********************************************************************/

  // sort vector of RcMatchPoints by chi2() 
  if( ptConst->chi2 > 0 &&  
      ptConst->chi2 < RcMatchConst::maxConsideredChi2 )
    {
      sort( fmatchList.begin(), fmatchList.end(), comp_chi2 );
      
      if( ptConst->use_pair_cut ) 
	{

	  goodCombCut = 0; // reset values
	  goodFastCut = 0;	  

	  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
	    {
	      if( matchList()[iMatch]->mark() != baseMark ) continue;
	      int checkOk = 1;
	      RcSegPoint  * ptLeft   =  leftOrigin(iMatch);
	      RcRpntPoint * ptRight  =  rightOrigin(iMatch);
	      
	      if( !(ptConst->pair_cut_ok( *ptLeft  )) ) checkOk = 0 ;
	      if( !(ptConst->pair_cut_ok( *ptRight )) ) checkOk = 0 ;
	      if( !checkOk )
		{ 
		  delete( fmatchList[iMatch]);
		  fmatchList.erase( &fmatchList[iMatch]) ; 
		  iMatch--;
		}else{
		  int mark = ptLeft->mark();
		  mark--;
		  ptLeft->setmark(mark );
		  mark = ptRight->mark();
		  mark--;
		  ptRight->setmark(mark );

		  // calculate MTRA statistics
		  if( ArteTable<MTRA>::size() )
		    {
		      ArtePointer<MTRA> imtra = 
			RcMtraPointer( ptLeft->mtraList(), 
				       ptRight->mtraList() );
		      if( imtra != 0 )
			{ 
			  goodCombCut++ ; 
			  if ( RcMtraReference(imtra ) )  goodFastCut++;
			}     
		    }

		}// good combination
	    }
	}
    }

  fNbad = matchList().size() - goodCombCut;
  fNlost = fNfastReal - goodFastCut ;

}

/*
//-------------------------------------------------------------
void RcSegRpntBuilder::matchWithMomentum()
{
  doMatch(&RcSegRpntBuilder::matchPairWithMomentum);
}

int RcSegRpntBuilder::matchPairWithMomentum( RcSegPoint  * segment, 
					     RcRpntPoint * rpnt,
					     RcMatchPoint* &result)
{
  result = 0;
   if ( fptRcConst == 0 ) return 0;

  
  // first check that simple cuts are fine   
  if( fptRcConst->use_delta_cut ) 
    {
      float segY = segment->y()+segment->ty()*(rpnt->z()-segment->z());
      if( fabs(segY-rpnt->y() ) >
	  fptRcConst->delta_y*1.1 ) // y direction doesn't fit
	{return 0;}
    }

  int iErr = 0;

  //-------------------------------------------------------
  // Here we try to get momentum from somewhere else
  //  
  //-------------------------------------------------------
  //
  //
  float pe1, pe2;
  iErr = yourMomentum(*segment, *rpnt, pe1, pe2);
  // HepDouble cov55 = if any
  if( iErr ) { return 0;}

  //-------------------------------------------------------
  

  // Here is suggestion how to operate
  // The number of temporary object could be optimized but
  // the original objects should not be prolongated many times 
  // to different directions

  RcSegPoint tmp1 = *segment;
  segment->setpe(pe1);
  segment->prolongate(fptRcConst->z_plane, iErr);
  if( iErr ) { *segment = tmp1; }// did not succeed to prolongate

  RcRpntPoint tmp2 = *rpnt;
  rpnt->setpe( pe2);
  rpnt->prolongate(fptRcConst->z_plane, iErr);

  if( !iErr && fptRcConst->use_delta_cut )
    {
      if( fptRcConst->use_delta_cut == 2 &&
	  !fptRcConst->energy_delta_cut_ok( *segment, *rpnt) )    iErr = 2;
      else if( !fptRcConst->delta_cut_ok( *segment, *rpnt) )      iErr = 1;
    }

  if( iErr ) // did not succeed to prolongate or combine 
    {
      *segment = tmp1;
      *rpnt    = tmp2;
    }


  // Here you have to make sure that both points are in proper state.
  // if z() is different it prolongates first RcPoint to Z of the second one.
  result = new RcMatchPoint( *segment, *rpnt,ptToMatchConst()->flag);

  *segment = tmp1;
  *rpnt = tmp2;

  // save momentum becouse it is not build in RcMatchPoint
  result->setpe( pe1 );
  //result->setcov(5,5, cov55 );

  if(  result->mark() )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(1);
  return 1;
}
*/

//-------------------------------------------------------------
void RcSegRpntBuilder::matchInTheMiddle()
{
  doMatch(&RcSegRpntBuilder::matchPairInTheMiddle);
}

extern "C" 
{ extern void FORTRAN_ROUTINE( rkmain )( int*,
                                         double*,
                                         double*,
                                         double [6],
                                         double [4],
                                         double [6],
                                         double [5][5],
                                         int*);
}

int RcSegRpntBuilder::matchPairInTheMiddle( RcSegPoint  * segment, 
					     RcRpntPoint * rpnt,
					     RcMatchPoint* &result)
{
  result = 0;
   if ( fptRcConst == 0 ) return 0;


   if( fabs(segment->z() - rpnt->z() )  < 100 )
     {
       cout << " matchInTheMiddle :ERROR: segment and the point is too close \n";
       return 0;
     }

   double Zmiddle =  fptRcConst->z_plane; //454.6;

   // first that it point to proper Y at OTR
   if( fabs(segment->y()+segment->ty()*(rpnt->z()-segment->z()) -rpnt->y())
       > 35.7 )     return 0;
   if( fabs(segment->x()+segment->tx()*(Zmiddle-segment->z()) -
	    ( rpnt->x()+ rpnt->tx()*(Zmiddle -rpnt->z()) ) ) > 18.6 )
     return 0;


  int iErr = 0;

  //-------------------------------------------------------
  // Estimation of momentum using the kick in the middle
  

  double Pin1[6]  = { segment->z(),  segment->x(),  segment->y(),  
		      segment->tx(),  segment->ty(),  segment->pe() };
  double Pin2[6]  = { rpnt->z(),  rpnt->x(),  rpnt->y(),  
		      rpnt->tx(),  rpnt->ty(),  rpnt->pe() };
  
  float distanceCor  = min( fabs(rpnt->z()-455), fabs(segment->z()-455) );
  if( distanceCor < 1 ) distanceCor = 1;
  double Prec[4] = { 0.001/distanceCor, 0.001/distanceCor, 
		       0.0001/distanceCor, 0.0001/distanceCor};
  static double Pout1[6]; static double Pout2[6] ;
  static double RKD1[5][5];
  static double RKD2[5][5];
  
  int Irkmain = 2; // default value - fast calculation of matrix
  double Fstep1 = fabs(segment->z()-455);
  double Fstep2 = fabs(rpnt->z()-455);

  // first approximation : kick 
  float mom = ( rpnt->tx() - segment->tx() ) /0.64 *
    sqrt((1+segment->tx()*segment->tx() ) / 
	 (1+segment->tx()*segment->tx()+segment->ty()*segment->ty()) );
  Pin1[5] = mom;
  Pin2[5] = mom;
  
  // iterations
  const float precision = 0.00001; // 0.01 mrad

  // C++ style of array is opposite to fortran's one
  // when we use '[]' the numeration starts from 0
  for( unsigned int i = 0 ; i < 5; ++i )
    {    

      rkmain_(&Irkmain, &Zmiddle, &Fstep1, Pin1, Prec, Pout1, RKD1, &iErr );
      if( iErr ) return 0;
      if( fabs(RKD1[4][2]) < 0.1e-20 ){ break;}
      rkmain_(&Irkmain, &Zmiddle, &Fstep2, Pin2, Prec, Pout2, RKD2, &iErr );
      if( iErr ) return 0;
      if( fabs(RKD2[4][2]) < 0.1e-20 ){  break;}
      
      double residualtx = Pout1[3] - Pout2[3]; // 0-z, 1-x, 2-y, 3-tx, 4-ty, 5-p 
      Pin1[5] -= residualtx /( RKD1[4][2] - RKD2[4][2] );
      Pin2[5] = Pin1[5];
      if( fabs( residualtx )< precision ) break;
    }

  //-------------------------------------------------------
  RcSegPoint start1 = *segment;
  RcRpntPoint start2 = *rpnt;

  if( fabs(Pout1[0] -Zmiddle ) > 5 || fabs(Pout2[0] -Zmiddle ) > 5 )
    { cout << "MatchInTheMiddle: Something fishy with Z coordinate \n"; return 0;}

  if(  fptRcConst->chi2 > 0 && 
       fptRcConst->chi2 < RcMatchConst::maxConsideredChi2 ) 
    {      
      // chi2 must make sense, therefore prolongate correctly again
      segment->setpe( Pin1[5] );
      rpnt->setpe(    Pin2[5] );
      segment->prolongate( Zmiddle, iErr );
      rpnt->prolongate(    Zmiddle, iErr );
    }else{ 

      segment->setZ(  Pout1[0] );
      segment->setX(  Pout1[1]);
      segment->setY(  Pout1[2]);
      segment->settx( Pout1[3]);
      segment->setty( Pout1[4]);
      segment->setpe( Pin1[5]);
      
      rpnt->setZ(  Pout2[0] );
      rpnt->setX(  Pout2[1]);
      rpnt->setY(  Pout2[2]);
      rpnt->settx( Pout2[3]);
      rpnt->setty( Pout2[4]);
      rpnt->setpe( Pin2[5]);
    }

  // cut on X
  if( fptRcConst->use_delta_cut && fptRcConst->flag.ifmarked_X() )
    if( fabs( segment->x() - rpnt->x() ) >  fptRcConst->delta_x ) 
      iErr = 1;
    
  if( iErr ) // did not succeed to prolongate or combine 
    {
      *segment = start1;
      *rpnt    = start2;
      return 0;
    }

  // recover rpnt, move segment to it

  RcSegPoint  middle1 = *segment;
  RcRpntPoint middle2 = *rpnt;

  *rpnt = start2;
  segment->prolongate(rpnt->z(), iErr );

  // cut on Y
  float dy = 0;
  float dty = 0;
  if( !iErr && fptRcConst->use_delta_cut && fptRcConst->flag.ifmarked_Y() )
    {
      dy = segment->y() - rpnt->y();
      if( fabs( dy ) >  fptRcConst->delta_y ) 
	iErr = 1;     
    }
  if( !iErr && fptRcConst->use_delta_cut && fptRcConst->flag.ifmarked_TY() )
    {
      dty = segment->ty() - rpnt->ty();
      if( fabs( dty ) >  fptRcConst->delta_ty ) 
	iErr = 1;     
    }

  if( iErr ) // did not succeed to prolongate or combine 
    {
      *segment = start1;
      //*rpnt    = start2;
      return 0;
    }

  *segment  = middle1;
  *rpnt     = middle2;

  // Here you have to make sure that both points are in proper state.
  // if z() is different it prolongates first RcPoint to Z of the second one.
  result = new RcMatchPoint( *segment, *rpnt,ptToMatchConst()->flag);

  if( fptRcConst->use_delta_cut && fptRcConst->flag.ifmarked_Y() )
    result->setDeltaY(dy);

  if( fptRcConst->use_delta_cut && fptRcConst->flag.ifmarked_TY() )
    result->setDeltaTY(dty);

  *segment = start1;
  *rpnt = start2;

  // save momentum becouse it is not build in RcMatchPoint
  result->setpe( Pin1[5] );
  //result->setcov(5,5, cov55 );

  if(  result->mark() )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(1);
  return 1;
}




void RcSegRpntBuilder::saveRSEG(int & iErr ) const
{
  iErr = 0;
  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
    {
      RcMatchPoint * matchPoint = fmatchList[iMatch];
      if( matchPoint->mark() < 0 ) continue;
      RSEG rseg;
      rseg.zf  = matchPoint->z();
      rseg.xf  = matchPoint->x();
      rseg.yf  = matchPoint->y();
      rseg.txf = matchPoint->tx();
      rseg.tyf = matchPoint->ty();
      rseg.pf  = matchPoint->pe();

      for( unsigned int i = 0; i < 5 ; ++i )
        for( unsigned int j = i; j < 5 ; ++j )
          rseg.cvf[c_matrix_pack[i][j]] = matchPoint->cov()(i+1,j+1);
      
      rseg.chi   = matchPoint->chi2();
      rseg.fit   = 7;
      rseg.time  = 888899;
      rseg.dtim  = matchPoint->mark();

      RcSegPoint* ptLeft   =  leftOrigin(iMatch);
      RcRpntPoint* ptRight =  rightOrigin(iMatch);
      if( ptRight->link()->rseg.index() )
	rseg.cmp   =
	  ptLeft->link()->cmp | ptRight->link()->rseg->cmp | Rsegc::bit18;
      else
	rseg.cmp   = ptLeft->link()->cmp | Rsegc::patt | Rsegc::bit18;

      rseg.ze  = 0;
      rseg.xe  = matchPoint->deltaX();
      rseg.ye  = matchPoint->deltaY();
      rseg.txe = matchPoint->deltaTX();
      rseg.tye = matchPoint->deltaTY();
      rseg.pe  = matchPoint->deltaPE();

      for( unsigned int i = 0; i < 15 ; ++i )
        rseg.cve[i] = 0;
      
      rseg.err  = 0;
      rseg.npar =  ptToMatchConst()->flag.Nvariables();
      rseg.nhit = 2*rseg.npar;

      ArtePointer<RSEG> Rseg=ArteTable<RSEG>::push_back(rseg);
      int iErr1 = 0;
      saveRelationWithMatch( ptLeft, ptRight, Rseg, iErr1 );
      if( iErr1 ) iErr = iErr1;

      // save relation with MTRA

      if( ArteTable<MTRA>::size() )
        {
          ArtePointer<MTRA> mtra = 
            RcMtraPointer(ptLeft->mtraList(), ptRight->mtraList());
          //      if( RcMtraPointer(matchPoint->mtraList().size() == 1 )
          // ArtePointer<MTRA> mtra = *(matchPoint->mtraList().begin());
          if( mtra.index() ) 
            createRelation(Rseg, mtra, &RSEG::mtra);
        }

      ptLeft = 0;
      ptRight = 0;
      matchPoint = 0;
    } 
}

//--------------------------------------------
void  RcSegRpntBuilder::saveRelation(int & iErr) const
{
  if( !matchList().size() ){ iErr = 0; return; }
  iErr = 0;
  for(unsigned int iMatch=0; iMatch < matchList().size(); iMatch++)
    {      
      if( matchList()[iMatch]->mark() < 0 ) continue;

      int iErr1=1;
      saveRelation(leftOrigin(iMatch),
                   rightOrigin(iMatch), 
                   iErr1); 
      if( iErr1 ){ iErr = 1; }
    }
}



void RcSegRpntBuilder::saveRelationWithMatch(const RcSegPoint * ptPoint1,
					     const RcRpntPoint * ptPoint2,
					     ArtePointer<RSEG> rseg, 
					     int & iErr) const
{
  // get a relation for rpnt
  ArtePointer<RSEG> rsegRpnt = ptPoint2->link()->rseg;
  if( ! rsegRpnt.index() ){ iErr = 1; return ;} // no rseg is assigned

  // replace ptPoint1 and ptPoint2
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint1->link());
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, rsegRpnt);
  iErr = 0;
  return ;
}

void RcSegRpntBuilder::saveRelation( const RcSegPoint * ptPoint1,
				     const RcRpntPoint * ptPoint2,
				     int & iErr) const
{
  // get a relation for rpnt
  ArtePointer<RSEG> rsegRpnt = ptPoint2->link()->rseg;
  if( ! rsegRpnt.index() ){ iErr = 1; return ;} // no rseg is assigned

  iErr = 0;
  if( ptPoint1->link()->zf < rsegRpnt->zf )
    {
      mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
               ptPoint1->link(), rsegRpnt);
    }else{
      mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
               rsegRpnt, ptPoint1->link());
    }
  return ;
}
