#ifndef _RCHITBPOINT_HH_
#define _RCHITBPOINT_HH_
//-----------------------------------------------------------------------
// Created: 13.08.97,   Olya Igonkina

// MARPLE class of Points for Reconstruction which have link to HITB bank

//-----------------------------------------------------------------------

#include <iostream.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "marple/RcPoint.hh"

/** class with link to HITB bank.
    Class is inherited from RcPoint.
    If constructor RcHitBPoint( const ArtePointer <HITB> rhitLink) is used 
    {\bf X, Y, Z } values equal {\bf hitb->x, hitb->y, hitb->z} respectively.
    The {\bf hitb->q } value is saved as {\bf PE } and {\bf hitb->r} 
    as {\bf cov()(5,5)}.
 */
class RcHitBPoint : public RcPoint
{

public:
  ///                 default constructor
  inline RcHitBPoint(); 
  ///                 constructor with link to HITB bank    
  inline RcHitBPoint( const RcPoint &rp, 
		     const ArtePointer <HITB> rhitLink = 0 );
  /// constructor from HITB bank entry 
  inline RcHitBPoint( const ArtePointer <HITB> rhitLink);

  ///                 copy constructor
  inline RcHitBPoint( const RcHitBPoint &rp ); 

  ///                 destructor
  inline ~RcHitBPoint(); 

  ///                 return link to HITB table
  inline ArtePointer <HITB>  link() const; 

  ///                 set link to HITB table
  inline void  setlink( const ArtePointer <HITB> rhitLink = 0 );
  
  ///                 return relation with MTRA 
  const std::vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 return relation with HITB 
  const std::vector<ArtePointer<HITB> > hitbList() const; 

  ///                 assignment operator
  inline RcHitBPoint & operator = (const RcHitBPoint &rp);
  ///                 assignment operator
  inline RcHitBPoint & operator = (const RcPoint &rp); 

  /// no Kalman refit of RcHitBPoint implemented. dummy function
  HepDouble refitAll(int & iErr);
  /// no Kalman refit of RcHitBPoint implemented. dummy function
  HepDouble refitForward(int & iErr);
  /// no Kalman refit of RcHitBPoint implemented. dummy function
  HepDouble refitBack(int & iErr);

private:
  ///                 ArtePointer to HITB bank
  ArtePointer  <HITB> fHitBLink;
};

//                   ostream operator to print RcHitBPoint information
ostream & operator << (ostream &, const RcHitBPoint &);

#include "marple/RcHitBPoint.icc"

#endif

