#ifndef _RCLIMIT_H_
#define _RCLIMIT_H_
//---------------------------------------------------------------------
// Created: 23.08.97 by   Olya Igonkina  for MARPLE
//
// MARPLE structure of maximum \& minimum limits for matching procedure
//
//---------------------------------------------------------------------

#include <math.h>
#include "marple/RcMatchFlag.hh"

/** {\bf MARPLE structure of maximum #&# minimum limits for matching procedure}\\

  The structure is designed to reduce the number of RcPoints to be matched.
  It contains the maximum and minimum values( limits) for different parameters 
  defined by flag. Set limits for one part of objects to reduce 
  the number of objects to be matched from other part. The {\it icheck} 
  parameter of the functions allows to specify the flag value different 
  from the saved in structure.

  The parameters used by matching are defined by the {\bf flag} : \\

 \begin{tabular}{lcccccc}
   flag = & 1  & 1  & 1  & 1  & 1  & 1  \\
          & !  & !  & !  & !  & !  & pe \\
          & !  & !  & !  & !  & ty &    \\
          & !  & !  & !  & tx &    &    \\
          & !  & !  & y  &    &    &    \\
          & !  & x  &    &    &    &    \\
          & z  &    &    &    &    &    \\
 \end{tabular}

 set 1 at the corresponding position then       
 parameter will be taken into account       \\         
 otherwise set 0,                           \\       
 usual value of the flag is 11110 which means   
 that only { x,y,tx,ty } are taken into account  

 {\bf BE SURE that flag is defined !!! }



 @memo MARPLE structure of maximum #&# minimum limits for matching procedure 
*/
struct RcLimit
{
  RcMatchFlag flag;

  HepDouble   x_max;
  HepDouble   y_max;
  HepDouble   z_max;
  HepDouble   tx_max;
  HepDouble   ty_max;
  HepDouble   pe_max;

  HepDouble   x_min;
  HepDouble   y_min;
  HepDouble   z_min;
  HepDouble   tx_min;
  HepDouble   ty_min;
  HepDouble   pe_min;

  // functions set limits and check limits for RcPoint rp
  //    icheck       should have the form described above           
  // if icheck  = -1 then the member flag is used for parameters check 

  ///check and correct parameters par\_max and par\_min in dependence of rp
  void set(     const RcPoint & rp, 
		int icheck = -1); 

  ///return 1 if parameters of rp  in Limits par\_min , par\_max
  int valid(    const RcPoint & rp, 
		int icheck = -1) const; 

};

ostream & operator << (ostream &, const RcLimit &);

#endif
