#ifndef _RCRPNTPOINT_HH_
#define _RCRPNTPOINT_HH_
//-----------------------------------------------------------------------
// Created: 26.07.2000,   Olya Igonkina

// MARPLE class of Points for Reconstruction which have link to RPNT bank

//-----------------------------------------------------------------------

#include <iostream.h>
#include "marple/RcPoint.hh"

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

/** class with link to RPNT bank.
    Class is inherited from RcPoint.
 */
class RcRpntPoint : public RcPoint
{

public:
  ///                 default constructor
  inline RcRpntPoint(); 
  ///                 constructor with link to RPNT bank    
  inline RcRpntPoint( const RcPoint &rp, 
		      const ArtePointer <RPNT> rpntLink = 0 );
  /// constructor from RPNT bank entry, if any diagonal element of covariant matrix equal 0 it is set to infinity 
  RcRpntPoint( const ArtePointer <RPNT> rprntLink);
  
  ///                 copy constructor
  inline RcRpntPoint( const RcRpntPoint &rp ); 

  ///                 destructor
  inline ~RcRpntPoint(); 

  ///                 return link to RPNT table
  inline ArtePointer <RPNT>  link() const; 

  ///                 set link to RPNT table
  inline void  setlink( const ArtePointer <RPNT> rprntLink = 0 );
  
  ///                 return relation with MTRA, which is all MTRA assigned to RSEG, if none then MTRA from first RTRA connected
  const std::vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 return relation with HITB , which is all HITB assigned to RSEG, if none then HITB from first RTRA connected
  const std::vector<ArtePointer<HITB> > hitbList() const; 

  /// put RcRpntPoint information to RPNT bank
  void updateARTE( int & iErr ) const;

  ///                 assignment operator
  inline RcRpntPoint & operator = (const RcRpntPoint &rp);
  ///                 assignment operator
  inline RcRpntPoint & operator = (const RcPoint &rp); 

private:
  ///                 ArtePointer to RPNT bank
  ArtePointer  <RPNT> fRpntLink;
};

//                   ostream operator to print RcRpntPoint information
ostream & operator << (ostream &, const RcRpntPoint &);

#include "marple/RcRpntPoint.icc"

#endif

