/************************************************************************
*
* MuHitInfo.h
* 
* first author : Jing-Ge Shiu 
* purpose      : head file for MuHitinfo.C
* includ file  : MuHitInfo.h, jsMuPixTool.h
* start date   : May. 09, 2000
*
*
*-----------------------------------------------------------------------
*
*/
// include ARTE head files for new C++ interface    
#include "arte/arte.hh"        
#include "arte/RSEG_cmp.h"
#include "auto/auto.hh"

#include "murec/jsMuPixTool.h"

typedef unsigned char  SWord; 
typedef unsigned short  Word;
typedef unsigned int   LWord;

#define MAXZPOS 38
// definition for inputerr
// low  byte for input ok
// high byte for input error
#define NOINPUT          0x0000
#define OKHITBINPUT      0x0001
#define OKGEDEINPUT      0x0002
#define OKXYZINPUT       0x0004
#define NOTOKHITBINPUT   0x0100
#define NOTOKGEDEINPUT   0x0200
#define NOTOKXYZINPUT    0x0400
#define MAXOKFLAG        0x00FF


class MuHitInfo {

public :
  MuHitInfo ();
  MuHitInfo (const MuHitInfo &) {};    // no copy method
  ~MuHitInfo () {} ;
 
  Word putmuhit ( float,float,float ); // input x,y,z
  Word putmuhit ( ArtePointer<HITB> ); // input hitbid
  Word putmuhit ( ArtePointer<GEDE>, int&, int&   ); // input gede and two int parameters
  Word putmuhit ( int, int&, int&   ); // input gedeid and two int parameters

  
  Word muhitok() { return inputerr;}
  int superl()    { return sl;};
  int dsubl()     { return dsub;};
  int rsubl()     { return rsub;};
  int readtype()  { return rtype;};
  int hitbcmp()   { return cmp;};
  int rsegcmp()   { return reccmp;};
  int chamberid() { return dno;};
  int channelid() { return rno;};
  int frontback() { return sd;};
  int xside()     { return xs;}
  int topbottom() { return updown;};
  int sec()       { return secno; };
  int pixxid()    { return rtype>0?pixx:-1;}; 
  int pixyid()    { return rtype>0?pixy:-1;}; 
  int zlay()      { return zlayer; }; 
  ArtePointer<HITB> hitb() { return hitbid;};
  ArtePointer<GEDE> gede() { return gedeid;};
  ArtePointer<GESL> gesl() { return geslid;}; 
  float x()       { return hitx;};
  float y()       { return hity;};
  float z()       { return hitz;};
  float angle()   { return rotangle;} 
  char* dname()   { return chname;}; 
  float getmuzpos(int zi) {
                    if (zno==0) getzpos();
                    if (zno==0||zi<1||zi>MAXZPOS) return 0.;
                    else return muzpos[zi-1];
                  }
  int  getzlayer(float);
  void printzpos();

protected:
  void getzpos();

private:
  int   zno;
  float muzpos[MAXZPOS];
  float hitx, hity, hitz;
  float rot[9],rotangle;
  int   cmp, reccmp, rtype, sl, dsub, rsub, sd, updown, dno, rno; 
  int   xs, secno, pixx, pixy, zlayer, fedno;
  char  chname[12];
  ArtePointer<HITB> hitbid;
  ArtePointer<GEDE> gedeid;
  ArtePointer<GESL> geslid;
  /* *********************************
     cmp    : readout component
     reccmp : cmp id for RSEG
     rtype  : readout type (0 tube, 1 pad, 2 pixel)
     sl     : superlayer (1..4)        
     dsub   : detector layer defined in ARTE (1..4 for sl=1,2  1..2 for sl=3,4)
     rsub   : readout sublayer defined by Robert (1..6)
     sd     : single layer (sector) id for double layer chambers (0,1)
     updown : top or bottom chamber (0,1)
     dno    : chamber id in the readout sublayer 
     rno    : readout channel id  
     gede   : gede id
     secno  : sector id (tube/pad : cable id; pixel : ASD card id)
     zlayer : z layer id for reconstruction layer(1...38)
     xs     : x side, +1 or -1
     pixx   : x coordinate for pixel chamber readout group (starting from 1), column no for pad readout (1-66)
     pixy   : y coordinate for pixel chamber readout group (starting from 1), row no for pad readout (1-60)
     hitb   : hitb id
     gesl   : gesl id
     ********************************* */
  Word inputerr; 
  void set2ini(void);
  int  getdn();
  int  getrn();  
  int  getrn(Word, Word);  
  MXPIX mupixinfo;

};










