/* This file defines IstreamMenu: it is derived from InteractiveMenu 
   and contains parser (via PtrWrapper).

    IstreamMenu {
      PtrWrapper<InteractiveParser> parser;
      void run_one_command(istream* is);
      void operator()(istream* is);
    };
   To start IstreamMenu menu; use operator()(istream* is);
   it goes up to the end of the menu. If parser returns EOF_MESSAGE in
   global_error() - global_error() is cleared. 
   To run only one command: use run_one_command(istream* is);
*/
   
// Author V.Balagura (balagura@mail.desy.de)   (23.04.98).

#ifndef _istream_menu_hh
#define _istream_menu_hh 1

#include "ptset/interactive.hh"
#include "ptset/interactive_parser.hh"

/* -------------------- IstreamMenu_1Par -------------------- */

class IstreamMenu : public InteractiveMenu {
 public:
  PtrWrapper<InteractiveParser> parser;
  void run_one_command(istream* is);
  void operator()(istream* is);
  /* gcc 2.7.2.3.f.1 compiler bug:
     void operator()(istream* is) shadows 
     PtrWrapper<InteractiveItem>& operator()(const string& s) from parent
     class InteractiveMenu. To get around this define it once more: */
  PtrWrapper<InteractiveItem>& operator()(const string& s) { 
    return InteractiveMenu::operator()(s); 
  }
};
inline void IstreamMenu::run_one_command(istream* is) {
  InteractiveParser::Output input = parser()->get_input(is);
  if ( global_error() ) return;
  InteractiveMenu::operator()(*input.commands, input.arguments);
}
inline void IstreamMenu::operator()(istream* is) {
  do { run_one_command(is); } while ( !global_error() );
  if ( global_error()() == InteractiveParser::EOF_MESSAGE )
    global_error().clear();
}

#endif
