/* see interactive_parser.hh for details */

// Author V.Balagura (balagura@mail.desy.de)   (24.04.98).

#include "ptset/interactive_parser.hh"

string InteractiveParser::EOF_MESSAGE("end of file reached");

void ComArg_InteractiveParser::clear_output() {
  coms.erase(coms.begin(), coms.end());
  while ( args.get() != EOF );
  args.clear();
}
void ComArg_InteractiveParser::print_error(const string& message,
					   string* add){
  global_error()(message);
  if ( add ) global_error() << *add;
  for ( list<string>::reverse_iterator iter=coms.rbegin();
	iter!=coms.rend(); ++iter ) global_error() << *iter;
  global_error() << string("---------- parser error ----------");
}

char InteractiveParser_WithDelimiters::get_coms
(istream* is, char com_delimiter, bool (*arg_delimiter)(char)) {
  char c;
  string s;
  *is >> ws;
  if ( !is->get(c) ) {
    global_error()(EOF_MESSAGE);
    return 0;
  }
  is->putback(c);
  if ( c != com_delimiter ) {
    getline(*is, s);
    print_error(string("command does not start from \"") +
		com_delimiter + string("\" sign"), &s);
    return 0;
  }
  while ( 1 ) {
    if ( !is->get(c) ) {
      print_error("end of file reached inside command", 0);
      return 0;
    }
    else if ( arg_delimiter(c) || c == com_delimiter ) {
      if ( !coms.empty() && coms.back().length() == 0 ) {
	coms.erase(--coms.end());
	is->putback(c);
	getline(*is, s);
	s += com_delimiter;
	print_error(string("empty command - no command after \"") +
		    com_delimiter + string("\" sign"), &s);
	return 0;
      }
      if ( c == com_delimiter ) coms.push_back(string());
      else return c; // arg_delimiter(c) == true
    }
    else coms.back() += c;
  }
}

void SlashLine_InteractiveParser::fill_output(istream* is) {
  clear_output();
  char c = get_coms(is, '/', is_space);
  if ( !global_error() ) {
    if ( c == '\n' ) return; 
    /* special case like: 
       /command
       /next_command
       here /command is followed by '\n'
       '\n' serves as delimiter between coms and args => it is eated by
       get_coms.
       If the staff after delimiter is always considered as args:
       /next_command serves as args. To destinguish this special case:
       get_coms(..) returns char (delimiter) which is checked here */
    int i;
    while (1) {     // skip white, non '\n'; if '\n': return
      if ( (i=is->peek()) == EOF ) return; // *is state remains good
      c = char(i);
      if ( is_space(c) ) {
	is->ignore();
	if ( c == '\n' ) return; // '\n' is taken from *is
	else continue;
      }
      else break;  // last symbol c remains in *is
    }

    // rest of line is not empty:
    string s;
    getline(*is, s);
    args << s;  // if not empty, args start from non white character
  }
}
void SlashBlock_InteractiveParser::fill_output(istream* is) {
  clear_output();
  get_coms(is, '/', is_left_bracket);
  if ( global_error() ) return;
  char c;
  int level = 1;
  while ( 1 ) {
    if ( !is->get(c) ) {
      string s;
      while ( args.get(c) ) s += c;
      print_error("end of file reached inside command argument block", &s);
      return;
    }
    else if ( c == '}' ) {
      --level;
      if ( level == 0 ) return;
    }
    else if ( c == '{' ) ++level;
    
    args.put(c);
  }
}
