// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin          
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//  Modified:
//  few comments about hits check                        18/08/00              
#include <iostream>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/State.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/ScatThick.hh"
#include "rang/Geom.hh"
#include "rang/zfpar.hh"
#include "rang/utfeld.hh"
#include "rang/RadEl.hh"

//-- Constructor
rangerHit::rangerHit() {
  cout << "This is the rangerHit default constructor" << endl;
  hitb = 0;
  vol = 0;
  sPath = 1;
};

rangerHit::rangerHit(ArtePointer<RHIT> rhit, rangerFitMode fm) 
: rangerHitBase(rhit,fm) {
#ifdef FULL
  cout << "This is the rangerHit(rhit) constructor, hitb=" 
       << rhit->hitb << endl;
#endif
  z = rhit->hitb->z;
  hitb = rhit->hitb;  
  ArtePointer<GEDE> pl = hitb->gede;
  vol = rangerGeom::vol( pl );
  sPath = sqrt(1 + rhit->tx * rhit->tx + rhit->ty * rhit->ty);
  if (sPath >=0 && sPath < 3);  // protection against unset rhit parms
  else sPath = 1;
  
//  cout << "..............................................done" << endl;
};

rangerHit::rangerHit(ArtePointer<HITB> hb, rangerFitMode fm) 
: rangerHitBase(hb,fm) {
#ifdef FULL
  cout << "This is the rangerHit(hitb) constructor, hitb=" 
       << hb << endl;
#endif
  z = hb->z;
  hitb = hb;  
  ArtePointer<GEDE> pl = hb->gede;
  vol = rangerGeom::vol( pl );
  sPath = 1;
};

rangerHit::~rangerHit(){
};


float rangerHit::dz() {
  ArtePointer<GEDE> pl = hitb->gede;
  return (pl->zmax - pl->zmin);
}

int rangerHit::plane() {
  ArtePointer<GEDE> pl = hitb->gede;
  return pl.index();
}

void rangerHit::storeRhit(ArtePointer<RHIT> rhit) {
  if (hitb == rhit->hitb) {
    state->storeRhit(rhit);
    // we should also set the quantity... 
    
    // WH: strange ... we need these checks
    //if(m && r && m->num_row()>0 && r->num_row()>0) 
    // rhit->q = (*m)(1) - (*r)(1) ;
    // else
    // rhit->q = 0;
  } else {
    cout << "rangerHit::storeRhit: disagreement between original hit "
	 << getHitb()<< " and hit " << rhit->hitb << " related to rhit "
	 << rhit << endl;
  }
}
