// 
//  Author:  Rainer Mankel                     Created:  09-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include <algo.h>
#include <math.h>
#include "rang/NavItem.hh"
#include "rang/Domain.hh"
#include "rang/DomWin.hh"
#include "rang/SNM.hh"
#include "rang/SNMList.hh"



rangerNavItem::rangerNavItem(rangerDomWin* dw) {
  wi = dw;
}


void rangerNavItem::setNavTable(const vector<rangerDomain*>::iterator dbeg,
				const vector<rangerDomain*>::iterator dend,
				const rangerSNMList& snl,
				const int minScope,
				const int maxScope,
				const int maxInt){

  vector<rangerDomain*>::iterator d0 = find(dbeg,dend,wi->dom());
  
  if (d0 < dbeg || d0 >= dend) return;

//   cout << " found domain " << (*d0)->nam() << endl;

  // locate corresponding SNM
  const rangerSNM* sn = snl.getSNM(wi->cmp(),wi->sec());
  if (! sn) return;

//   cout << " found sn for cmp/sec" << wi->cmp() << "/" << wi->sec() << endl; 
//   sn->dump(2);
  
  // loop over target sectors
  for (vector<pair<int,int> >::const_iterator ti=sn->neigh().begin();
       ti != sn->neigh().end(); ++ti) {
//     cout << "new search sector " << (*ti).first << "/" << (*ti).second << endl;

    // forward navigation table
    vector<rangerDomain*>::iterator df = d0;
    ++df;

    for (vector<rangerDomain*>::const_iterator dmi=df; dmi != dend; ++dmi) {

      // check number of domain steps
      int dist = dmi - df;
      if (fabs(dist) > maxInt) continue;

      // check domain scope
      if ((*dmi)->sco() < minScope || (*dmi)->sco() > maxScope) continue;

      // locate the cmp/sec
//       cout << "forward check in dom=" << (*dmi)->nam() << endl;

      int nFound = 0;
      for (vector <rangerDomWin*>::iterator dwi=(*dmi)->win().begin();
	   dwi != (*dmi)->win().end(); ++dwi) {
	if ((*dwi)->cmp() == (*ti).first && (*dwi)->sec() == (*ti).second) {
	  if ((*dwi)->getNI()) {
	      forwardTargets.push_back((*dwi)->getNI());
	      ++nFound;
	      break;
	  }
	}
      }
      if (nFound) break;
    }


    // backward navigation table
    vector<rangerDomain*>::iterator db = d0;
    --db;

    for (vector<rangerDomain*>::iterator dmi=db; dmi != dbeg-1; --dmi) {

      // check number of domain steps
      int dist = dmi - db;
      if (fabs(dist) > maxInt) continue;

      // check domain scope
      if ((*dmi)->sco() < minScope || (*dmi)->sco() > maxScope) continue;

      // locate the cmp/sec
//       cout << "backward check in dom=" << (*dmi)->nam() << endl;

      int nFound = 0;
      for (vector <rangerDomWin*>::iterator dwi=(*dmi)->win().begin();
	   dwi != (*dmi)->win().end(); ++dwi) {
	if ((*dwi)->cmp() == (*ti).first && (*dwi)->sec() == (*ti).second) {
	  if ((*dwi)->getNI()) {
	      backwardTargets.push_back((*dwi)->getNI());
	      ++nFound;
	      break;
	  }
	}
      }
      if (nFound) break;
    }
  }
}


void rangerNavItem::dump(const int level){
  cout << "    NavItem wicmp=" << wi->cmp()
       << " wisec=" << wi->sec()
       << " dom " << wi->dom()->nam()
       << " forward targets " << forwardTargets.size()
       << " backward targets " << backwardTargets.size() << endl;
  if (level >= 2) {
    for (vector<rangerNavItem*>::const_iterator ni=forwardTargets.begin();
	 ni != forwardTargets.end(); ++ni) {
      cout << "     FwdTarget win dom/cmp/sec="
	   << (*ni)->win()->dom()->nam() << " "
	   << (*ni)->win()->cmp() << "/" << (*ni)->win()->sec() << endl;
    }
    for (vector<rangerNavItem*>::const_iterator ni=backwardTargets.begin();
	 ni != backwardTargets.end(); ++ni) {
      cout << "     BwdTarget win dom/cmp/sec="
	   << (*ni)->win()->dom()->nam() << " "
	   << (*ni)->win()->cmp() << "/" << (*ni)->win()->sec() << endl;
    }
  }
}
