
// 
//  Author:  Rainer Mankel                     Created:  14-Oct-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
//  changed: A.Spiridonov                      25-Jan-2000
//           spiridion@ifh.de                  HERA-B Collaboration
//
// check if treated segments was produced by ranger   17/09/00  A. Spiridonov
// new UniqS for slt SLT seeded segments              05/11/00  A. Spiridonov

#include <iostream.h>
#include <math.h>
#include <vector.h>
#include <list.h>
#include <set.h>
#include <algo.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "rang/zfpar.hh"

class rangerSegment {

public:
  rangerSegment() {
    chi2 = 0;
    killFlag = 0;
    pcFlag = 0;
  }
  void insertRseg(const ArtePointer<RSEG> rseg){
    for ( ArteRelationIterator<HITB> hitbIt = rseg->hitb.begin();
	  hitbIt != rseg->hitb.end(); ++hitbIt) {
      ArtePointer<HITB> hitb(*hitbIt);
      hitSet.insert(hitb.index());
    }
    chi2 += rseg->chi;
    rsegList.push_back(rseg);
    // test if PC segment
      if( (rseg->fit >= 0)
         && ( rseg->cmp & Rsegc::patt || rseg->cmp & Rsegc::magt)   
         && !(rseg->cmp & Rsegc::bitsign) ) {
	pcFlag = 1;

	//    cout << " rseg " << rseg.index() << " fit " << (rseg->fit >= 0)
	//	 << " patt " << (rseg->cmp & Rsegc::patt)
	//	 << " magt " << (rseg->cmp & Rsegc::magt) 
	//	 << " bit " << !(rseg->cmp & Rsegc::bitsign) << endl;
	//	cout << " PC segment " << endl;

      }
  };

int nHits() const { return hitSet.size(); };
  float chiSq() const { return chi2; }
  float quality() const { 
    return (nHits() - chi2); 
  };

  void kill() { 
    killFlag = 1;
    for (list<ArtePointer<RSEG> >::iterator rsIt=rsegList.begin();
	 rsIt != rsegList.end(); rsIt++) {
      (*rsIt)->cmp = (*rsIt)->cmp | Rsegc::bitsign;  // set sign bit
    }
  };

  int killed() const { return killFlag; };
  int pcflagged() const { return pcFlag; };

  list<ArtePointer<RSEG> > rsegList;
  set<int, less<int> > hitSet;
  float chi2;
  int killFlag;
  int pcFlag;
  friend int operator<(const rangerSegment& s1,const rangerSegment& s2);
};

int operator<(const rangerSegment& s1,const rangerSegment& s2){
  if (fabs(s1.nHits() - s2.nHits()) > 4) {
    return (s1.nHits() < s2.nHits());
  }
  else {
    float q1 = s1.nHits() - s1.chiSq();
    float q2 = s2.nHits() - s2.chiSq();
    return (q1 < q2);
  }
};


//int findValidEarlierRseg(ArtePointer<RSEG> rseg) { return 0; };


int findValidEarlierRseg(ArtePointer<RSEG> rseg) {
  int earlier = 0;
//for (ArteRelationIterator<RSEG_RBEG> rIt =rseg->rbeg;
//       rIt.valid(); rIt.next()) {
  for (ArteRelationIterator<RSEG> rIt =rseg->rbeg.begin();
       rIt != rseg->rbeg.end(); ++rIt) {
//    ArtePointer<RSEG> rsRel( rIt );
    ArtePointer<RSEG> rsRel( *rIt );
    if (rsRel.index() < rseg.index()) earlier = rsRel.index();
  }
  return earlier;
}


void rangerUniq() {
  vector<rangerSegment> segVec;
  vector<int> segInd(ArtePointer<GEDE>::nrow(),0);

  for (ArteTable<RSEG>::iterator rsIt=ArteTable<RSEG>::begin(); 
       rsIt != ArteTable<RSEG>::end(); ++rsIt) {
    ArtePointer<RSEG> rseg(*rsIt);
    int rsOld = findValidEarlierRseg(rseg);
    if (rsOld) {  // append to existing rangerSegment
      segVec[segInd[rsOld-1]-1].insertRseg(rseg);
      segInd[rseg.index()-1] = segInd[rsOld-1];
    }
    else {        // create new rangerSegment
      rangerSegment rS;
      rS.insertRseg(rseg);
      segVec.push_back(rS);
      segInd[rseg.index()-1] = segVec.size();
    }
  }

  // output
  for (vector<rangerSegment>::iterator sIt=segVec.begin();
       sIt != segVec.end(); sIt++) {
  }

  for (vector<rangerSegment>::iterator sIt1=segVec.begin();
       sIt1 != segVec.end(); sIt1++) {
    if ((*sIt1).killed()) continue;
//    cout << "===First segment==== ";
//    for (list<ArtePointer<RSEG> >::iterator iT=((*sIt1).rsegList).begin();
//	 iT != ((*sIt1).rsegList).end(); iT++) {
//      cout << "rseg=" << (*iT);
//      for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	    mtraIt.valid(); mtraIt.next() ) {
//	ArtePointer<MTRA> mtra(mtraIt);
//	cout << " mtra=" << mtra;
//      }
//    }
//    cout << endl;
    vector<rangerSegment>::iterator sNext = sIt1;
    sNext++;
    for (vector<rangerSegment>::iterator sIt2=sNext;
       sIt2 != segVec.end(); sIt2++) {
      if ((*sIt2).killed()) continue;
//      cout << "======Second segment== ";
//      for (list<ArtePointer<RSEG> >::iterator iT=((*sIt2).rsegList).begin();
//	   iT != ((*sIt2).rsegList).end(); iT++) {
//	cout << "rseg=" << (*iT);
//	for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	      mtraIt.valid(); mtraIt.next() ) {
//	  ArtePointer<MTRA> mtra(mtraIt);
//	  cout << " mtra=" << mtra;
//	}
//      }
 //     cout << endl;
      

//      set_intersection(((*sIt1).hitSet).begin(),
//		       ((*sIt1).hitSet).end(),
//		       ((*sIt2).hitSet).begin(),
//		       ((*sIt2).hitSet).end(),
//		       ostream_iterator<const int> (cout, " "));
      
//      cout << endl;

      set<int, less<int> > hitIntersect;
      set_intersection(((*sIt1).hitSet).begin(),
		       ((*sIt1).hitSet).end(),
		       ((*sIt2).hitSet).begin(),
		       ((*sIt2).hitSet).end(),
		       inserter(hitIntersect,hitIntersect.begin()));
      float frac1 = ((float) hitIntersect.size())/ (*sIt1).nHits();
      float frac2 = ((float) hitIntersect.size())/ (*sIt2).nHits();
      frac2 = frac2 + frac1 - frac1;
//      cout << "Intersect size=" << hitIntersect.size() 
//	   << " frac1=" << ((float) hitIntersect.size())/ (*sIt1).nHits()
//	   << " frac2=" << ((float) hitIntersect.size())/ (*sIt2).nHits()
//	   << endl;
      // $$$$$$$$$$$$$$$$$$$$$$$$$$
      //      if (frac1 > .5 && frac2 > .5) {
      if((int) hitIntersect.size() > 2) {

	//	if ((*sIt1).quality() > (*sIt2).quality()) {
	if ((*sIt1) > (*sIt2)) {
	  (*sIt2).kill();
//	  cout << " killing segment 2" << endl;
	}
	else {
	  (*sIt1).kill();
//	  cout << " killing segment 2 and break" << endl;
	  break;    // quit loop over second segment
	}
      }
    }
  }
}    


void rangerUniqP() {
  //                             PC segments are cheked
  vector<rangerSegment> segVec;
  //  vector<int> segInd;

  for (ArteTable<RSEG>::iterator rsIt=ArteTable<RSEG>::begin(); 
       rsIt != ArteTable<RSEG>::end(); ++rsIt) {

    // check if ranger rseg

    //if( (*rsIt)->cmp & Rsegc::bitsign )  continue;
    //if( (*rsIt)->fit != 0  ) continue;

    if( (*rsIt)->cmp & Rsegc::bitsign || (*rsIt)->fit != 0
	|| !( (*rsIt)->cmp & Rsegc::patt 
	      || (*rsIt)->cmp & Rsegc::magt ) )  continue;

    if( ((*rsIt)->cmp & Rsegc::slt) && zfpar_.sltclone ) continue;  

    ArtePointer<RSEG> rseg(*rsIt);
    rangerSegment rS;
    rS.insertRseg(rseg);
    segVec.push_back(rS);
    //    segInd.push_back(segInd.size()+1);
  }

  // output
  //for (vector<rangerSegment>::iterator sIt=segVec.begin();
  //     sIt != segVec.end(); sIt++) {
  //}

  for (vector<rangerSegment>::iterator sIt1=segVec.begin();
       sIt1 != segVec.end(); sIt1++) {

    //    cout << "===First segment==== killed " << (*sIt1).killed()
    //	 << " pcflag " << (*sIt1).pcflagged() << endl;

    if ((*sIt1).killed() || !(*sIt1).pcflagged()) continue;
    vector<rangerSegment>::iterator sNext = sIt1;
    sNext++;
    if(sNext == segVec.end()) continue;

//    cout << "===First segment==== ";
//    for (list<ArtePointer<RSEG> >::iterator iT=((*sIt1).rsegList).begin();
//	 iT != ((*sIt1).rsegList).end(); iT++) {
//      cout << "rseg=" << (*iT);
//      for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	    mtraIt.valid(); mtraIt.next() ) {
//	ArtePointer<MTRA> mtra(mtraIt);
//	cout << " mtra=" << mtra;
//      }
//    }
//    cout << endl;

    for (vector<rangerSegment>::iterator sIt2=sNext;
       sIt2 != segVec.end(); sIt2++) {
      if ((*sIt2).killed() || !(*sIt2).pcflagged()) continue;
//      cout << "======Second segment== ";
//      for (list<ArtePointer<RSEG> >::iterator iT=((*sIt2).rsegList).begin();
//	   iT != ((*sIt2).rsegList).end(); iT++) {
//	cout << "rseg=" << (*iT);
//	for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	      mtraIt.valid(); mtraIt.next() ) {
//	  ArtePointer<MTRA> mtra(mtraIt);
//	  cout << " mtra=" << mtra;
//	}
//      }
 //     cout << endl;
      

//      set_intersection(((*sIt1).hitSet).begin(),
//		       ((*sIt1).hitSet).end(),
//		       ((*sIt2).hitSet).begin(),
//		       ((*sIt2).hitSet).end(),
//		       ostream_iterator<const int> (cout, " "));
      
//      cout << endl;

      set<int, less<int> > hitIntersect;
      set_intersection(((*sIt1).hitSet).begin(),
		       ((*sIt1).hitSet).end(),
		       ((*sIt2).hitSet).begin(),
		       ((*sIt2).hitSet).end(),
		       inserter(hitIntersect,hitIntersect.begin()));
      float frac1 = ((float) hitIntersect.size())/ (*sIt1).nHits();
      float frac2 = ((float) hitIntersect.size())/ (*sIt2).nHits();
      frac2 = frac2 + frac1 - frac1; 
//      cout << "Intersect size=" << hitIntersect.size() 
//	   << " frac1=" << ((float) hitIntersect.size())/ (*sIt1).nHits()
//	   << " frac2=" << ((float) hitIntersect.size())/ (*sIt2).nHits()
//	   << endl;
      // $$$$$$$$$$$$$$$$$$$$$$$$$$
      //if (frac1 > .5 && frac2 > .5) {
      if((int) hitIntersect.size() > 2) {

	//	if ((*sIt1).quality() > (*sIt2).quality()) {
	if ((*sIt1) > (*sIt2)) {
	  (*sIt2).kill();
//	  cout << " killing segment 2" << endl;
	}
	else {
	  (*sIt1).kill();
//	  cout << " killing segment 2 and break" << endl;
	  break;    // quit loop over second segment
	}
      }
    }
  }
}        // end of UniqP    


void rangerUniqA() {
  //                             All ranger segments are cheked
  vector<rangerSegment> segVec;
  vector<int> segInd;

  for (ArteTable<RSEG>::iterator rsIt=ArteTable<RSEG>::begin(); 
       rsIt != ArteTable<RSEG>::end(); ++rsIt) {

    // check if ranger rseg

    //if( (*rsIt)->cmp & Rsegc::bitsign )  continue;
    //if( !((*rsIt)->cmp & Rsegc::slt) )  continue;
    //if( (*rsIt)->fit != 0  ) continue;

    if( (*rsIt)->cmp & Rsegc::bitsign || (*rsIt)->fit != 0
	|| !( (*rsIt)->cmp & Rsegc::patt || (*rsIt)->cmp & Rsegc::trit 
	      || (*rsIt)->cmp & Rsegc::magt ) )  continue;

    if( ((*rsIt)->cmp & Rsegc::slt) && zfpar_.sltclone ) continue; 

    ArtePointer<RSEG> rseg(*rsIt);
    int rsOld = findValidEarlierRseg(rseg);


    int segInd_rsOld = -1;
    if (rsOld) {  // check rsOld

      for(int i=0; i < segInd.size(); ++i)
	{

	  //cout << " UniqA.C i " << i << " segInd[i] " << segInd[i] << endl;

	  if( segInd[i] == rsOld) { segInd_rsOld = i;break;}
	}
    }

    //cout << "UniqA.C : rseg will be checked  " << *rsIt 
    //	 << " rsOld " << rsOld  
    //	 << " segInd_rsOld " << segInd_rsOld << endl; 

    if (rsOld && segInd_rsOld > -1 ) {  // append to existing rangerSegment

      //cout << " UniqA.C uppend : segInd_rsOld " << segInd_rsOld << endl; 

      segVec[segInd_rsOld].insertRseg(rseg);

      //      segInd.push_back(segInd.size()+1); 
      //      segInd.push_back(segInd[rsOld-1]);  

      segInd[segInd_rsOld] = rseg.index();

      //cout << " UniqA.C after insert segInd[segInd_rsOld] "
      //<< segInd[segInd_rsOld] << endl;
        
   }
    else {        // create new rangerSegment
      //cout << " Uniq.C  : create new " << endl;
      rangerSegment rS;
      rS.insertRseg(rseg);
      segVec.push_back(rS);
      //      segInd.push_back(segInd.size()+1);
      segInd.push_back(rseg.index());

      //cout << " UniqA.C  : create new OK segInd.size() " << segInd.size() 
      //	   << " (rseg.index() " << rseg.index() << endl;
    }
  }

  // output
  //  for (vector<rangerSegment>::iterator sIt=segVec.begin();
  //     sIt != segVec.end(); sIt++) {
  //}

  for (vector<rangerSegment>::iterator sIt1=segVec.begin();
       sIt1 != segVec.end(); sIt1++) {
    if ((*sIt1).killed()) continue;
    vector<rangerSegment>::iterator sNext = sIt1;
    sNext++;
    if(sNext == segVec.end()) continue;

//    cout << "===First segment==== ";
//    for (list<ArtePointer<RSEG> >::iterator iT=((*sIt1).rsegList).begin();
//	 iT != ((*sIt1).rsegList).end(); iT++) {
//      cout << "rseg=" << (*iT);
//      for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	    mtraIt.valid(); mtraIt.next() ) {
//	ArtePointer<MTRA> mtra(mtraIt);
//	cout << " mtra=" << mtra;
//      }
//    }
//    cout << endl;

    for (vector<rangerSegment>::iterator sIt2=sNext;
       sIt2 != segVec.end(); sIt2++) {
      if ((*sIt2).killed()) continue;
//      cout << "======Second segment== ";
//      for (list<ArtePointer<RSEG> >::iterator iT=((*sIt2).rsegList).begin();
//	   iT != ((*sIt2).rsegList).end(); iT++) {
//	cout << "rseg=" << (*iT);
//	for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	      mtraIt.valid(); mtraIt.next() ) {
//	  ArtePointer<MTRA> mtra(mtraIt);
//	  cout << " mtra=" << mtra;
//	}
//      }
 //     cout << endl;
      

//      set_intersection(((*sIt1).hitSet).begin(),
//		       ((*sIt1).hitSet).end(),
//		       ((*sIt2).hitSet).begin(),
//		       ((*sIt2).hitSet).end(),
//		       ostream_iterator<const int> (cout, " "));
      
//      cout << endl;

      set<int, less<int> > hitIntersect;
      set_intersection(((*sIt1).hitSet).begin(),
		       ((*sIt1).hitSet).end(),
		       ((*sIt2).hitSet).begin(),
		       ((*sIt2).hitSet).end(),
		       inserter(hitIntersect,hitIntersect.begin()));
      float frac1 = ((float) hitIntersect.size())/ (*sIt1).nHits();
      float frac2 = ((float) hitIntersect.size())/ (*sIt2).nHits();
      frac2 = frac2 + frac1 - frac1; 
//      cout << "Intersect size=" << hitIntersect.size() 
//	   << " frac1=" << ((float) hitIntersect.size())/ (*sIt1).nHits()
//	   << " frac2=" << ((float) hitIntersect.size())/ (*sIt2).nHits()
//	   << endl;
      // $$$$$$$$$$$$$$$$$$$$$$$$$$
      //      if (frac1 > .5 && frac2 > .5) {
      if((int) hitIntersect.size() > 2) {

	//	if ((*sIt1).quality() > (*sIt2).quality()) {
	if ((*sIt1) > (*sIt2)) {
	  (*sIt2).kill();
//	  cout << " killing segment 2" << endl;
	}
	else {
	  (*sIt1).kill();
//	  cout << " killing segment 2 and break" << endl;
	  break;    // quit loop over second segment
	}
      }
    }
  }

  cout << "UniqA : return " << endl;

}                //         end of UniqA

void rangerUniqS() {
  //                             SLT seeded segments are cheked
  vector<rangerSegment> segVec;
  vector<int> segInd;

  for (ArteTable<RSEG>::iterator rsIt=ArteTable<RSEG>::begin(); 
       rsIt != ArteTable<RSEG>::end(); ++rsIt) {

    // check if ranger rseg

    //if( (*rsIt)->cmp & Rsegc::bitsign )  continue;
    //if( !((*rsIt)->cmp & Rsegc::slt) )  continue;
    //if( (*rsIt)->fit != 0  ) continue;

    if( (*rsIt)->cmp & Rsegc::bitsign || (*rsIt)->fit != 0
	|| !( (*rsIt)->cmp & Rsegc::patt || (*rsIt)->cmp & Rsegc::trit 
	      || (*rsIt)->cmp & Rsegc::magt ) )  continue;

    if( !((*rsIt)->cmp & Rsegc::slt) && zfpar_.sltclone ) continue;

    ArtePointer<RSEG> rseg(*rsIt);
    int rsOld = findValidEarlierRseg(rseg);


    int segInd_rsOld = -1;
    if (rsOld) {  // check rsOld

      for(int i=0; i < segInd.size(); ++i)
	{

	  //cout << " UniqS.C i " << i << " segInd[i] " << segInd[i] << endl;

	  if( segInd[i] == rsOld) { segInd_rsOld = i;break;}
	}
    }

    //cout << "UniqS.C : rseg will be checked  " << *rsIt 
    //	 << " rsOld " << rsOld  
    //	 << " segInd_rsOld " << segInd_rsOld << endl; 

    if (rsOld && segInd_rsOld > -1 ) {  // append to existing rangerSegment

      //cout << " UniqS.C uppend : segInd_rsOld " << segInd_rsOld << endl; 

      segVec[segInd_rsOld].insertRseg(rseg);

      //      segInd.push_back(segInd.size()+1); 
      //      segInd.push_back(segInd[rsOld-1]);  

      segInd[segInd_rsOld] = rseg.index();

      //cout << " UniqS.C after insert segInd[segInd_rsOld] "
      //<< segInd[segInd_rsOld] << endl;
        
   }
    else {        // create new rangerSegment
   
      rangerSegment rS;
      rS.insertRseg(rseg);
      segVec.push_back(rS);
      //      segInd.push_back(segInd.size()+1);
      segInd.push_back(rseg.index());

      //cout << " UniqS.C  : create new OK segInd.size() " << segInd.size() 
      //	   << " (rseg.index() " << rseg.index() << endl;
    }
  }

  // output
  //  for (vector<rangerSegment>::iterator sIt=segVec.begin();
  //     sIt != segVec.end(); sIt++) {
  //}

  for (vector<rangerSegment>::iterator sIt1=segVec.begin();
       sIt1 != segVec.end(); sIt1++) {
    if ((*sIt1).killed()) continue;
    vector<rangerSegment>::iterator sNext = sIt1;
    sNext++;
    if(sNext == segVec.end()) continue;

//    cout << "===First segment==== ";
//    for (list<ArtePointer<RSEG> >::iterator iT=((*sIt1).rsegList).begin();
//	 iT != ((*sIt1).rsegList).end(); iT++) {
//      cout << "rseg=" << (*iT);
//      for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	    mtraIt.valid(); mtraIt.next() ) {
//	ArtePointer<MTRA> mtra(mtraIt);
//	cout << " mtra=" << mtra;
//      }
//    }
//    cout << endl;

    for (vector<rangerSegment>::iterator sIt2=sNext;
       sIt2 != segVec.end(); sIt2++) {
      if ((*sIt2).killed()) continue;
//      cout << "======Second segment== ";
//      for (list<ArtePointer<RSEG> >::iterator iT=((*sIt2).rsegList).begin();
//	   iT != ((*sIt2).rsegList).end(); iT++) {
//	cout << "rseg=" << (*iT);
//	for ( ArteRelationIterator<RSEG_MTRA> mtraIt( (*iT)->mtra );
//	      mtraIt.valid(); mtraIt.next() ) {
//	  ArtePointer<MTRA> mtra(mtraIt);
//	  cout << " mtra=" << mtra;
//	}
//      }
 //     cout << endl;
      

//      set_intersection(((*sIt1).hitSet).begin(),
//		       ((*sIt1).hitSet).end(),
//		       ((*sIt2).hitSet).begin(),
//		       ((*sIt2).hitSet).end(),
//		       ostream_iterator<const int> (cout, " "));
      
//      cout << endl;

      set<int, less<int> > hitIntersect;
      set_intersection(((*sIt1).hitSet).begin(),
		       ((*sIt1).hitSet).end(),
		       ((*sIt2).hitSet).begin(),
		       ((*sIt2).hitSet).end(),
		       inserter(hitIntersect,hitIntersect.begin()));
      float frac1 = ((float) hitIntersect.size())/ (*sIt1).nHits();
      float frac2 = ((float) hitIntersect.size())/ (*sIt2).nHits();
      frac2 = frac2 + frac1 - frac1; 
//      cout << "Intersect size=" << hitIntersect.size() 
//	   << " frac1=" << ((float) hitIntersect.size())/ (*sIt1).nHits()
//	   << " frac2=" << ((float) hitIntersect.size())/ (*sIt2).nHits()
//	   << endl;
      // $$$$$$$$$$$$$$$$$$$$$$$$$$
      //      if (frac1 > .5 && frac2 > .5) {
      if((int) hitIntersect.size() > 2) {

	//	if ((*sIt1).quality() > (*sIt2).quality()) {
	if ((*sIt1) > (*sIt2)) {
	  (*sIt2).kill();
//	  cout << " killing segment 2" << endl;
	}
	else {
	  (*sIt1).kill();
//	  cout << " killing segment 2 and break" << endl;
	  break;    // quit loop over second segment
	}
      }
    }
  }

  cout << "UniqS : return " << endl;

}                //         end of UniqS



extern "C" {
  void rguniq_() { rangerUniq(); }
}

extern "C" void rguniqa_() { rangerUniqA(); }

extern "C" void rguniqp_() { rangerUniqP(); }

extern "C" void rguniqs_() { rangerUniqS(); }
