// 
//  Author:  Rainer Mankel                     Created:  23-Apr-99...
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//  Modified:  A.Spiridonov                    08-Apr-2000 
// 

#define RGHLADD_LOCK

// rghladd_    ( clone of rghl_ )

// Fortran interface to domain hit list of C++ navigator.
// For domain layer pointer given as parameter, the list of domain hits
// is written into a fixed-length array which must be defined in the
// calling program.
// The number of array elements is NOT reset before writing. 
//                                
#include <math.h>
#include "rang/Geom.hh"
#include "rang/DomObj.hh"
#include "rang/DomLay.hh"
#include "rang/HL.hh"
#include "rang/zfpar.hh"
#include "rang/zcut.hh"
#include "rang/rgxseg.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C" {
  void rghladd_(rangerDomLay*& dl,  // pointer to domain layer (input)
	     const int& maxHits, // space available in array (input)
	     ArtePointer<HITB>* hitVec,  // pointer to hit array for output
	     int& nHits) {       // number of hits written into array (output)

    if (!rangerDO) {
      cout << "rghl: rangerDO=" << rangerDO << endl;
      return;
    }

    //    nHits = 0;

    // loop over hits in domain layer
    for (vector<ArtePointer<HITB> >::const_iterator hi=dl->hl()->h().begin();
	 hi != dl->hl()->h().end(); ++hi) {

#ifdef RGHLADD_LOCK
      // remove hit if already used
      if (zfpar_.segstor) {
	if (((*hi)->rseg).begin() != ((*hi)->rseg).end()) continue;
      }
      else {
	if (((*hi)->rtra).begin() != ((*hi)->rtra).end()) continue;
      }
#endif

      // put into array
      if (nHits < maxHits) {
	hitVec[nHits] = (*hi);
	++nHits;
      }
      else {
	static int id1;
	arteMsg << "rghl.C : overflowing nhl, some hits skipped" << endl;
	arteMsg.allp(id1,1);
	break;
      }
    }
  }
}
    

