// 
//  Author:  Rainer Mankel                     Created:  23-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//           Modified: A.Spiridonov            08-Apr-2000  
// 

#define RGHL_LOCK

// rghlpr_

// Fortran interface to domain hit list of C++ navigator.
// For domain layer pointer given as parameter, the list of domain hits
// is written into a fixed-length array which must be defined in the
// calling program.
// Then hits are collected from proxy layers of this domain layer  
// The number of array elements is reset before writing. 

#include <math.h>
#include "rang/Geom.hh"
#include "rang/DomObj.hh"
#include "rang/DomLay.hh"
#include "rang/ProxyLay.hh"
#include "rang/HL.hh"
#include "rang/zfpar.hh"
#include "rang/zcut.hh"
#include "rang/rgxseg.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C"
{
  void rghl_(rangerDomLay* const &, const int&, ArtePointer<HITB>*, int&);
  void rghladd_(rangerDomLay* const &, const int&, ArtePointer<HITB>*, int&);
}

extern "C"  void rghlpr_(rangerDomLay*& dl,  // pointer to domain layer (input)
	      const int& maxHits, // space available in array (input)
	      ArtePointer<HITB>* hitVec,  // pointer to hit array for output
	      int& nHits) {       // number of hits written into array (output)

  if (!rangerDO) {
    cout << "rghl: rangerDO=" << rangerDO << endl;

  }
  
  rghl_(dl,maxHits,hitVec,nHits);

  // loop over proxy layers
  for (vector<rangerProxyLay*>::iterator pli=dl->proxylay().begin();
       pli != dl->proxylay().end(); ++pli) 
    {
      int nHit1 = nHits;
      rghladd_((*pli)->tolay(),maxHits,hitVec,nHits);
      //cout << " ============================================== " << endl;
      //cout << "rghlpr: " << (nHits - nHit1) << " proxy hits added" 
      //    << " to " << nHit1 << " existing hits" << endl;
      //nHits += nHit1;
    }
}

	   
