// --*-C++-*--
#ifndef _RCX_H_
#define _RCX_H_

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) modification of debugging code
// 15 Sep 2000 (DD) refit of decays
// 11 Oct 2000 (DD) keyword size changed 10->20
// 30 Oct 2000 (DD) new branch for vertex following
//
// *****************************************************************************

#include "reco/rcXdata.h"
#include "reco/rcXattribute.h"
#include "reco/rcXvertex.h"
#include "reco/rcXdecay.h"

#include "vector.h"
#include "multimap.h"

#include "CLHEP/Vector/LorentzVector.h"
#include "CLHEP/Vector/LorentzRotation.h"
#include "CLHEP/Vector/ThreeVector.h"

#ifdef  USE_ROOT_DD
#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TTree.h"
#include "TNtuple.h"
#include "TBranch.h"
#include "TDirectory.h"
#endif


class rcX : public datamass, public vertexmass {

  // -- histos, trees, branches
  void              bookXHistos();
#ifdef  USE_ROOT_DD
  TFile             *x_file;
  TTree             *x_tree;
  TH1F              *x_loss;
  TH1F              *x_events;
  TH1F              *x_fills;
#endif
  TRACKER_SEGMENT   *vds;
  TRACKER_SEGMENT   *itr;
  TRACKER_SEGMENT   *otr;
  TRACKER_SEGMENT   *muon;
  TRACK_PARAMETER   *track;
  TRACK_PARAMETER   *mc_track;
  RICH_RING         *rich;
  ECAL_CLUSTER      *ecal;
  ECAL_CLUSTER      *brem;
  TYPE              *type;
  LIKELIHOODS       *lr, *lr_rise, *lr_rite;
  TRACK_PARAMETER   x_par;
  VERTEX            x_vertex;
  VERTEX            mcx_vertex;
  VERTEX            primary_vertex;
  VERTEX            primary_vertex_average;
  VERTEX            mc_primary_vertex;
  TARGET            x_wire;
  HERAB_EVENT       event;

  char              keyword[20], title[100];
  bool              first;
  int               old_evt;
  int               old_total;

  /// -- tracks
  vector<attribute>  *daugh_list;
  vector<ArtePointer<RTRA> >        *rtra_list[10];// maximum daughters!!
  vector<ArtePointer<RTRA> >::iterator it_list[10];// maximum daughters!!
  /// -- find tracks in RTRA 
  int     findTracksRTRA();
  /// -- find tracks in MTRA using MC information (only right tracks)
  int     findTracksMTRA();
  /// -- find tracks from already reconstructed particles
  /// -- for fit=motherfit
  int     findRecoX();
  /// -- description of decay
  decay mother;
  void    countLund();
  /// -- rule for choosing best track
  multimap<float, ArtePointer<RTRA>, greater<float> >           *rtra_all[10];// maximum daughters!!
  multimap<float, ArtePointer<RTRA>, greater<float> >::iterator it_rtra_all;
  /// -- checking segment difference for current combination
  int TrackDiff();
  /// -- clear and copy track information
  void cleartree();
  void copy(int i);
  int  copytree();
  /// -- book arte tables
  void bookRTRA();
  void bookX();
  /// make refit of decays that will be used for analysis 
  int massRefit(ArtePointer<RTRA> t);
  /// -- quality assignment
  float quality(ArtePointer<RTRA> rtra);
  /// -- check of final kinematic cuts
  int   checkkinematics();

 public:
  rcX(char *keyword, char *title, int verbose);
  ~rcX();
  
  // -- Add track
  inline void addDaughter(attribute &a) {
    int clones=a.clone;
    for (int i=1; i<=clones; i++) {
      a.clone=i;
      daugh_list->push_back(a); 
    }
  }
  void printTracks();
  void printDaughters();

  // -- mass reco
  void    rc();
  int     calcmass();
  int     calcvertex();
  inline void setMother(decay &m) { mother=m; }
  int     total;
};


#endif
