
// *****************************************************************************
//
// source:    rc99Akaclass.C
//
// type:      source code
//
// created:   20 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rc99Akaclass
//
// called by: 
//
// purpose:   handle kuip commands /RECO/CLASS
//
// changes:
// 20 Jul 2000 (AG) creation
// 25 Jul 2000 (AG) polished code
// 27 Jul 2000 (AG) improved printout
// 02 Aug 2000 (AG) introduced ECLASS_DVTX, ECLASS_SLTHPT
// 03 Aug 2000 (AG) improved kumac handling
// 03 Aug 2000 (AG) SLTHPT -> SLTCUT
// 04 Aug 2000 (AG) removed bug
// 16 Aug 2000 (AG) introdcued SLTYPS
// 10 Nov 2000 (AG) added classes
// 13 Nov 2000 (AG) introduced cutCAL1HT
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "reco/eclass.hh"

// globals
int classify    = 0; // general
int classJPSIEE = 0; // bit  1
int classJPSIMM = 0; // bit  2
int classFCNC   = 0; // bit  3
int classCAL1HT = 0; // bit  4
int classVZERO  = 0; // bit  5
int classVZERO2 = 0; // bit  6
int classDVTX   = 0; // bit  7
int classSLTYPS = 0; // bit  8
int classSLTCUT = 0; // bit  9
int classSLTEEB = 0; // bit 10
int classSLTEE  = 0; // bit 11
int classSLTMM  = 0; // bit 12
int classSLTDV  = 0; // bit 13
int classTLT    = 0; // bit 14
int classFLTTRG = 1; // bit 15
int classRNDTRG = 1; // bit 16
int classRECYPS = 0; // bit 17
int classRECDV  = 0; // bit 18
int classCLASS19 = 0; // bit 19
int classCLASS20 = 0; // bit 20
int classCLASS21 = 0; // bit 21
int classCLASS22 = 0; // bit 22
int classCLASS23 = 0; // bit 23
int classCLASS24 = 0; // bit 24
int classCLASS25 = 0; // bit 25
int classCLASS26 = 0; // bit 26
int classCLASS27 = 0; // bit 27
int classCLASS28 = 0; // bit 28
int classCLASS29 = 0; // bit 29
int classCLASS30 = 0; // bit 30
int classCLASS31 = 0; // bit 31
int classCLASS32 = 0; // bit 32

// externals
extern int FADEB;

// externals
extern float cutCAL1HT;
extern float cutRECYPS;
extern float cutRECDV;

// prototypes
extern "C" void kugetc_( char*, int *, int );

//
// function code
//
extern "C" void
kaclass_()
{
  ARTE_mark( "kaclass_ 0", 0 );

  //
  // read in class
  //
  int len  = 0;     
  int len1 = 6;
  char input[7];
  memset( input, '\0', 7 ); 
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *cl = (char *) malloc( len+1 );
  memset( cl, '\0', len+1 );
  strncpy( cl, input, len );
  for( int i=0; i<len; i++ ) cl[i] = toupper( cl[i] );
  if( strcmp( cl, "ALL" )    && //  all
      strcmp( cl, "JPSIEE" ) && //  1
      strcmp( cl, "JPSIMM" ) && //  2
      strcmp( cl, "CAL1HT" ) && //  3
      strcmp( cl, "FCNC"   ) && //  4
      strcmp( cl, "VZERO"  ) && //  5
      strcmp( cl, "VZERO2" ) && //  6
      strcmp( cl, "DVTX"   ) && //  7
      strcmp( cl, "SLTYPS" ) && //  8
      strcmp( cl, "SLTCUT" ) && //  9
      strcmp( cl, "SLTEEB" ) && // 10
      strcmp( cl, "SLTEE"  ) && // 11
      strcmp( cl, "SLTMM"  ) && // 12
      strcmp( cl, "SLTDV"  ) && // 13
      strcmp( cl, "TLT"    ) && // 14
      strcmp( cl, "FLTTRG" ) && // 15
      strcmp( cl, "RNDTRG" ) && // 16
      strcmp( cl, "RECYPS" ) && // 17
      strcmp( cl, "RECDV"  ) && // 18
      strcmp( cl, "CLASS19" ) && // 19
      strcmp( cl, "CLASS20" ) && // 20
      strcmp( cl, "CLASS21" ) && // 21
      strcmp( cl, "CLASS22" ) && // 22
      strcmp( cl, "CLASS23" ) && // 23
      strcmp( cl, "CLASS24" ) && // 24
      strcmp( cl, "CLASS25" ) && // 25
      strcmp( cl, "CLASS26" ) && // 26
      strcmp( cl, "CLASS27" ) && // 27
      strcmp( cl, "CLASS28" ) && // 28
      strcmp( cl, "CLASS29" ) && // 29
      strcmp( cl, "CLASS30" ) && // 30
      strcmp( cl, "CLASS31" ) && // 31
      strcmp( cl, "CLASS32" )    // 32
      ){ 
    cout << "RECON/CLASS: class=" << cl << " UNKOWN!" << endl;
    return;
  }
  
  //
  // read in status
  //
  len1 = 7;
  memset( input, '\0', 7 );
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *status  = (char *) malloc( len+1 );
  memset( status, '\0', len+1 );
  strncpy( status, input, len );
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( status[i] );
  if( strcmp( status, "ON"  ) &&
      strcmp( status, "OFF" )    ){
    cout << "RECON/CLASS: status=" << status << " UNKOWN!" << endl;
    return;
  }

  //
  // read in optional parameter
  //
  len1 = 7;
  memset( input, '\0', 7 );
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *para  = (char *) malloc( len+1 );
  memset( para, '\0', len+1 );
  strncpy( para, input, len );
  double parameter = atof( para );
  cout << "RECON/CLASS " << cl << " " << status << " parameter=" << parameter << endl;
    
  //
  // print
  //
  if( FADEB ) cout << "RECON/CLASS: " << cl << " " << status << " parameter=" << parameter << endl;

  //
  // set classification flags
  //
  if( !strcmp( status, "ON" ) ){

    if( !strcmp( cl, "ALL"    ) ){
      classJPSIEE = 1; //  1
      classJPSIMM = 1; //  2
      classFCNC   = 1; //  3
      classCAL1HT = 1; //  4
      classVZERO  = 1; //  5
      classVZERO2 = 1; //  6
      classDVTX   = 1; //  7
      classSLTYPS = 1; //  8
      classSLTCUT = 1; //  9
      classSLTEEB = 1; // 10
      classSLTEE  = 1; // 11
      classSLTMM  = 1; // 12
      classSLTDV  = 1; // 13
      classTLT    = 1; // 14
      classFLTTRG = 1; // 15
      classRNDTRG = 1; // 16
      classRECYPS = 1; // 17
      classRECDV  = 1; // 18
      classCLASS19 = 1; // 19
      classCLASS20 = 1; // 20
      classCLASS21 = 1; // 21
      classCLASS22 = 1; // 22
      classCLASS23 = 1; // 23
      classCLASS24 = 1; // 24
      classCLASS25 = 1; // 25
      classCLASS26 = 1; // 26
      classCLASS27 = 1; // 27
      classCLASS28 = 1; // 28
      classCLASS29 = 1; // 29
      classCLASS30 = 1; // 30
      classCLASS31 = 1; // 31
      classCLASS32 = 1; // 32
    }

    if( !strcmp( cl, "JPSIEE" ) ) classJPSIEE = 1; //  1
    if( !strcmp( cl, "JPSIMM" ) ) classJPSIMM = 1; //  2
    if( !strcmp( cl, "FCNC"   ) ) classFCNC   = 1; //  3
    if( !strcmp( cl, "CAL1HT" ) ){
      classCAL1HT = 1;                             //  4
      if( parameter != -999999 ) cutCAL1HT = parameter;
    }
    if( !strcmp( cl, "VZERO"  ) ) classVZERO  = 1; //  5
    if( !strcmp( cl, "VZERO2" ) ) classVZERO2 = 1; //  6
    if( !strcmp( cl, "DVTX"   ) ) classDVTX   = 1; //  7
    if( !strcmp( cl, "SLTYPS" ) ) classSLTYPS = 1; //  8
    if( !strcmp( cl, "SLTCUT" ) ) classSLTCUT = 1; //  9
    if( !strcmp( cl, "SLTEEB" ) ) classSLTEEB = 1; // 10
    if( !strcmp( cl, "SLTEE"  ) ) classSLTEE  = 1; // 11
    if( !strcmp( cl, "SLTMM"  ) ) classSLTMM  = 1; // 12
    if( !strcmp( cl, "SLTDV"  ) ) classSLTDV  = 1; // 13
    if( !strcmp( cl, "TLT"    ) ) classTLT    = 1; // 14
    if( !strcmp( cl, "FLTTRG" ) ) classFLTTRG = 1; // 15
    if( !strcmp( cl, "RNDTRG" ) ) classRNDTRG = 1; // 16
    if( !strcmp( cl, "RECYPS" ) ){
      classRECYPS = 1;                             // 17
      if( parameter != -999999 ) cutRECYPS = parameter;
    }
    if( !strcmp( cl, "RECDV"  ) ){
      classRECDV = 1;                              // 18
      if( parameter != -999999 ) cutRECDV = parameter;
    }
    if( !strcmp( cl, "CLASS19" ) ) classCLASS19 = 1; // 19
    if( !strcmp( cl, "CLASS20" ) ) classCLASS20 = 1; // 20
    if( !strcmp( cl, "CLASS21" ) ) classCLASS21 = 1; // 21
    if( !strcmp( cl, "CLASS22" ) ) classCLASS22 = 1; // 22
    if( !strcmp( cl, "CLASS23" ) ) classCLASS23 = 1; // 23
    if( !strcmp( cl, "CLASS24" ) ) classCLASS24 = 1; // 24
    if( !strcmp( cl, "CLASS25" ) ) classCLASS25 = 1; // 25
    if( !strcmp( cl, "CLASS26" ) ) classCLASS26 = 1; // 26
    if( !strcmp( cl, "CLASS27" ) ) classCLASS27 = 1; // 27
    if( !strcmp( cl, "CLASS28" ) ) classCLASS28 = 1; // 28
    if( !strcmp( cl, "CLASS29" ) ) classCLASS29 = 1; // 29
    if( !strcmp( cl, "CLASS30" ) ) classCLASS30 = 1; // 30
    if( !strcmp( cl, "CLASS31" ) ) classCLASS31 = 1; // 31
    if( !strcmp( cl, "CLASS32" ) ) classCLASS32 = 1; // 32
  }

  if( !strcmp( status, "OFF" ) ){
    if( !strcmp( cl, "ALL"    ) ){
      classJPSIEE = 0; //  1
      classJPSIMM = 0; //  2
      classFCNC   = 0; //  3
      classCAL1HT = 0; //  4
      classVZERO  = 0; //  5
      classVZERO2 = 0; //  6
      classDVTX   = 0; //  7
      classSLTYPS = 0; //  8
      classSLTCUT = 0; //  9
      classSLTEEB = 0; // 10
      classSLTEE  = 0; // 11
      classSLTMM  = 0; // 12
      classSLTDV  = 0; // 13
      classTLT    = 0; // 14
      classFLTTRG = 0; // 15
      classRNDTRG = 0; // 16
      classRECYPS = 0; // 17
      classRECDV  = 0; // 18
      classCLASS19 = 0; // 19
      classCLASS20 = 0; // 20
      classCLASS21 = 0; // 21
      classCLASS22 = 0; // 22
      classCLASS23 = 0; // 23
      classCLASS24 = 0; // 24
      classCLASS25 = 0; // 25
      classCLASS26 = 0; // 26
      classCLASS27 = 0; // 27
      classCLASS28 = 0; // 28
      classCLASS29 = 0; // 29
      classCLASS30 = 0; // 30
      classCLASS31 = 0; // 31
      classCLASS32 = 0; // 32
    }
    if( !strcmp( cl, "JPSIEE" ) ) classJPSIEE = 0; //  1
    if( !strcmp( cl, "JPSIMM" ) ) classJPSIMM = 0; //  2
    if( !strcmp( cl, "FCNC"   ) ) classFCNC   = 0; //  3
    if( !strcmp( cl, "CAL1HT" ) ) classCAL1HT = 0; //  4
    if( !strcmp( cl, "VZERO"  ) ) classVZERO  = 0; //  5
    if( !strcmp( cl, "VZERO2" ) ) classVZERO2 = 0; //  6
    if( !strcmp( cl, "DVTX"   ) ) classDVTX   = 0; //  7
    if( !strcmp( cl, "SLTYPS" ) ) classSLTYPS = 0; //  8
    if( !strcmp( cl, "SLTCUT" ) ) classSLTCUT = 0; //  9
    if( !strcmp( cl, "SLTEEB" ) ) classSLTEEB = 0; // 10
    if( !strcmp( cl, "SLTEE"  ) ) classSLTEE  = 0; // 11
    if( !strcmp( cl, "SLTMM"  ) ) classSLTMM  = 0; // 12
    if( !strcmp( cl, "SLTDV"  ) ) classSLTDV  = 0; // 13
    if( !strcmp( cl, "TLT"    ) ) classTLT    = 0; // 14
    if( !strcmp( cl, "FLTTRG" ) ) classFLTTRG = 0; // 15
    if( !strcmp( cl, "RNDTRG" ) ) classRNDTRG = 0; // 16
    if( !strcmp( cl, "RECYPS" ) ) classRECYPS = 0; // 17
    if( !strcmp( cl, "RECDV"  ) ) classRECDV  = 0; // 18  }
    if( !strcmp( cl, "CLASS19" ) ) classCLASS19 = 0; // 19
    if( !strcmp( cl, "CLASS20" ) ) classCLASS20 = 0; // 20
    if( !strcmp( cl, "CLASS21" ) ) classCLASS21 = 0; // 21
    if( !strcmp( cl, "CLASS22" ) ) classCLASS22 = 0; // 22
    if( !strcmp( cl, "CLASS23" ) ) classCLASS23 = 0; // 23
    if( !strcmp( cl, "CLASS24" ) ) classCLASS24 = 0; // 24
    if( !strcmp( cl, "CLASS25" ) ) classCLASS25 = 0; // 25
    if( !strcmp( cl, "CLASS26" ) ) classCLASS26 = 0; // 26
    if( !strcmp( cl, "CLASS27" ) ) classCLASS27 = 0; // 27
    if( !strcmp( cl, "CLASS28" ) ) classCLASS28 = 0; // 28
    if( !strcmp( cl, "CLASS29" ) ) classCLASS29 = 0; // 29
    if( !strcmp( cl, "CLASS30" ) ) classCLASS30 = 0; // 30
    if( !strcmp( cl, "CLASS31" ) ) classCLASS31 = 0; // 31
    if( !strcmp( cl, "CLASS32" ) ) classCLASS32 = 0; // 32
  }

  // set general flag
  classify |= classJPSIEE;
  classify |= classJPSIMM;
  classify |= classFCNC;
  classify |= classCAL1HT;
  classify |= classVZERO;
  classify |= classVZERO2;
  classify |= classDVTX;
  classify |= classSLTYPS;
  classify |= classSLTCUT;
  classify |= classSLTEEB;
  classify |= classSLTEE;
  classify |= classSLTMM;
  classify |= classSLTDV;
  classify |= classTLT;
  classify |= classFLTTRG;
  classify |= classRNDTRG;
  classify |= classRNDTRG;
  classify |= classRNDTRG;
  classify |= classCLASS19;
  classify |= classCLASS20;
  classify |= classCLASS21;
  classify |= classCLASS22;
  classify |= classCLASS23;
  classify |= classCLASS24;
  classify |= classCLASS25;
  classify |= classCLASS26;
  classify |= classCLASS27;
  classify |= classCLASS28;
  classify |= classCLASS29;
  classify |= classCLASS30;
  classify |= classCLASS31;
  classify |= classCLASS32;

  //
  // print
  //
  cout << "RECON/CLASS: " << cl << " " << status << endl;
  if( FADEB ){
    cout << "RECON/CLASS: general=" << classify    << endl; // general
    cout << "             JPSIEE= " << classJPSIEE << endl; //  1
    cout << "             JPSIMM= " << classJPSIMM << endl; //  2
    cout << "             FCNC=   " << classFCNC   << endl; //  3
    cout << "             CAL1HT= " << classCAL1HT << endl; //  4
    cout << "             VZERO=  " << classVZERO  << endl; //  5
    cout << "             VZERO2= " << classVZERO2 << endl; //  6
    cout << "             DVTX=   " << classDVTX   << endl; //  7
    cout << "             SLTYPS= " << classSLTYPS << endl; //  8
    cout << "             SLTCUT= " << classSLTCUT << endl; //  9
    cout << "             SLTEEB  " << classSLTEEB << endl; // 10
    cout << "             SLTEE=  " << classSLTEE  << endl; // 11
    cout << "             SLTMM=  " << classSLTMM  << endl; // 12
    cout << "             SLTDV=  " << classSLTDV  << endl; // 13
    cout << "             TLT=    " << classTLT    << endl; // 14
    cout << "             FLTTRG= " << classFLTTRG << endl; // 15
    cout << "             RNDTRG= " << classRNDTRG << endl; // 16
    cout << "             RECYPS= " << classRNDTRG << endl; // 17
    cout << "             RECDV=  " << classRNDTRG << endl; // 18
    //cout << "             CLASS19= " << classCLASS19 << endl; // 19
    //cout << "             CLASS20= " << classCLASS20 << endl; // 20
    //cout << "             CLASS21= " << classCLASS21 << endl; // 21
    //cout << "             CLASS22= " << classCLASS22 << endl; // 22
    //cout << "             CLASS23= " << classCLASS23 << endl; // 23
    //cout << "             CLASS24= " << classCLASS24 << endl; // 24
    //cout << "             CLASS25= " << classCLASS25 << endl; // 25
    //cout << "             CLASS26= " << classCLASS26 << endl; // 26
    //cout << "             CLASS27= " << classCLASS27 << endl; // 27
    //cout << "             CLASS28= " << classCLASS28 << endl; // 28
    //cout << "             CLASS29= " << classCLASS29 << endl; // 29
    //cout << "             CLASS30= " << classCLASS30 << endl; // 30
    //cout << "             CLASS31= " << classCLASS31 << endl; // 31
    //cout << "             CLASS32= " << classCLASS32 << endl; // 32
  }

  ARTE_mark( "kaclass_ 1", 0 );
}
