// ********************************************************************
//
//  Routine:      rc99Bremsstrahlung
//  Author:       T. Oest
//
//  Description:  this routine returns the Bremsstrahlung
//                for a given track
//
//  Modification:
//                radius for Bremsstrahlung search fixed to 4 cm
//               (before old momentum dependent approximation)
//                                                          13/11/00 D. Samtleben
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"

//
// function
//
float rc99Bremsstrahlung(ArtePointer<RTRA> rtra) {
//-----------------------------------------------------------------
//   select two body decays
//-----------------------------------------------------------------

    float energy= 0.0;
    float minImpact = 100000.;

    float xECAL, yECAL, zECAL;
    float xTrack, yTrack, zTrack, txTrack, tyTrack;
    float yimpact, ximpact, rimpact;

    ArteRelationIterator<RCCL> rcclTrack  = rtra->rccl.begin();

    for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
         rccl != ArteTable<RCCL>::end(); ++rccl ) {

     if ((*rccl) != rcclTrack){

      xECAL    = (*rccl)->x;
      yECAL    = (*rccl)->y;
      zECAL    = (*rccl)->z;
      xTrack   = rtra->xf;
      yTrack   = rtra->yf;
      zTrack   = rtra->zf;
      txTrack  = rtra->txf;
      tyTrack  = rtra->tyf;

      yimpact  = yTrack + tyTrack * (zECAL - zTrack) - yECAL;
      ximpact  = xTrack + txTrack * (zECAL - zTrack) - xECAL;
      rimpact  = sqrt (ximpact*ximpact + yimpact*yimpact);
      if (rimpact < minImpact) {
       minImpact = rimpact;
       energy    = (*rccl)->esum;
      };

     };
    }; 

// float ycut = (1.0 + 3. * (1. - energy / 100.))/sqrt(2);
 float ycut=4.;
 if (minImpact > ycut) energy = 0.0;

 return energy;

};



