
// *****************************************************************************
//
// source:    rcRANGER.C
//
// type:      source code
//
// created:   13 Nov 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Department of Physics
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcRANGER
//
// called by: rc99AStandalone()
//
// purpose:   handle tracking routines
//
// hints:     timing is done outside
//
// changes:
// 13 Nov 2000 (AG) creation
// 13 Nov 2000 (UU) change default OTr hit preparation to OtrI
// 14 Nov 2000 (AG) final polishing and approval
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>

// ARTE includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "silarte/silarte.hh"
#include "cats/cats.h"
#include "rang/rgfit.hh"
#include "itrarte/ITR_arte.h"
#include "farm/farm.hh"
#include "reco/rc99Areco.hh"
#include "Otr/OtrEvent.h"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"

// globals
int maxhits     = 500000;
int hitprepflag = -1;

// prototypes
void waxevnt();
extern "C" void tfevntr_();

//
// function code
//
int
rcRANGER( int recoITR, int recoOTR )
{
  ARTE_mark( "rcRANGER 0", 0 );

  static Chrono *cron_rc99AITR = 0;
  static Chrono *cron_rcOTRHIT = 0;
  static Chrono *cron_rc99AOTR = 0;

  static bool first = 1;
  if( first ){
    first = 0;
    ChronoManager CM;
    CM.instance();
    
    cron_rc99AITR   = CM.createChrono( "rc99AITR"   );
    cron_rcOTRHIT   = CM.createChrono( "rcOTRHIT"   );
    cron_rc99AOTR   = CM.createChrono( "rc99AOTR"   );
  }

  float time_rc99AITR = 0;
  float time_rcOTRHIT = 0;
  float time_rc99AOTR = 0;

  //==================================================================
  // 1. ITR create hits for data
  //==================================================================
  
  int ITRflag = 0;
  //    if( ArteTable<MTRA>::begin() == ArteTable<MTRA>::end() ){
  if( !ArteTable<MCIN>::booked() ){
    if( recoITR == 1 || recoITR == 3 ){
      ARTE_mark( "ItrEvnt", 0 );
      cron_rc99AITR->start();
      //
      ITRflag = ItrEvnt();
      //
      cron_rc99AITR->stop();
      time_rc99AITR = cron_rc99AITR->lastTime();
    }
  }
 
  //==================================================================
  // 2. use steering variables if set.
  //==================================================================
  if( hitprepflag == -1 ){
    char *f = 0;
    
    /* hit preparation for OTR  */
    if( ( f = getenv( "OtrHitPreparation") ) ) hitprepflag = atoi(f);
    else hitprepflag = 0;
    
    /* not supported anymore, just for backward compability */    
    if( ( f = getenv( "WAX_OCCUPCUT" ) ) ) maxhits = atoi(f);
    
    /* use this one instead */    
    if( ( f = getenv( "OtrOccupancyCut" ) ) ) maxhits = atoi(f);
    
    if( maxhits<500000 ){    
      cout << "rc99AOTR[warning]: Occupancy cut!!! Number of hits lower "
	   <<  maxhits << endl;
    }  
  }
  
  //==================================================================
  // 3. create hits:  fill HITB table with OTR hits
  //==================================================================
  int OTRflag = 0;
  if( !ArteTable<MCIN>::booked() ){
    if( recoOTR == 1 || recoOTR == 3 ){

      cron_rcOTRHIT->start();

      int notrhits = ArteTable<HITB>::size();
      switch( hitprepflag ){
      case 0:
	ARTE_mark( "OtrEventUpdate", 0 );
	OtrEventUpdate();
	break;
      case 1:
	ARTE_mark( "waxevnt", 0 );
	waxevnt();
	break;
      case 2:
	ARTE_mark( "waxevnt", 0 );
	waxevnt();
	ARTE_mark( "OtrEventUpdate", 0 );
	OtrEventUpdate();
	break;
      }
      notrhits = ArteTable<HITB>::size() - notrhits;
      if( notrhits > maxhits ) OTRflag = -1;

      cron_rcOTRHIT->stop();
      time_rcOTRHIT = cron_rcOTRHIT->lastTime();
    }
  }
  //==================================================================
  // 4. ranger tracking
  //==================================================================

  if( (( recoOTR > 1 ) && !OTRflag) || (( recoITR > 1 ) && !ITRflag) ){
    ARTE_mark( "tfevntr_", 0 );
    cron_rc99AOTR->start();
    //
    tfevntr_();
    //
    cron_rc99AOTR->stop();
    time_rc99AOTR = cron_rc99AOTR->lastTime();
  }
  
  // print
  cout << " RANG: "
       << "itr=" << time_rc99AITR << " "
       << "hit=" << time_rcOTRHIT << " "
       << "otr=" << time_rc99AOTR << " "
       << endl;
    
  ARTE_mark( "rcRANGER 1", 0 );
  return 0;
}
