#ifndef __LENSSYSTEM_H__
#define __LENSSYSTEM_H__


// -------------------------------------------------------------------------
/// description of  the lense system

#ifdef NOARTE
#include "auto/RIC.hh"
#endif

class LensSystem {
Table AngEff,WaveEff;
int id;
public:
  LensSystem(ScriptFile& f);
  LensSystem(ArtePointer<CLEN> a);
  void print(FILE* fp);

  inline double MinLambda() const { return WaveEff.x(0); }
  inline double MaxLambda() const { return WaveEff.x(WaveEff.numpoints()-1); }
  inline double AngularEfficiency   (double a)  { return AngEff.f(a); }
  inline double WaveLengthEfficiency(double l)  { return WaveEff.f(l); }
  inline Table& WaveLengthEfficiency()          { return WaveEff; }
  inline int type() const { return id; }
/// trace/transform photon through lense system
  inline int trace(const Photon& p,double *x,double *y)  {
    *x=-p.x().x()/2;
    *y=-p.x().y()/2;
    int ret=NO_LOSS;
    if (rnd_normal()>WaveLengthEfficiency(p.lambda())) {
      ret=LEN_LOSS_WAVELENGTH;
      }
    else if (rnd_normal()>AngularEfficiency(p.n().z())) {
      ret=LEN_LOSS_ANGULAR;
      }
    return ret;
    }

/// inverse tracing/transformation through the lense system
  inline Vec3 inversetrace(double x,double y,double costhe=-1E37,double phi=-1E37) const {
    if (phi>=0) {
	costhe+=0;
	phi+=0;
      //printf("dummy print to keep compiler quiet %f %f\n",costhe,phi);
      }	
    return Vec3(-2*x,-2*y,0);
    }
  
  };

#endif
