//--*-C++-*--
#ifndef _RICHDB_H_
#define _RICHDB_H_


#include "rich/ricutil.h"
#include "rich/PhotonDetection.h"
#include "rich/PhiLambda.h"

extern "C" {
#include "event.h"
#include "etb.h"
#include "emg.h"
#include "component.h"
#include "grr.h"
#include "libr.h"
#include "log.h"
#include "table.h"
}


class DAQ_INITIALIZATION {
 private:
  Tables tid;
  ScriptFile* file;
  char *PMType[3], *ASDConn[4], *ASD[4], *way;
  int *SuperMod, *PMBRow, *PMBCol, *MotherB, *DaughterB, *DBConn;
  int *PMT, *Pad, *Channel;
  int *Chan_ID;
  short *Chan_Stat;
  int fsize;
 public:
  
  /// Constructor opening the database.  
  DAQ_INITIALIZATION(char *sours=0);

  /// Destructor closing the database's tables or data files.
  ~DAQ_INITIALIZATION();

  
  /// returns the fields size of booked tables
  inline int FS(){return fsize;}
  
  /// Takes the data from ASD table.
  int ASDInitialization();
  
  /// Takes the data from M16WireMap table
  int M16WInitialization();
  
  /// Takes the data from M4WireMap table
  int M4WInitialization();
  
  /// Set the status of all channels equal 1.
  int DefaultStatus();
  
  /// Takes the data from ChannelStatus table.
  int ChannelStatus();
  
  /// Takes the status of the channels from data file.
  int StatusfromFile();
  
  /// if 'sours' is the name of file, reading information from file,
  /// if no argument 'sours', reading information from database.
  int sours_selector (char* subject=0);

  /// Printing out the list of bad channels tables. Each table relates 
  /// with run number and consists from list of bad channels for this run
  int BadTablesList();
  
  /// Reterns the information about one  ASD8 board
  int asd_scan(char* ftag,char* fasd, int* fsm, int* fr,\
	       int *fc, int *ffedid, int *fd, int *fcon);
  /// Reterns the string with the mapping of one channel 
  int m16_scan(char* s);
  
  /// Reterns the string with the mapping of one channel 
  int m4_scan(char* s);
  
  /// Reterns the information about status of one chanel
  int channelstatus_scan(int* id, short* stat);
  
  /// Fills database with chanel status
  static void channelstatus_fill(int *id, short int *stat, int nchan, int run_no);

  /// Keytable assignment flag, (default distribute=1)
  static int distribute;

  static int RichGeoServer;  
  static int RichASDtable ; 
  static int RichCDIMtable; 
  static int RichCEFFtable; 
  static int RichCLENtable; 
  static int RichCPDWtable; 
  static int RichCPMDtable; 
  static int RichCPMItable; 
  static int RichCPMTtable; 
  static int RichCPOItable; 
  static int RichCSMItable; 
  static int RichCSPMtable; 
  static int RichCSSMtable; 
  static int RichCSUPtable; 
  static int RichCVEStable; 
  static int RichChannelStatusTable; 
  static int RichM16WireMapTable; 
  static int RichM4WireMapTable; 

  /// Get information on key table servers
  static int RichKeyInit();
  static int RichKeyTableInit(int &RichTable, char* TableName);
  /// Prepare table for reading
  static int PrepareTable(Tables &table, char *tname, int RichTable);

  /// Print used major/minor
  static void PrintTable(int RichTable,char *TableName);
  /// Print used major/minor for all tables
  static void PrintUsedTables();

};

extern DAQ_INITIALIZATION *init;

#endif



