//*****************************************************************************
//
// source:    matchvds.C
//
// type:      source code
//
// created:   14 Nov 2000
//
// author:    Brian Petersen
//            E-mail: aagaard@nbi.dk
//
// symbol:    MatchSltRTRA
//
// called by:
//
// purpose:   Match SLT track with VDS segment
//
// hints:
//
// changes:
// 14 Nov 2000 (BP) Creation
//
//*****************************************************************************
#include <iostream.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "sltarte/matchslt.hh"

extern "C" {
#include "sltinfo.h"
}
#define POW(x) ((x)*(x))

 /* trk is trackindex from SLT_TRIGGER */

void MatchSltVDS(int trk,ArtePointer<RSEG> &track,float &xchi,float &ychi){

  SLT_SILITRACK sili;
  SLT_SILITRACKCOV cov;

  track=0;
  xchi=ychi=999999.;

  if (SLTSiliTrackInfo(trk,&sili)!=SLT_OK) {
    cout<<"Error getting L2sili track for track";
    cout<<trk<<endl;
    return;
  }
  if (SLTSiliTrackCovInfo(trk,&cov)!=SLT_OK) {
    cout<<"Error getting covariance matrix for track";
    cout<<trk<<endl;
    return;
  }
  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {
    if ((*rseg)->fit <= 0 ||  (*rseg)->fit == 8 ||
	(*rseg)->fit == 10){

      if ((*rseg)->cmp & Rsegc::vxd ) {
	double cix,ciy;
	float z=(*rseg)->zf;
	float x=sili.bx+sili.tx*z;
	float y=sili.by+sili.ty*z;
	
	float etx=cov.xcov[0];
	float ety=cov.ycov[0];
	float ex=cov.xcov[0]*z*z-2*z*cov.xcov[1]+cov.xcov[2];
	float ey=cov.ycov[0]*z*z-2*z*cov.ycov[1]+cov.ycov[2];
	
	cix  = POW(sili.tx-(*rseg)->txf)/(etx+(*rseg)->cvf[5]);
	cix += POW(x-(*rseg)->xf)/(ex+(*rseg)->cvf[0]);
	ciy  = POW(sili.ty-(*rseg)->tyf)/(ety+(*rseg)->cvf[9]);
	ciy += POW(y-(*rseg)->yf)/(ey+(*rseg)->cvf[2]);
	
	if (cix+ciy<xchi+ychi) {
	  xchi=cix;
	  ychi=ciy;
	  track=*rseg;
	}
      }
    }
  }
}
