// *****************************************************************************
//
// source:    L3_Target.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************

#include "L3_Target.hh"
#include <stdio.h>

L3_Target& L3_Target::operator=(const L3_Target &p)
{
   t_cmp=p.t_cmp;
   t_wnr=p.t_wnr;
   t_dx=p.t_dx;
   t_dy=p.t_dy;
   t_dz=p.t_dz;
   center = p.center;
   A = p.A;
   Ainverse = p.Ainverse;
   t_ms=p.t_ms;
   t_x0bm=p.t_x0bm;
   t_y0bm=p.t_y0bm;
   t_sxbm=p.t_sxbm;
   t_sybm=p.t_sybm;
   strcpy(t_name,p.t_name);
   return *this;
}

#ifdef L3_ARTE_FRAME
L3_Target::L3_Target(const ArtePointer<GTAR> p) : center(3),A(3,3),Ainverse(3,3)
{
   #define NN "L3_Target::L3_Target: "
   t_cmp=p->cmp;
   t_wnr=p->wnr;
   t_dx=p->dx;
   t_dy=p->dy;
   t_dz=p->dz;
   center[0] = p->x;
   center[1] = p->y;
   center[2] = p->z;
   for(int i=0;i<3;i++) for(int k=0;k<3;k++) A[i][k] = p->rot[k+3*i];
   int status = 42;
   Ainverse = A.inverse(status);
   if( status != 0 ) P2(NN"Matrix inversion returns error code",status);
   t_ms=p->ms;
   t_x0bm=p->x0bm;
   t_y0bm=p->y0bm;
   t_sxbm=p->sxbm;
   t_sybm=p->sybm;
#ifdef L3_ARTE2
   t_name[0]=0;
#else
   strncpy(t_name,p->name,4);
   t_name[4]=0;
#endif
   if( !isalnum(t_name[0]) ) strcpy(t_name,get_name());
   #undef NN
}
#endif

#ifdef L3_SLP_FRAME
L3_Target::L3_Target(const L2TARG &p) : center(3),A(3,3),Ainverse(3,3)
{
   #define NN "L3_Target::L3_Target: "
   t_cmp=p.cmp;
   t_wnr=p.wnr;
   t_dx=p.dx;
   t_dy=p.dy;
   t_dz=p.dz;
   center[0] = p.x;
   center[1] = p.y;
   center[2] = p.z;
   for(int i=0;i<3;i++) for(int k=0;k<3;k++) A[i][k] = p.rot[i][k];
   int status = 42;
   Ainverse = A.inverse(status);
   if( status != 0 ) P2(NN"Matrix inversion returns error code",status);
   t_ms=p.ms;
   t_x0bm=p.x0bm;
   t_y0bm=p.y0bm;
   t_sxbm=p.sxbm;
   t_sybm=p.sybm;
   strcpy(t_name,get_name());
   #undef NN
}
#endif

char* L3_Target::get_name()
{
  static char *wirename[] = {
    "1TOP","1BOT","1INN","1OUT",
    "2TOP","2BOT","2INN","2OUT",
  };
  int index = 0;
  if( t_dx > t_dy ){   //horizontal wire
    if( y() > 0 )
      index = 0; //above beam
    else
      index = 1; //below beam
  } else {  //vertical wire
    if( x() > 0 )
      index = 2; //+x, inner
    else 
      index = 3; //-x , outer
  }
  if( z() < -2.9 ) index += 4; //station II
  return wirename[index];
}

ostream& operator<<(ostream& c,const L3_Target &p )
{
  char buf[120];
  //header
  c << "**** Target wire number " << p.t_wnr << " - cmp=" << p.t_cmp << ' ' << p.t_name << " ***" << endl; 
  //wire positions
  sprintf(buf,"\tx=%.3f\ty=%.3f\tz=%.3f\n",p.center[0],p.center[1],p.center[2]); c << buf;
  sprintf(buf,"\tdx=%.3f\tdy=%.3f\tdz=%.3f\n",p.t_dx,p.t_dy,p.t_dz); c << buf;
  //rotation maxtrix
  sprintf(buf,"\t      ( %.2f, %.2f, %.2f )\n",p.A[0][0],p.A[1][0],p.A[2][0]);  c << buf;
  sprintf(buf,"\trot = ( %.2f, %.2f, %.2f )\n",p.A[0][1],p.A[1][1],p.A[2][1]);  c << buf;
  sprintf(buf,"\t      ( %.2f, %.2f, %.2f )\n",p.A[0][2],p.A[1][2],p.A[2][2]);  c << buf;
  //misc
  sprintf(buf,"\tms=%.2f\n",p.t_ms); c << buf;
  sprintf(buf,"\tx0bm=%.2f, y0bm=%.2f\n",p.t_x0bm,p.t_y0bm); c << buf;
  sprintf(buf,"\tsxbm=%.2f, sxbm=%.2f\n",p.t_sxbm,p.t_sxbm); c << buf;
  return c;
}
